@@ -2,7 +2,9 @@
 #include "libslic3r/Utils.hpp"
 #include "AppConfig.hpp"
 #include "Exception.hpp"
+#include "LocalesUtils.hpp"
 #include "Thread.hpp"
+#include "format.hpp"
 
 #include <utility>
 #include <vector>
@@ -16,15 +18,24 @@
 #include <boost/property_tree/ptree_fwd.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/format/format_fwd.hpp>
+#include <boost/log/trivial.hpp>
 
-//#include <wx/string.h>
-//#include "I18N.hpp"
+#ifdef WIN32
+//FIXME replace the two following includes with <boost/md5.hpp> after it becomes mainstream.
+#include <boost/uuid/detail/md5.hpp>
+#include <boost/algorithm/hex.hpp>
+#endif
 
 namespace Slic3r {
 
 static const std::string VENDOR_PREFIX = "vendor:";
 static const std::string MODEL_PREFIX = "model:";
-static const std::string VERSION_CHECK_URL = "https://files.prusa3d.com/wp-content/uploads/repository/PrusaSlicer-settings-master/live/PrusaSlicer.version";
+// Because of a crash in PrusaSlicer 2.3.0/2.3.1 when showing an update notification with some locales, we don't want PrusaSlicer 2.3.0/2.3.1
+// to show this notification. On the other hand, we would like PrusaSlicer 2.3.2 to show an update notification of the upcoming PrusaSlicer 2.4.0.
+// Thus we will let PrusaSlicer 2.3.2 and couple of follow-up versions to download the version number from an alternate file until the PrusaSlicer 2.3.0/2.3.1
+// are phased out, then we will revert to the original name.
+//static const std::string VERSION_CHECK_URL = "https://files.prusa3d.com/wp-content/uploads/repository/PrusaSlicer-settings-master/live/PrusaSlicer.version";
+static const std::string VERSION_CHECK_URL = "https://files.prusa3d.com/wp-content/uploads/repository/PrusaSlicer-settings-master/live/PrusaSlicer.version2";
 
 const std::string AppConfig::SECTION_FILAMENTS = "filaments";
 const std::string AppConfig::SECTION_MATERIALS = "sla_materials";
@@ -32,6 +43,10 @@ const std::string AppConfig::SECTION_MATERIALS = "sla_materials";
 void AppConfig::reset()
 {
     m_storage.clear();
+    m_vendors.clear();
+    m_dirty = false;
+    m_orig_version = Semver::invalid();
+    m_legacy_datadir = false;
     set_defaults();
 };
 
@@ -60,22 +75,21 @@ void AppConfig::set_defaults()
         if (get("drop_project_action").empty())
             set("drop_project_action", "1");
 
-        if (get("version_check").empty())
-            set("version_check", "1");
         if (get("preset_update").empty())
             set("preset_update", "1");
 
         if (get("export_sources_full_pathnames").empty())
             set("export_sources_full_pathnames", "0");
 
-#if ENABLE_CUSTOMIZABLE_FILES_ASSOCIATION_ON_WIN
 #ifdef _WIN32
         if (get("associate_3mf").empty())
             set("associate_3mf", "0");
         if (get("associate_stl").empty())
             set("associate_stl", "0");
+
+        if (get("tabs_as_menu").empty())
+            set("tabs_as_menu", "0");
 #endif // _WIN32
-#endif // ENABLE_CUSTOMIZABLE_FILES_ASSOCIATION_ON_WIN
 
         // remove old 'use_legacy_opengl' parameter from this config, if present
         if (!get("use_legacy_opengl").empty())
@@ -109,6 +123,9 @@ void AppConfig::set_defaults()
 
         if (get("auto_toolbar_size").empty())
             set("auto_toolbar_size", "100");
+ 
+       if (get("notify_release").empty())
+           set("notify_release", "all"); // or "none" or "release"
 
 #if ENABLE_ENVIRONMENT_MAP
         if (get("use_environment_map").empty())
@@ -123,15 +140,25 @@ void AppConfig::set_defaults()
 
         if (get("default_action_on_select_preset").empty())
             set("default_action_on_select_preset", "none");     // , "transfer", "discard" or "save" 
+
+        if (get("default_action_on_new_project").empty())
+            set("default_action_on_new_project", "none");       // , "keep(transfer)", "discard" or "save" 
+
+        if (get("color_mapinulation_panel").empty())
+            set("color_mapinulation_panel", "0");
+
+        if (get("order_volumes").empty())
+            set("order_volumes", "1");
+
+        if (get("clear_undo_redo_stack_on_new_project").empty())
+            set("clear_undo_redo_stack_on_new_project", "1");
     }
-#if ENABLE_CUSTOMIZABLE_FILES_ASSOCIATION_ON_WIN
     else {
 #ifdef _WIN32
         if (get("associate_gcode").empty())
             set("associate_gcode", "0");
 #endif // _WIN32
     }
-#endif // ENABLE_CUSTOMIZABLE_FILES_ASSOCIATION_ON_WIN
 
     if (get("seq_top_layer_only").empty())
         set("seq_top_layer_only", "1");
@@ -148,12 +175,22 @@ void AppConfig::set_defaults()
     if (get("show_splash_screen").empty())
         set("show_splash_screen", "1");
 
-#if ENABLE_CTRL_M_ON_WINDOWS
+    if (get("show_hints").empty())
+        set("show_hints", "1");
+
+    if (get("allow_ip_resolve").empty())
+        set("allow_ip_resolve", "1");
+
 #ifdef _WIN32
     if (get("use_legacy_3DConnexion").empty())
         set("use_legacy_3DConnexion", "0");
+
+    if (get("dark_color_mode").empty())
+        set("dark_color_mode", "0");
+
+    if (get("sys_menu_enabled").empty())
+        set("sys_menu_enabled", "1");
 #endif // _WIN32
-#endif // ENABLE_CTRL_M_ON_WINDOWS
 
     // Remove legacy window positions/sizes
     erase("", "main_frame_maximized");
@@ -164,25 +201,116 @@ void AppConfig::set_defaults()
     erase("", "object_settings_size");
 }
 
-std::string AppConfig::load()
+#ifdef WIN32
+static std::string appconfig_md5_hash_line(const std::string_view data)
 {
+    //FIXME replace the two following includes with <boost/md5.hpp> after it becomes mainstream.
+    // return boost::md5(data).hex_str_value();
+    // boost::uuids::detail::md5 is an internal namespace thus it may change in the future.
+    // Also this implementation is not the fastest, it was designed for short blocks of text.
+    using boost::uuids::detail::md5;
+    md5              md5_hash;
+    // unsigned int[4], 128 bits
+    md5::digest_type md5_digest{};
+    std::string      md5_digest_str;
+    md5_hash.process_bytes(data.data(), data.size());
+    md5_hash.get_digest(md5_digest);
+    boost::algorithm::hex(md5_digest, md5_digest + std::size(md5_digest), std::back_inserter(md5_digest_str));
+    // MD5 hash is 32 HEX digits long.
+    assert(md5_digest_str.size() == 32);
+    // This line will be emited at the end of the file.
+    return "# MD5 checksum " + md5_digest_str + "\n";
+};
+
+// Assume that the last line with the comment inside the config file contains a checksum and that the user didn't modify the config file.
+static bool verify_config_file_checksum(boost::nowide::ifstream &ifs)
+{
+    auto read_whole_config_file = [&ifs]() -> std::string {
+        std::stringstream ss;
+        ss << ifs.rdbuf();
+        return ss.str();
+    };
+
+    ifs.seekg(0, boost::nowide::ifstream::beg);
+    std::string whole_config = read_whole_config_file();
+
+    // The checksum should be on the last line in the config file.
+    if (size_t last_comment_pos = whole_config.find_last_of('#'); last_comment_pos != std::string::npos) {
+        // Split read config into two parts, one with checksum, and the second part is part with configuration from the checksum was computed.
+        // Verify existence and validity of the MD5 checksum line at the end of the file.
+        // When the checksum isn't found, the checksum was not saved correctly, it was removed or it is an older config file without the checksum.
+        // If the checksum is incorrect, then the file was either not saved correctly or modified.
+        if (std::string_view(whole_config.c_str() + last_comment_pos, whole_config.size() - last_comment_pos) == appconfig_md5_hash_line({ whole_config.data(), last_comment_pos }))
+            return true;
+    }
+    return false;
+}
+#endif
+
+std::string AppConfig::load(const std::string &path)
+{
+    this->reset();
+
     // 1) Read the complete config file into a boost::property_tree.
     namespace pt = boost::property_tree;
     pt::ptree tree;
-    boost::nowide::ifstream ifs(AppConfig::config_path());
+    boost::nowide::ifstream ifs;
+    bool                    recovered = false;
+
     try {
+        ifs.open(path);
+#ifdef WIN32
+        // Verify the checksum of the config file without taking just for debugging purpose.
+        if (!verify_config_file_checksum(ifs))
+            BOOST_LOG_TRIVIAL(info) << "The configuration file " << path <<
+            " has a wrong MD5 checksum or the checksum is missing. This may indicate a file corruption or a harmless user edit.";
+
+        ifs.seekg(0, boost::nowide::ifstream::beg);
+#endif
         pt::read_ini(ifs, tree);
     } catch (pt::ptree_error& ex) {
-        // Error while parsing config file. We'll customize the error message and rethrow to be displayed.
-        // ! But to avoid the use of _utf8 (related to use of wxWidgets) 
-        // we will rethrow this exception from the place of load() call, if returned value wouldn't be empty
-        /*
-        throw Slic3r::RuntimeError(
-        	_utf8(L("Error parsing PrusaSlicer config file, it is probably corrupted. "
-                    "Try to manually delete the file to recover from the error. Your user profiles will not be affected.")) + 
-        	"\n\n" + AppConfig::config_path() + "\n\n" + ex.what());
-        */
-        return ex.what();
+#ifdef WIN32
+        // The configuration file is corrupted, try replacing it with the backup configuration.
+        ifs.close();
+        std::string backup_path = (boost::format("%1%.bak") % path).str();
+        if (boost::filesystem::exists(backup_path)) {
+            // Compute checksum of the configuration backup file and try to load configuration from it when the checksum is correct.
+            boost::nowide::ifstream backup_ifs(backup_path);
+            if (!verify_config_file_checksum(backup_ifs)) {
+                BOOST_LOG_TRIVIAL(error) << format("Both \"%1%\" and \"%2%\" are corrupted. It isn't possible to restore configuration from the backup.", path, backup_path);
+                backup_ifs.close();
+                boost::filesystem::remove(backup_path);
+            } else if (std::string error_message; copy_file(backup_path, path, error_message, false) != SUCCESS) {
+                BOOST_LOG_TRIVIAL(error) << format("Configuration file \"%1%\" is corrupted. Failed to restore from backup \"%2%\": %3%", path, backup_path, error_message);
+                backup_ifs.close();
+                boost::filesystem::remove(backup_path);
+            } else {
+                BOOST_LOG_TRIVIAL(info) << format("Configuration file \"%1%\" was corrupted. It has been succesfully restored from the backup \"%2%\".", path, backup_path);
+                // Try parse configuration file after restore from backup.
+                try {
+                    ifs.open(path);
+                    pt::read_ini(ifs, tree);
+                    recovered = true;
+                } catch (pt::ptree_error& ex) {
+                    BOOST_LOG_TRIVIAL(info) << format("Failed to parse configuration file \"%1%\" after it has been restored from backup: %2%", path, ex.what());
+                }
+            }
+        } else
+#endif // WIN32
+            BOOST_LOG_TRIVIAL(info) << format("Failed to parse configuration file \"%1%\": %2%", path, ex.what());
+        if (! recovered) {
+            // Report the initial error of parsing PrusaSlicer.ini.
+            // Error while parsing config file. We'll customize the error message and rethrow to be displayed.
+            // ! But to avoid the use of _utf8 (related to use of wxWidgets) 
+            // we will rethrow this exception from the place of load() call, if returned value wouldn't be empty
+            /*
+            throw Slic3r::RuntimeError(
+                _utf8(L("Error parsing PrusaSlicer config file, it is probably corrupted. "
+                        "Try to manually delete the file to recover from the error. Your user profiles will not be affected.")) +
+                "\n\n" + AppConfig::config_path() + "\n\n" + ex.what());
+            */
+            return ex.what();
+        }
     }
 
     // 2) Parse the property_tree, extract the sections and key / value pairs.
@@ -245,6 +373,11 @@ std::string AppConfig::load()
     return "";
 }
 
+std::string AppConfig::load()
+{
+    return this->load(AppConfig::config_path());
+}
+
 void AppConfig::save()
 {
     {
@@ -259,22 +392,21 @@ void AppConfig::save()
     const auto path = config_path();
     std::string path_pid = (boost::format("%1%.%2%") % path % get_current_pid()).str();
 
-    boost::nowide::ofstream c;
-    c.open(path_pid, std::ios::out | std::ios::trunc);
+    std::stringstream config_ss;
     if (m_mode == EAppMode::Editor)
-        c << "# " << Slic3r::header_slic3r_generated() << std::endl;
+        config_ss << "# " << Slic3r::header_slic3r_generated() << std::endl;
     else
-        c << "# " << Slic3r::header_gcodeviewer_generated() << std::endl;
+        config_ss << "# " << Slic3r::header_gcodeviewer_generated() << std::endl;
     // Make sure the "no" category is written first.
-    for (const std::pair<std::string, std::string> &kvp : m_storage[""])
-        c << kvp.first << " = " << kvp.second << std::endl;
+    for (const auto& kvp : m_storage[""])
+        config_ss << kvp.first << " = " << kvp.second << std::endl;
     // Write the other categories.
-    for (const auto category : m_storage) {
+    for (const auto& category : m_storage) {
     	if (category.first.empty())
     		continue;
-    	c << std::endl << "[" << category.first << "]" << std::endl;
-    	for (const std::pair<std::string, std::string> &kvp : category.second)
-	        c << kvp.first << " = " << kvp.second << std::endl;
+        config_ss << std::endl << "[" << category.first << "]" << std::endl;
+        for (const auto& kvp : category.second)
+            config_ss << kvp.first << " = " << kvp.second << std::endl;
 	}
     // Write vendor sections
     for (const auto &vendor : m_vendors) {
@@ -282,17 +414,42 @@ void AppConfig::save()
         for (const auto &model : vendor.second) { size_sum += model.second.size(); }
         if (size_sum == 0) { continue; }
 
-        c << std::endl << "[" << VENDOR_PREFIX << vendor.first << "]" << std::endl;
+        config_ss << std::endl << "[" << VENDOR_PREFIX << vendor.first << "]" << std::endl;
 
         for (const auto &model : vendor.second) {
-            if (model.second.size() == 0) { continue; }
+            if (model.second.empty()) { continue; }
             const std::vector<std::string> variants(model.second.begin(), model.second.end());
             const auto escaped = escape_strings_cstyle(variants);
-            c << MODEL_PREFIX << model.first << " = " << escaped << std::endl;
+            config_ss << MODEL_PREFIX << model.first << " = " << escaped << std::endl;
         }
     }
-    c.close();
+    // One empty line before the MD5 sum.
+    config_ss << std::endl;
 
+    std::string config_str = config_ss.str();
+    boost::nowide::ofstream c;
+    c.open(path_pid, std::ios::out | std::ios::trunc);
+    c << config_str;
+#ifdef WIN32
+    // WIN32 specific: The final "rename_file()" call is not safe in case of an application crash, there is no atomic "rename file" API
+    // provided by Windows (sic!). Therefore we save a MD5 checksum to be able to verify file corruption. In addition,
+    // we save the config file into a backup first before moving it to the final destination.
+    c << appconfig_md5_hash_line(config_str);
+#endif
+    c.close();
+    
+#ifdef WIN32
+    // Make a backup of the configuration file before copying it to the final destination.
+    std::string error_message;
+    std::string backup_path = (boost::format("%1%.bak") % path).str();
+    // Copy configuration file with PID suffix into the configuration file with "bak" suffix.
+    if (copy_file(path_pid, backup_path, error_message, false) != SUCCESS)
+        BOOST_LOG_TRIVIAL(error) << "Copying from " << path_pid << " to " << backup_path << " failed. Failed to create a backup configuration.";
+#endif
+
+    // Rename the config atomically.
+    // On Windows, the rename is likely NOT atomic, thus it may fail if PrusaSlicer crashes on another thread in the meanwhile.
+    // To cope with that, we already made a backup of the config on Windows.
     rename_file(path_pid, path);
     m_dirty = false;
 }
@@ -374,7 +531,8 @@ void AppConfig::set_recent_projects(const std::vector<std::string>& recent_proje
     }
 }
 
-void AppConfig::set_mouse_device(const std::string& name, double translation_speed, double translation_deadzone, float rotation_speed, float rotation_deadzone, double zoom_speed, bool swap_yz)
+void AppConfig::set_mouse_device(const std::string& name, double translation_speed, double translation_deadzone,
+                                 float rotation_speed, float rotation_deadzone, double zoom_speed, bool swap_yz)
 {
     std::string key = std::string("mouse_device:") + name;
     auto it = m_storage.find(key);
@@ -382,11 +540,11 @@ void AppConfig::set_mouse_device(const std::string& name, double translation_spe
         it = m_storage.insert(std::map<std::string, std::map<std::string, std::string>>::value_type(key, std::map<std::string, std::string>())).first;
 
     it->second.clear();
-    it->second["translation_speed"] = std::to_string(translation_speed);
-    it->second["translation_deadzone"] = std::to_string(translation_deadzone);
-    it->second["rotation_speed"] = std::to_string(rotation_speed);
-    it->second["rotation_deadzone"] = std::to_string(rotation_deadzone);
-    it->second["zoom_speed"] = std::to_string(zoom_speed);
+    it->second["translation_speed"] = float_to_string_decimal_point(translation_speed);
+    it->second["translation_deadzone"] = float_to_string_decimal_point(translation_deadzone);
+    it->second["rotation_speed"] = float_to_string_decimal_point(rotation_speed);
+    it->second["rotation_deadzone"] = float_to_string_decimal_point(rotation_deadzone);
+    it->second["zoom_speed"] = float_to_string_decimal_point(zoom_speed);
     it->second["swap_yz"] = swap_yz ? "1" : "0";
 }
 
@@ -395,7 +553,7 @@ std::vector<std::string> AppConfig::get_mouse_device_names() const
     static constexpr const char   *prefix     = "mouse_device:";
     static const size_t  prefix_len = strlen(prefix);
     std::vector<std::string> out;
-    for (const std::pair<std::string, std::map<std::string, std::string>>& key_value_pair : m_storage)
+    for (const auto& key_value_pair : m_storage)
         if (boost::starts_with(key_value_pair.first, prefix) && key_value_pair.first.size() > prefix_len)
             out.emplace_back(key_value_pair.first.substr(prefix_len));
     return out;
@@ -408,6 +566,8 @@ void AppConfig::update_config_dir(const std::string &dir)
 
 void AppConfig::update_skein_dir(const std::string &dir)
 {
+    if (is_shapes_dir(dir))
+        return; // do not save "shapes gallery" directory
     this->set("recent", "skein_directory", dir);
 }
 /*
@@ -440,7 +600,7 @@ std::string AppConfig::get_last_output_dir(const std::string& alt, const bool re
 		if (it2 != it->second.end() && it3 != it->second.end() && !it2->second.empty() && it3->second == "1")
 			return it2->second;
 	}
-	return alt;
+	return is_shapes_dir(alt) ? get_last_dir() : alt;
 }
 
 void AppConfig::update_last_output_dir(const std::string& dir, const bool removable)
