@@ -13,10 +13,10 @@
 #include "../BoundingBox.hpp"
 #include "../Exception.hpp"
 #include "../Utils.hpp"
+#include "../ExPolygon.hpp"
 
 namespace Slic3r {
 
-class ExPolygon;
 class Surface;
 enum InfillPattern : int;
 
@@ -44,6 +44,9 @@ struct FillParams
     float       anchor_length       { 1000.f };
     float       anchor_length_max   { 1000.f };
 
+    // G-code resolution.
+    double      resolution          { 0.0125 };
+
     // Don't adjust spacing to fill the space evenly.
     bool        dont_adjust 	{ true };
 
@@ -133,26 +136,10 @@ public:
     static void connect_infill(Polylines &&infill_ordered, const Polygons &boundary, const BoundingBox& bbox, Polylines &polylines_out, const double spacing, const FillParams &params);
     static void connect_infill(Polylines &&infill_ordered, const std::vector<const Polygon*> &boundary, const BoundingBox &bbox, Polylines &polylines_out, double spacing, const FillParams &params);
 
-    static coord_t  _adjust_solid_spacing(const coord_t width, const coord_t distance);
+    static void connect_base_support(Polylines &&infill_ordered, const std::vector<const Polygon*> &boundary_src, const BoundingBox &bbox, Polylines &polylines_out, const double spacing, const FillParams &params);
+    static void connect_base_support(Polylines &&infill_ordered, const Polygons &boundary_src, const BoundingBox &bbox, Polylines &polylines_out, const double spacing, const FillParams &params);
 
-    // Align a coordinate to a grid. The coordinate may be negative,
-    // the aligned value will never be bigger than the original one.
-    static coord_t _align_to_grid(const coord_t coord, const coord_t spacing) {
-        // Current C++ standard defines the result of integer division to be rounded to zero,
-        // for both positive and negative numbers. Here we want to round down for negative
-        // numbers as well.
-        coord_t aligned = (coord < 0) ?
-        		((coord - spacing + 1) / spacing) * spacing :
-        		(coord / spacing) * spacing;
-        assert(aligned <= coord);
-        return aligned;
-    }
-    static Point   _align_to_grid(Point   coord, Point   spacing) 
-        { return Point(_align_to_grid(coord(0), spacing(0)), _align_to_grid(coord(1), spacing(1))); }
-    static coord_t _align_to_grid(coord_t coord, coord_t spacing, coord_t base) 
-        { return base + _align_to_grid(coord - base, spacing); }
-    static Point   _align_to_grid(Point   coord, Point   spacing, Point   base)
-        { return Point(_align_to_grid(coord(0), spacing(0), base(0)), _align_to_grid(coord(1), spacing(1), base(1))); }
+    static coord_t  _adjust_solid_spacing(const coord_t width, const coord_t distance);
 };
 
 } // namespace Slic3r
