@@ -12,7 +12,7 @@ class Surface;
 class FillRectilinear : public Fill
 {
 public:
-    Fill* clone() const override { return new FillRectilinear(*this); };
+    Fill* clone() const override { return new FillRectilinear(*this); }
     ~FillRectilinear() override = default;
     Polylines fill_surface(const Surface *surface, const FillParams &params) override;
 
@@ -32,18 +32,18 @@ protected:
 class FillAlignedRectilinear : public FillRectilinear
 {
 public:
-    Fill* clone() const override { return new FillAlignedRectilinear(*this); };
+    Fill* clone() const override { return new FillAlignedRectilinear(*this); }
     ~FillAlignedRectilinear() override = default;
 
 protected:
     // Always generate infill at the same angle.
-    virtual float _layer_angle(size_t idx) const { return 0.f; }
+    virtual float _layer_angle(size_t idx) const override { return 0.f; }
 };
 
 class FillMonotonic : public FillRectilinear
 {
 public:
-    Fill* clone() const override { return new FillMonotonic(*this); };
+    Fill* clone() const override { return new FillMonotonic(*this); }
     ~FillMonotonic() override = default;
     Polylines fill_surface(const Surface *surface, const FillParams &params) override;
 	bool no_sort() const override { return true; }
@@ -52,7 +52,7 @@ public:
 class FillGrid : public FillRectilinear
 {
 public:
-    Fill* clone() const override { return new FillGrid(*this); };
+    Fill* clone() const override { return new FillGrid(*this); }
     ~FillGrid() override = default;
     Polylines fill_surface(const Surface *surface, const FillParams &params) override;
 
@@ -64,7 +64,7 @@ protected:
 class FillTriangles : public FillRectilinear
 {
 public:
-    Fill* clone() const override { return new FillTriangles(*this); };
+    Fill* clone() const override { return new FillTriangles(*this); }
     ~FillTriangles() override = default;
     Polylines fill_surface(const Surface *surface, const FillParams &params) override;
 
@@ -76,7 +76,7 @@ protected:
 class FillStars : public FillRectilinear
 {
 public:
-    Fill* clone() const override { return new FillStars(*this); };
+    Fill* clone() const override { return new FillStars(*this); }
     ~FillStars() override = default;
     Polylines fill_surface(const Surface *surface, const FillParams &params) override;
 
@@ -88,7 +88,7 @@ protected:
 class FillCubic : public FillRectilinear
 {
 public:
-    Fill* clone() const override { return new FillCubic(*this); };
+    Fill* clone() const override { return new FillCubic(*this); }
     ~FillCubic() override = default;
     Polylines fill_surface(const Surface *surface, const FillParams &params) override;
 
@@ -97,7 +97,22 @@ protected:
     float _layer_angle(size_t idx) const override { return 0.f; }
 };
 
+class FillSupportBase : public FillRectilinear
+{
+public:
+    Fill* clone() const override { return new FillSupportBase(*this); }
+    ~FillSupportBase() override = default;
+    Polylines fill_surface(const Surface *surface, const FillParams &params) override;
 
-}; // namespace Slic3r
+protected:
+    // The grid fill will keep the angle constant between the layers, see the implementation of Slic3r::Fill.
+    float _layer_angle(size_t idx) const override { return 0.f; }
+};
+
+Points sample_grid_pattern(const ExPolygon &expolygon, coord_t spacing);
+Points sample_grid_pattern(const ExPolygons &expolygons, coord_t spacing);
+Points sample_grid_pattern(const Polygons &polygons, coord_t spacing);
+
+} // namespace Slic3r
 
 #endif // slic3r_FillRectilinear_hpp_
