@@ -15,6 +15,7 @@
 #include "../I18N.hpp"
 #include "../Geometry.hpp"
 #include "../CustomGCode.hpp"
+#include "../LocalesUtils.hpp"
 
 #include "AMF.hpp"
 
@@ -24,6 +25,7 @@ namespace pt = boost::property_tree;
 
 #include <boost/filesystem/operations.hpp>
 #include <boost/algorithm/string.hpp>
+#include <boost/log/trivial.hpp>
 #include <boost/nowide/fstream.hpp>
 #include "miniz_extension.hpp"
 
@@ -62,10 +64,11 @@ namespace Slic3r
 
 struct AMFParserContext
 {
-    AMFParserContext(XML_Parser parser, DynamicPrintConfig* config, Model* model) :
+    AMFParserContext(XML_Parser parser, DynamicPrintConfig* config, ConfigSubstitutionContext* config_substitutions, Model* model) :
         m_parser(parser),
         m_model(*model), 
-        m_config(config)
+        m_config(config),
+        m_config_substitutions(config_substitutions)
     {
         m_path.reserve(12);
     }
@@ -241,11 +244,11 @@ struct AMFParserContext
     // Map from obect name to object idx & instances.
     std::map<std::string, Object> m_object_instances_map;
     // Vertices parsed for the current m_object.
-    std::vector<float>       m_object_vertices;
+    std::vector<Vec3f>       m_object_vertices;
     // Current volume allocated for an amf/object/mesh/volume subtree.
     ModelVolume             *m_volume { nullptr };
     // Faces collected for the current m_volume.
-    std::vector<int>         m_volume_facets;
+    std::vector<Vec3i>       m_volume_facets;
     // Transformation matrix of a volume mesh from its coordinate system to Object's coordinate system.
     Transform3d 			 m_volume_transform;
     // Current material allocated for an amf/metadata subtree.
@@ -256,6 +259,8 @@ struct AMFParserContext
     std::string              m_value[5];
     // Pointer to config to update if config data are stored inside the amf file
     DynamicPrintConfig      *m_config { nullptr };
+    // Config substitution rules and collected config substitution log.
+    ConfigSubstitutionContext *m_config_substitutions { nullptr };
 
 private:
     AMFParserContext& operator=(AMFParserContext&);
@@ -497,6 +502,7 @@ void AMFParserContext::characters(const XML_Char *s, int len)
 
 void AMFParserContext::endElement(const char * /* name */)
 {
+    assert(is_decimal_separator_point());
     switch (m_path.back()) {
 
     // Constellation transformation:
@@ -592,9 +598,7 @@ void AMFParserContext::endElement(const char * /* name */)
     case NODE_TYPE_VERTEX:
         assert(m_object);
         // Parse the vertex data
-        m_object_vertices.emplace_back((float)atof(m_value[0].c_str()));
-        m_object_vertices.emplace_back((float)atof(m_value[1].c_str()));
-        m_object_vertices.emplace_back((float)atof(m_value[2].c_str()));
+        m_object_vertices.emplace_back(float(atof(m_value[0].c_str())), float(atof(m_value[1].c_str())), float(atof(m_value[2].c_str())));
         m_value[0].clear();
         m_value[1].clear();
         m_value[2].clear();
@@ -603,9 +607,7 @@ void AMFParserContext::endElement(const char * /* name */)
     // Faces of the current volume:
     case NODE_TYPE_TRIANGLE:
         assert(m_object && m_volume);
-        m_volume_facets.emplace_back(atoi(m_value[0].c_str()));
-        m_volume_facets.emplace_back(atoi(m_value[1].c_str()));
-        m_volume_facets.emplace_back(atoi(m_value[2].c_str()));
+        m_volume_facets.emplace_back(atoi(m_value[0].c_str()), atoi(m_value[1].c_str()), atoi(m_value[2].c_str()));
         m_value[0].clear();
         m_value[1].clear();
         m_value[2].clear();
@@ -615,39 +617,46 @@ void AMFParserContext::endElement(const char * /* name */)
     case NODE_TYPE_VOLUME:
     {
 		assert(m_object && m_volume);
-		TriangleMesh  mesh;
-        stl_file	 &stl = mesh.stl;
-        stl.stats.type = inmemory;
-        stl.stats.number_of_facets = int(m_volume_facets.size() / 3);
-        stl.stats.original_num_facets = stl.stats.number_of_facets;
-        stl_allocate(&stl);
+        if (m_volume_facets.empty()) {
+            this->stop("An empty triangle mesh found");
+            return;
+        }
 
-        bool has_transform = ! m_volume_transform.isApprox(Transform3d::Identity(), 1e-10);
-        for (size_t i = 0; i < m_volume_facets.size();) {
-            stl_facet &facet = stl.facet_start[i/3];
-            for (unsigned int v = 0; v < 3; ++v)
-            {
-                unsigned int tri_id = m_volume_facets[i++] * 3;
-                if (tri_id < 0 || tri_id + 2 >= m_object_vertices.size()) {
-                    this->stop("Malformed triangle mesh");
-                    return;
-                }
-                facet.vertex[v] = Vec3f(m_object_vertices[tri_id + 0], m_object_vertices[tri_id + 1], m_object_vertices[tri_id + 2]);
-            }
-        }        
-        stl_get_size(&stl);
-        mesh.repair();
-		m_volume->set_mesh(std::move(mesh));
-        // stores the volume matrix taken from the metadata, if present
-        if (has_transform)
-            m_volume->source.transform = Slic3r::Geometry::Transformation(m_volume_transform);
-        if (m_volume->source.input_file.empty() && (m_volume->type() == ModelVolumeType::MODEL_PART))
         {
+            // Verify validity of face indices, find the vertex span.
+            int min_id = m_volume_facets.front()[0];
+            int max_id = min_id;
+            for (const Vec3i& face : m_volume_facets) {
+                for (const int tri_id : face) {
+                    if (tri_id < 0 || tri_id >= int(m_object_vertices.size())) {
+                        this->stop("Malformed triangle mesh");
+                        return;
+                    }
+                    min_id = std::min(min_id, tri_id);
+                    max_id = std::max(max_id, tri_id);
+                }
+            }
+
+            // rebase indices to the current vertices list
+            for (Vec3i &face : m_volume_facets)
+                face -= Vec3i(min_id, min_id, min_id);
+
+            indexed_triangle_set its { std::move(m_volume_facets), { m_object_vertices.begin() + min_id, m_object_vertices.begin() + max_id + 1 } };
+            its_compactify_vertices(its);
+            if (its_volume(its) < 0)
+                its_flip_triangles(its);
+            m_volume->set_mesh(std::move(its));
+        }
+
+        // stores the volume matrix taken from the metadata, if present
+        if (bool has_transform = !m_volume_transform.isApprox(Transform3d::Identity(), 1e-10); has_transform)
+            m_volume->source.transform = Slic3r::Geometry::Transformation(m_volume_transform);
+
+        if (m_volume->source.input_file.empty() && (m_volume->type() == ModelVolumeType::MODEL_PART)) {
             m_volume->source.object_idx = (int)m_model.objects.size() - 1;
             m_volume->source.volume_idx = (int)m_model.objects.back()->volumes.size() - 1;
             m_volume->center_geometry_after_creation();
-        }
-        else
+        } else
             // pass false if the mesh offset has been already taken from the data 
             m_volume->center_geometry_after_creation(m_volume->source.input_file.empty());
 
@@ -699,8 +708,16 @@ void AMFParserContext::endElement(const char * /* name */)
         }
 
     case NODE_TYPE_METADATA:
-        if ((m_config != nullptr) && strncmp(m_value[0].c_str(), SLIC3R_CONFIG_TYPE, strlen(SLIC3R_CONFIG_TYPE)) == 0)
-            m_config->load_from_gcode_string(m_value[1].c_str());
+        if ((m_config != nullptr) && strncmp(m_value[0].c_str(), SLIC3R_CONFIG_TYPE, strlen(SLIC3R_CONFIG_TYPE)) == 0) {
+            //FIXME Loading a "will be one day a legacy format" of configuration in a form of a G-code comment.
+            // Each config line is prefixed with a semicolon (G-code comment), that is ugly.
+
+            // Replacing the legacy function with load_from_ini_string_commented leads to issues when
+            // parsing 3MFs from before PrusaSlicer 2.0.0 (which can have duplicated entries in the INI.
+            // See https://github.com/prusa3d/PrusaSlicer/issues/7155. We'll revert it for now.
+            //m_config_substitutions->substitutions = m_config->load_from_ini_string_commented(std::move(m_value[1].c_str()), m_config_substitutions->rule);
+            ConfigBase::load_from_gcode_string_legacy(*m_config, std::move(m_value[1].c_str()), *m_config_substitutions);
+        }
         else if (strncmp(m_value[0].c_str(), "slic3r.", 7) == 0) {
             const char *opt_key = m_value[0].c_str() + 7;
             if (print_config_def.options.find(opt_key) != print_config_def.options.end()) {
@@ -718,7 +735,7 @@ void AMFParserContext::endElement(const char * /* name */)
                     config = &it->second;
                 }
                 if (config)
-                    config->set_deserialize(opt_key, m_value[1]);
+                    config->set_deserialize(opt_key, m_value[1], *m_config_substitutions);
             } else if (m_path.size() == 3 && m_path[1] == NODE_TYPE_OBJECT && m_object && strcmp(opt_key, "layer_height_profile") == 0) {
                 // Parse object's layer height profile, a semicolon separated list of floats.
                 char *p = m_value[1].data();
@@ -797,6 +814,9 @@ void AMFParserContext::endElement(const char * /* name */)
                 else if (strcmp(opt_key, "source_in_inches") == 0) {
                     m_volume->source.is_converted_from_inches = m_value[1] == "1";
                 }
+                else if (strcmp(opt_key, "source_in_meters") == 0) {
+                    m_volume->source.is_converted_from_meters = m_value[1] == "1";
+                }
             }
         } else if (m_path.size() == 3) {
             if (m_path[1] == NODE_TYPE_MATERIAL) {
@@ -827,7 +847,7 @@ void AMFParserContext::endDocument()
 {
     for (const auto &object : m_object_instances_map) {
         if (object.second.idx == -1) {
-            printf("Undefined object %s referenced in constellation\n", object.first.c_str());
+            BOOST_LOG_TRIVIAL(error) << "Undefined object " << object.first.c_str() << " referenced in constellation";
             continue;
         }
         for (const Instance &instance : object.second.instances)
@@ -843,24 +863,24 @@ void AMFParserContext::endDocument()
 }
 
 // Load an AMF file into a provided model.
-bool load_amf_file(const char *path, DynamicPrintConfig *config, Model *model)
+bool load_amf_file(const char *path, DynamicPrintConfig *config, ConfigSubstitutionContext *config_substitutions, Model *model)
 {
     if ((path == nullptr) || (model == nullptr))
         return false;
 
     XML_Parser parser = XML_ParserCreate(nullptr); // encoding
     if (!parser) {
-        printf("Couldn't allocate memory for parser\n");
+        BOOST_LOG_TRIVIAL(error) << "Couldn't allocate memory for parser";
         return false;
     }
 
     FILE *pFile = boost::nowide::fopen(path, "rt");
     if (pFile == nullptr) {
-        printf("Cannot open file %s\n", path);
+        BOOST_LOG_TRIVIAL(error) << "Cannot open file " << path;
         return false;
     }
 
-    AMFParserContext ctx(parser, config, model);
+    AMFParserContext ctx(parser, config, config_substitutions, model);
     XML_SetUserData(parser, (void*)&ctx);
     XML_SetElementHandler(parser, AMFParserContext::startElement, AMFParserContext::endElement);
     XML_SetCharacterDataHandler(parser, AMFParserContext::characters);
@@ -870,14 +890,12 @@ bool load_amf_file(const char *path, DynamicPrintConfig *config, Model *model)
     for (;;) {
         int len = (int)fread(buff, 1, 8192, pFile);
         if (ferror(pFile)) {
-            printf("AMF parser: Read error\n");
+            BOOST_LOG_TRIVIAL(error) << "AMF parser: Read error";
             break;
         }
         int done = feof(pFile);
         if (XML_Parse(parser, buff, len, done) == XML_STATUS_ERROR || ctx.error()) {
-            printf("AMF parser: Parse error at line %d:\n%s\n",
-                  (int)XML_GetCurrentLineNumber(parser),
-                  ctx.error_message());
+            BOOST_LOG_TRIVIAL(error) << "AMF parser: Parse error at line " << int(XML_GetCurrentLineNumber(parser)) << ": " << ctx.error_message();
             break;
         }
         if (done) {
@@ -904,23 +922,23 @@ bool load_amf_file(const char *path, DynamicPrintConfig *config, Model *model)
     return result;
 }
 
-bool extract_model_from_archive(mz_zip_archive& archive, const mz_zip_archive_file_stat& stat, DynamicPrintConfig* config, Model* model, bool check_version)
+bool extract_model_from_archive(mz_zip_archive& archive, const mz_zip_archive_file_stat& stat, DynamicPrintConfig* config, ConfigSubstitutionContext* config_substitutions, Model* model, bool check_version)
 {
     if (stat.m_uncomp_size == 0)
     {
-        printf("Found invalid size\n");
+        BOOST_LOG_TRIVIAL(error) << "Found invalid size";
         close_zip_reader(&archive);
         return false;
     }
 
     XML_Parser parser = XML_ParserCreate(nullptr); // encoding
     if (!parser) {
-        printf("Couldn't allocate memory for parser\n");
+        BOOST_LOG_TRIVIAL(error) << "Couldn't allocate memory for parser";
         close_zip_reader(&archive);
         return false;
     }
 
-    AMFParserContext ctx(parser, config, model);
+    AMFParserContext ctx(parser, config, config_substitutions, model);
     XML_SetUserData(parser, (void*)&ctx);
     XML_SetElementHandler(parser, AMFParserContext::startElement, AMFParserContext::endElement);
     XML_SetCharacterDataHandler(parser, AMFParserContext::characters);
@@ -954,14 +972,14 @@ bool extract_model_from_archive(mz_zip_archive& archive, const mz_zip_archive_fi
     }
     catch (std::exception& e)
     {
-        printf("%s\n", e.what());
+        BOOST_LOG_TRIVIAL(error) << "Error reading AMF file: " << e.what();
         close_zip_reader(&archive);
         return false;
     }
 
     if (res == 0)
     {
-        printf("Error while extracting model data from zip archive");
+        BOOST_LOG_TRIVIAL(error) << "Error while extracting model data from zip archive";
         close_zip_reader(&archive);
         return false;
     }
@@ -980,7 +998,7 @@ bool extract_model_from_archive(mz_zip_archive& archive, const mz_zip_archive_fi
 }
 
 // Load an AMF archive into a provided model.
-bool load_amf_archive(const char* path, DynamicPrintConfig* config, Model* model, bool check_version)
+bool load_amf_archive(const char* path, DynamicPrintConfig* config, ConfigSubstitutionContext* config_substitutions, Model* model, bool check_version)
 {
     if ((path == nullptr) || (model == nullptr))
         return false;
@@ -990,7 +1008,7 @@ bool load_amf_archive(const char* path, DynamicPrintConfig* config, Model* model
 
     if (!open_zip_reader(&archive, path))
     {
-        printf("Unable to init zip reader\n");
+        BOOST_LOG_TRIVIAL(error) << "Unable to init zip reader";
         return false;
     }
 
@@ -1006,10 +1024,10 @@ bool load_amf_archive(const char* path, DynamicPrintConfig* config, Model* model
             {
                 try
                 {
-                    if (!extract_model_from_archive(archive, stat, config, model, check_version))
+                    if (!extract_model_from_archive(archive, stat, config, config_substitutions, model, check_version))
                     {
                         close_zip_reader(&archive);
-                        printf("Archive does not contain a valid model");
+                        BOOST_LOG_TRIVIAL(error) << "Archive does not contain a valid model";
                         return false;
                     }
                 }
@@ -1048,11 +1066,13 @@ bool load_amf_archive(const char* path, DynamicPrintConfig* config, Model* model
 
 // Load an AMF file into a provided model.
 // If config is not a null pointer, updates it if the amf file/archive contains config data
-bool load_amf(const char* path, DynamicPrintConfig* config, Model* model, bool check_version)
+bool load_amf(const char* path, DynamicPrintConfig* config, ConfigSubstitutionContext* config_substitutions, Model* model, bool check_version)
 {
+    CNumericLocalesSetter locales_setter; // use "C" locales and point as a decimal separator
+
     if (boost::iends_with(path, ".amf.xml"))
         // backward compatibility with older slic3r output
-        return load_amf_file(path, config, model);
+        return load_amf_file(path, config, config_substitutions, model);
     else if (boost::iends_with(path, ".amf"))
     {
         boost::nowide::ifstream file(path, boost::nowide::ifstream::binary);
@@ -1063,7 +1083,7 @@ bool load_amf(const char* path, DynamicPrintConfig* config, Model* model, bool c
         file.read(zip_mask.data(), 2);
         file.close();
 
-        return (zip_mask == "PK") ? load_amf_archive(path, config, model, check_version) : load_amf_file(path, config, model);
+        return (zip_mask == "PK") ? load_amf_archive(path, config, config_substitutions, model, check_version) : load_amf_file(path, config, config_substitutions, model);
     }
     else
         return false;
@@ -1175,10 +1195,6 @@ bool store_amf(const char* path, Model* model, const DynamicPrintConfig* config,
         int              num_vertices = 0;
         for (ModelVolume *volume : object->volumes) {
             vertices_offsets.push_back(num_vertices);
-            if (! volume->mesh().repaired)
-                throw Slic3r::FileIOError("store_amf() requires repair()");
-			if (! volume->mesh().has_shared_vertices())
-				throw Slic3r::FileIOError("store_amf() requires shared vertices");
             const indexed_triangle_set &its = volume->mesh().its;
             const Transform3d& matrix = volume->get_matrix();
             for (size_t i = 0; i < its.vertices.size(); ++i) {
@@ -1231,8 +1247,11 @@ bool store_amf(const char* path, Model* model, const DynamicPrintConfig* config,
                 stream << "        <metadata type=\"slic3r.source_offset_y\">" << volume->source.mesh_offset(1) << "</metadata>\n";
                 stream << "        <metadata type=\"slic3r.source_offset_z\">" << volume->source.mesh_offset(2) << "</metadata>\n";
             }
+            assert(! volume->source.is_converted_from_inches || ! volume->source.is_converted_from_meters);
             if (volume->source.is_converted_from_inches)
                 stream << "        <metadata type=\"slic3r.source_in_inches\">1</metadata>\n";
+            else if (volume->source.is_converted_from_meters)
+                stream << "        <metadata type=\"slic3r.source_in_meters\">1</metadata>\n";
 			stream << std::setprecision(std::numeric_limits<float>::max_digits10);
             const indexed_triangle_set &its = volume->mesh().its;
             for (size_t i = 0; i < its.indices.size(); ++i) {
@@ -1247,40 +1266,25 @@ bool store_amf(const char* path, Model* model, const DynamicPrintConfig* config,
         stream << "  </object>\n";
         if (!object->instances.empty()) {
             for (ModelInstance *instance : object->instances) {
-                char buf[512];
-                sprintf(buf,
-                    "    <instance objectid=\"%zu\">\n"
-                    "      <deltax>%lf</deltax>\n"
-                    "      <deltay>%lf</deltay>\n"
-                    "      <deltaz>%lf</deltaz>\n"
-                    "      <rx>%lf</rx>\n"
-                    "      <ry>%lf</ry>\n"
-                    "      <rz>%lf</rz>\n"
-                    "      <scalex>%lf</scalex>\n"
-                    "      <scaley>%lf</scaley>\n"
-                    "      <scalez>%lf</scalez>\n"
-                    "      <mirrorx>%lf</mirrorx>\n"
-                    "      <mirrory>%lf</mirrory>\n"
-                    "      <mirrorz>%lf</mirrorz>\n"
-                    "      <printable>%d</printable>\n"
-                    "    </instance>\n",
-                    object_id,
-                    instance->get_offset(X),
-                    instance->get_offset(Y),
-                    instance->get_offset(Z),
-                    instance->get_rotation(X),
-                    instance->get_rotation(Y),
-                    instance->get_rotation(Z),
-                    instance->get_scaling_factor(X),
-                    instance->get_scaling_factor(Y),
-                    instance->get_scaling_factor(Z),
-                    instance->get_mirror(X),
-                    instance->get_mirror(Y),
-                    instance->get_mirror(Z),
-                    instance->printable);
+                std::stringstream buf;
+                buf << "    <instance objectid=\"" << object_id << "\">\n"
+                    << "      <deltax>"  << instance->get_offset(X)         << "</deltax>\n"
+                    << "      <deltay>"  << instance->get_offset(Y)         << "</deltay>\n"
+                    << "      <deltaz>"  << instance->get_offset(Z)         << "</deltaz>\n"
+                    << "      <rx>"      << instance->get_rotation(X)       << "</rx>\n"
+                    << "      <ry>"      << instance->get_rotation(Y)       << "</ry>\n"
+                    << "      <rz>"      << instance->get_rotation(Z)       << "</rz>\n"
+                    << "      <scalex>"  << instance->get_scaling_factor(X) << "</scalex>\n"
+                    << "      <scaley>"  << instance->get_scaling_factor(Y) << "</scaley>\n"
+                    << "      <scalez>"  << instance->get_scaling_factor(Z) << "</scalez>\n"
+                    << "      <mirrorx>" << instance->get_mirror(X)         << "</mirrorx>\n"
+                    << "      <mirrory>" << instance->get_mirror(Y)         << "</mirrory>\n"
+                    << "      <mirrorz>" << instance->get_mirror(Z)         << "</mirrorz>\n"
+                    << "      <printable>" << instance->printable << "</printable>\n"
+                    << "    </instance>\n";
 
                 //FIXME missing instance->scaling_factor
-                instances.append(buf);
+                instances.append(buf.str());
             }
         }
     }
