@@ -11,77 +11,107 @@
 
 #include <Eigen/Geometry> 
 
+#include "LocalesUtils.hpp"
+
 namespace Slic3r {
 
 class BoundingBox;
+class BoundingBoxf;
 class Line;
 class MultiPoint;
 class Point;
-typedef Point Vector;
+using Vector = Point;
+
+// Base template for eigen derived vectors
+template<int N, int M, class T>
+using Mat = Eigen::Matrix<T, N, M, Eigen::DontAlign, N, M>;
+
+template<int N, class T> using Vec = Mat<N, 1, T>;
 
 // Eigen types, to replace the Slic3r's own types in the future.
 // Vector types with a fixed point coordinate base type.
-typedef Eigen::Matrix<coord_t,  2, 1, Eigen::DontAlign> Vec2crd;
-typedef Eigen::Matrix<coord_t,  3, 1, Eigen::DontAlign> Vec3crd;
-typedef Eigen::Matrix<int,      2, 1, Eigen::DontAlign> Vec2i;
-typedef Eigen::Matrix<int,      3, 1, Eigen::DontAlign> Vec3i;
-typedef Eigen::Matrix<int32_t,  2, 1, Eigen::DontAlign> Vec2i32;
-typedef Eigen::Matrix<int64_t,  2, 1, Eigen::DontAlign> Vec2i64;
-typedef Eigen::Matrix<int32_t,  3, 1, Eigen::DontAlign> Vec3i32;
-typedef Eigen::Matrix<int64_t,  3, 1, Eigen::DontAlign> Vec3i64;
+using Vec2crd = Eigen::Matrix<coord_t,  2, 1, Eigen::DontAlign>;
+using Vec3crd = Eigen::Matrix<coord_t,  3, 1, Eigen::DontAlign>;
+using Vec2i   = Eigen::Matrix<int,      2, 1, Eigen::DontAlign>;
+using Vec3i   = Eigen::Matrix<int,      3, 1, Eigen::DontAlign>;
+using Vec4i   = Eigen::Matrix<int,      4, 1, Eigen::DontAlign>;
+using Vec2i32 = Eigen::Matrix<int32_t,  2, 1, Eigen::DontAlign>;
+using Vec2i64 = Eigen::Matrix<int64_t,  2, 1, Eigen::DontAlign>;
+using Vec3i32 = Eigen::Matrix<int32_t,  3, 1, Eigen::DontAlign>;
+using Vec3i64 = Eigen::Matrix<int64_t,  3, 1, Eigen::DontAlign>;
 
 // Vector types with a double coordinate base type.
-typedef Eigen::Matrix<float,    2, 1, Eigen::DontAlign> Vec2f;
-typedef Eigen::Matrix<float,    3, 1, Eigen::DontAlign> Vec3f;
-typedef Eigen::Matrix<double,   2, 1, Eigen::DontAlign> Vec2d;
-typedef Eigen::Matrix<double,   3, 1, Eigen::DontAlign> Vec3d;
+using Vec2f   = Eigen::Matrix<float,    2, 1, Eigen::DontAlign>;
+using Vec3f   = Eigen::Matrix<float,    3, 1, Eigen::DontAlign>;
+using Vec2d   = Eigen::Matrix<double,   2, 1, Eigen::DontAlign>;
+using Vec3d   = Eigen::Matrix<double,   3, 1, Eigen::DontAlign>;
 
-typedef std::vector<Point>                              Points;
-typedef std::vector<Point*>                             PointPtrs;
-typedef std::vector<const Point*>                       PointConstPtrs;
-typedef std::vector<Vec3crd>                            Points3;
-typedef std::vector<Vec2d>                              Pointfs;
-typedef std::vector<Vec2d>                              Vec2ds;
-typedef std::vector<Vec3d>                              Pointf3s;
+using Points         = std::vector<Point>;
+using PointPtrs      = std::vector<Point*>;
+using PointConstPtrs = std::vector<const Point*>;
+using Points3        = std::vector<Vec3crd>;
+using Pointfs        = std::vector<Vec2d>;
+using Vec2ds         = std::vector<Vec2d>;
+using Pointf3s       = std::vector<Vec3d>;
 
-typedef Eigen::Matrix<float,  2, 2, Eigen::DontAlign> Matrix2f;
-typedef Eigen::Matrix<double, 2, 2, Eigen::DontAlign> Matrix2d;
-typedef Eigen::Matrix<float,  3, 3, Eigen::DontAlign> Matrix3f;
-typedef Eigen::Matrix<double, 3, 3, Eigen::DontAlign> Matrix3d;
+using Matrix2f       = Eigen::Matrix<float,  2, 2, Eigen::DontAlign>;
+using Matrix2d       = Eigen::Matrix<double, 2, 2, Eigen::DontAlign>;
+using Matrix3f       = Eigen::Matrix<float,  3, 3, Eigen::DontAlign>;
+using Matrix3d       = Eigen::Matrix<double, 3, 3, Eigen::DontAlign>;
+using Matrix4f       = Eigen::Matrix<float,  4, 4, Eigen::DontAlign>;
+using Matrix4d       = Eigen::Matrix<double, 4, 4, Eigen::DontAlign>;
 
-typedef Eigen::Transform<float,  2, Eigen::Affine, Eigen::DontAlign> Transform2f;
-typedef Eigen::Transform<double, 2, Eigen::Affine, Eigen::DontAlign> Transform2d;
-typedef Eigen::Transform<float,  3, Eigen::Affine, Eigen::DontAlign> Transform3f;
-typedef Eigen::Transform<double, 3, Eigen::Affine, Eigen::DontAlign> Transform3d;
+template<int N, class T>
+using Transform = Eigen::Transform<float, N, Eigen::Affine, Eigen::DontAlign>;
 
-inline bool operator<(const Vec2d &lhs, const Vec2d &rhs) { return lhs(0) < rhs(0) || (lhs(0) == rhs(0) && lhs(1) < rhs(1)); }
+using Transform2f    = Eigen::Transform<float,  2, Eigen::Affine, Eigen::DontAlign>;
+using Transform2d    = Eigen::Transform<double, 2, Eigen::Affine, Eigen::DontAlign>;
+using Transform3f    = Eigen::Transform<float,  3, Eigen::Affine, Eigen::DontAlign>;
+using Transform3d    = Eigen::Transform<double, 3, Eigen::Affine, Eigen::DontAlign>;
 
-// One likely does not want to perform the cross product with a 32bit accumulator.
-//inline int32_t cross2(const Vec2i32 &v1, const Vec2i32 &v2) { return v1(0) * v2(1) - v1(1) * v2(0); }
-inline int64_t cross2(const Vec2i64 &v1, const Vec2i64 &v2) { return v1(0) * v2(1) - v1(1) * v2(0); }
-inline float   cross2(const Vec2f   &v1, const Vec2f   &v2) { return v1(0) * v2(1) - v1(1) * v2(0); }
-inline double  cross2(const Vec2d   &v1, const Vec2d   &v2) { return v1(0) * v2(1) - v1(1) * v2(0); }
+// I don't know why Eigen::Transform::Identity() return a const object...
+template<int N, class T> Transform<N, T> identity() { return Transform<N, T>::Identity(); }
+inline const auto &identity3f = identity<3, float>;
+inline const auto &identity3d = identity<3, double>;
+
+inline bool operator<(const Vec2d &lhs, const Vec2d &rhs) { return lhs.x() < rhs.x() || (lhs.x() == rhs.x() && lhs.y() < rhs.y()); }
+
+template<int Options>
+int32_t cross2(const Eigen::MatrixBase<Eigen::Matrix<int32_t, 2, 1, Options>> &v1, const Eigen::MatrixBase<Eigen::Matrix<int32_t, 2, 1, Options>> &v2) = delete;
+
+template<typename T, int Options>
+inline T cross2(const Eigen::MatrixBase<Eigen::Matrix<T, 2, 1, Options>> &v1, const Eigen::MatrixBase<Eigen::Matrix<T, 2, 1, Options>> &v2)
+{
+    return v1.x() * v2.y() - v1.y() * v2.x();
+}
+
+template<typename Derived, typename Derived2>
+inline typename Derived::Scalar cross2(const Eigen::MatrixBase<Derived> &v1, const Eigen::MatrixBase<Derived2> &v2)
+{
+    static_assert(std::is_same<typename Derived::Scalar, typename Derived2::Scalar>::value, "cross2(): Scalar types of 1st and 2nd operand must be equal.");
+    return v1.x() * v2.y() - v1.y() * v2.x();
+}
 
 template<typename T, int Options>
 inline Eigen::Matrix<T, 2, 1, Eigen::DontAlign> perp(const Eigen::MatrixBase<Eigen::Matrix<T, 2, 1, Options>> &v) { return Eigen::Matrix<T, 2, 1, Eigen::DontAlign>(- v.y(), v.x()); }
 
 template<class T, int N, int Options>
-Eigen::Matrix<T, 2, 1, Eigen::DontAlign> to_2d(const Eigen::MatrixBase<Eigen::Matrix<T, N, 1, Options>> &ptN) { return { ptN(0), ptN(1) }; }
+Eigen::Matrix<T, 2, 1, Eigen::DontAlign> to_2d(const Eigen::MatrixBase<Eigen::Matrix<T, N, 1, Options>> &ptN) { return { ptN.x(), ptN.y() }; }
 
 template<class T, int Options>
-Eigen::Matrix<T, 3, 1, Eigen::DontAlign> to_3d(const Eigen::MatrixBase<Eigen::Matrix<T, 2, 1, Options>> & pt, const T z) { return { pt(0), pt(1), z }; }
+Eigen::Matrix<T, 3, 1, Eigen::DontAlign> to_3d(const Eigen::MatrixBase<Eigen::Matrix<T, 2, 1, Options>> & pt, const T z) { return { pt.x(), pt.y(), z }; }
 
 inline Vec2d   unscale(coord_t x, coord_t y) { return Vec2d(unscale<double>(x), unscale<double>(y)); }
-inline Vec2d   unscale(const Vec2crd &pt) { return Vec2d(unscale<double>(pt(0)), unscale<double>(pt(1))); }
-inline Vec2d   unscale(const Vec2d   &pt) { return Vec2d(unscale<double>(pt(0)), unscale<double>(pt(1))); }
+inline Vec2d   unscale(const Vec2crd &pt) { return Vec2d(unscale<double>(pt.x()), unscale<double>(pt.y())); }
+inline Vec2d   unscale(const Vec2d   &pt) { return Vec2d(unscale<double>(pt.x()), unscale<double>(pt.y())); }
 inline Vec3d   unscale(coord_t x, coord_t y, coord_t z) { return Vec3d(unscale<double>(x), unscale<double>(y), unscale<double>(z)); }
-inline Vec3d   unscale(const Vec3crd &pt) { return Vec3d(unscale<double>(pt(0)), unscale<double>(pt(1)), unscale<double>(pt(2))); }
-inline Vec3d   unscale(const Vec3d   &pt) { return Vec3d(unscale<double>(pt(0)), unscale<double>(pt(1)), unscale<double>(pt(2))); }
+inline Vec3d   unscale(const Vec3crd &pt) { return Vec3d(unscale<double>(pt.x()), unscale<double>(pt.y()), unscale<double>(pt.z())); }
+inline Vec3d   unscale(const Vec3d   &pt) { return Vec3d(unscale<double>(pt.x()), unscale<double>(pt.y()), unscale<double>(pt.z())); }
 
-inline std::string to_string(const Vec2crd &pt) { return std::string("[") + std::to_string(pt(0)) + ", " + std::to_string(pt(1)) + "]"; }
-inline std::string to_string(const Vec2d   &pt) { return std::string("[") + std::to_string(pt(0)) + ", " + std::to_string(pt(1)) + "]"; }
-inline std::string to_string(const Vec3crd &pt) { return std::string("[") + std::to_string(pt(0)) + ", " + std::to_string(pt(1)) + ", " + std::to_string(pt(2)) + "]"; }
-inline std::string to_string(const Vec3d   &pt) { return std::string("[") + std::to_string(pt(0)) + ", " + std::to_string(pt(1)) + ", " + std::to_string(pt(2)) + "]"; }
+inline std::string to_string(const Vec2crd &pt) { return std::string("[") + float_to_string_decimal_point(pt.x()) + ", " + float_to_string_decimal_point(pt.y()) + "]"; }
+inline std::string to_string(const Vec2d   &pt) { return std::string("[") + float_to_string_decimal_point(pt.x()) + ", " + float_to_string_decimal_point(pt.y()) + "]"; }
+inline std::string to_string(const Vec3crd &pt) { return std::string("[") + float_to_string_decimal_point(pt.x()) + ", " + float_to_string_decimal_point(pt.y()) + ", " + float_to_string_decimal_point(pt.z()) + "]"; }
+inline std::string to_string(const Vec3d   &pt) { return std::string("[") + float_to_string_decimal_point(pt.x()) + ", " + float_to_string_decimal_point(pt.y()) + ", " + float_to_string_decimal_point(pt.z()) + "]"; }
 
 std::vector<Vec3f> transform(const std::vector<Vec3f>& points, const Transform3f& t);
 Pointf3s transform(const Pointf3s& points, const Transform3d& t);
@@ -91,7 +121,7 @@ template<int N, class T> using Vec = Eigen::Matrix<T,  N, 1, Eigen::DontAlign, N
 class Point : public Vec2crd
 {
 public:
-    typedef coord_t coord_type;
+    using coord_type = coord_t;
 
     Point() : Vec2crd(0, 0) {}
     Point(int32_t x, int32_t y) : Vec2crd(coord_t(x), coord_t(y)) {}
@@ -104,6 +134,7 @@ public:
     Point(const Eigen::MatrixBase<OtherDerived> &other) : Vec2crd(other) {}
     static Point new_scale(coordf_t x, coordf_t y) { return Point(coord_t(scale_(x)), coord_t(scale_(y))); }
     static Point new_scale(const Vec2d &v) { return Point(coord_t(scale_(v.x())), coord_t(scale_(v.y()))); }
+    static Point new_scale(const Vec2f &v) { return Point(coord_t(scale_(v.x())), coord_t(scale_(v.y()))); }
 
     // This method allows you to assign Eigen expressions to MyVectorType
     template<typename OtherDerived>
@@ -113,19 +144,17 @@ public:
         return *this;
     }
 
-    bool operator< (const Point& rhs) const { return (*this)(0) < rhs(0) || ((*this)(0) == rhs(0) && (*this)(1) < rhs(1)); }
-
-    Point& operator+=(const Point& rhs) { (*this)(0) += rhs(0); (*this)(1) += rhs(1); return *this; }
-    Point& operator-=(const Point& rhs) { (*this)(0) -= rhs(0); (*this)(1) -= rhs(1); return *this; }
-	Point& operator*=(const double &rhs) { (*this)(0) = coord_t((*this)(0) * rhs); (*this)(1) = coord_t((*this)(1) * rhs); return *this; }
-    Point operator*(const double &rhs) { return Point((*this)(0) * rhs, (*this)(1) * rhs); }
+    Point& operator+=(const Point& rhs) { this->x() += rhs.x(); this->y() += rhs.y(); return *this; }
+    Point& operator-=(const Point& rhs) { this->x() -= rhs.x(); this->y() -= rhs.y(); return *this; }
+	Point& operator*=(const double &rhs) { this->x() = coord_t(this->x() * rhs); this->y() = coord_t(this->y() * rhs); return *this; }
+    Point operator*(const double &rhs) { return Point(this->x() * rhs, this->y() * rhs); }
 
     void   rotate(double angle) { this->rotate(std::cos(angle), std::sin(angle)); }
     void   rotate(double cos_a, double sin_a) {
-        double cur_x = (double)(*this)(0);
-        double cur_y = (double)(*this)(1);
-        (*this)(0) = (coord_t)round(cos_a * cur_x - sin_a * cur_y);
-        (*this)(1) = (coord_t)round(cos_a * cur_y + sin_a * cur_x);
+        double cur_x = (double)this->x();
+        double cur_y = (double)this->y();
+        this->x() = (coord_t)round(cos_a * cur_x - sin_a * cur_y);
+        this->y() = (coord_t)round(cos_a * cur_y + sin_a * cur_x);
     }
 
     void   rotate(double angle, const Point &center);
@@ -143,6 +172,11 @@ public:
     Point  projection_onto(const Line &line) const;
 };
 
+inline bool operator<(const Point &l, const Point &r) 
+{ 
+    return l.x() < r.x() || (l.x() == r.x() && l.y() < r.y());
+}
+
 inline bool is_approx(const Point &p1, const Point &p2, coord_t epsilon = coord_t(SCALED_EPSILON))
 {
 	Point d = (p2 - p1).cwiseAbs();
@@ -179,8 +213,35 @@ inline Point lerp(const Point &a, const Point &b, double t)
     return ((1. - t) * a.cast<double>() + t * b.cast<double>()).cast<coord_t>();
 }
 
-extern BoundingBox get_extents(const Points &pts);
-extern BoundingBox get_extents(const std::vector<Points> &pts);
+BoundingBox get_extents(const Points &pts);
+BoundingBox get_extents(const std::vector<Points> &pts);
+BoundingBoxf get_extents(const std::vector<Vec2d> &pts);
+
+// Test for duplicate points in a vector of points.
+// The points are copied, sorted and checked for duplicates globally.
+bool        has_duplicate_points(std::vector<Point> &&pts);
+inline bool has_duplicate_points(const std::vector<Point> &pts)
+{
+    std::vector<Point> cpy = pts;
+    return has_duplicate_points(std::move(cpy));
+}
+
+// Test for duplicate points in a vector of points.
+// Only successive points are checked for equality.
+inline bool has_duplicate_successive_points(const std::vector<Point> &pts)
+{
+    for (size_t i = 1; i < pts.size(); ++ i)
+        if (pts[i - 1] == pts[i])
+            return true;
+    return false;
+}
+
+// Test for duplicate points in a vector of points.
+// Only successive points are checked for equality. Additionally, first and last points are compared for equality.
+inline bool has_duplicate_successive_points_closed(const std::vector<Point> &pts)
+{
+    return has_duplicate_successive_points(pts) || (pts.size() >= 2 && pts.front() == pts.back());
+}
 
 namespace int128 {
     // Exact orientation predicate,
@@ -194,7 +255,7 @@ namespace int128 {
 // To be used by std::unordered_map, std::unordered_multimap and friends.
 struct PointHash {
     size_t operator()(const Vec2crd &pt) const {
-        return std::hash<coord_t>()(pt(0)) ^ std::hash<coord_t>()(pt(1));
+        return coord_t((89 * 31 + int64_t(pt.x())) * 31 + pt.y());
     }
 };
 
@@ -255,7 +316,7 @@ public:
         const Point *pt = m_point_accessor(value);
         if (pt != nullptr) {
             // Range of fragment starts around grid_corner, close to pt.
-            auto range = m_map.equal_range(Point((*pt)(0)>>m_grid_log2, (*pt)(1)>>m_grid_log2));
+            auto range = m_map.equal_range(Point((*pt).x()>>m_grid_log2, (*pt).y()>>m_grid_log2));
             // Remove the first item.
             for (auto it = range.first; it != range.second; ++ it) {
                 if (it->second == value) {
@@ -274,12 +335,12 @@ public:
         const ValueType *value_min = nullptr;
         double           dist_min = std::numeric_limits<double>::max();
         // Round pt to a closest grid_cell corner.
-        Vec2crd            grid_corner((pt(0)+(m_grid_resolution>>1))>>m_grid_log2, (pt(1)+(m_grid_resolution>>1))>>m_grid_log2);
+        Vec2crd            grid_corner((pt.x()+(m_grid_resolution>>1))>>m_grid_log2, (pt.y()+(m_grid_resolution>>1))>>m_grid_log2);
         // For four neighbors of grid_corner:
         for (coord_t neighbor_y = -1; neighbor_y < 1; ++ neighbor_y) {
             for (coord_t neighbor_x = -1; neighbor_x < 1; ++ neighbor_x) {
                 // Range of fragment starts around grid_corner, close to pt.
-                auto range = m_map.equal_range(Vec2crd(grid_corner(0) + neighbor_x, grid_corner(1) + neighbor_y));
+                auto range = m_map.equal_range(Vec2crd(grid_corner.x() + neighbor_x, grid_corner.y() + neighbor_y));
                 // Find the map entry closest to pt.
                 for (auto it = range.first; it != range.second; ++it) {
                     const ValueType &value = it->second;
@@ -303,14 +364,14 @@ public:
     std::vector<std::pair<const ValueType*, double>> find_all(const Vec2crd &pt) {
         // Iterate over 4 closest grid cells around pt,
         // Round pt to a closest grid_cell corner.
-        Vec2crd      grid_corner((pt(0)+(m_grid_resolution>>1))>>m_grid_log2, (pt(1)+(m_grid_resolution>>1))>>m_grid_log2);
+        Vec2crd      grid_corner((pt.x()+(m_grid_resolution>>1))>>m_grid_log2, (pt.y()+(m_grid_resolution>>1))>>m_grid_log2);
         // For four neighbors of grid_corner:
         std::vector<std::pair<const ValueType*, double>> out;
         const double r2 = double(m_search_radius) * m_search_radius;
         for (coord_t neighbor_y = -1; neighbor_y < 1; ++ neighbor_y) {
             for (coord_t neighbor_x = -1; neighbor_x < 1; ++ neighbor_x) {
                 // Range of fragment starts around grid_corner, close to pt.
-                auto range = m_map.equal_range(Vec2crd(grid_corner(0) + neighbor_x, grid_corner(1) + neighbor_y));
+                auto range = m_map.equal_range(Vec2crd(grid_corner.x() + neighbor_x, grid_corner.y() + neighbor_y));
                 // Find the map entry closest to pt.
                 for (auto it = range.first; it != range.second; ++it) {
                     const ValueType &value = it->second;
@@ -327,7 +388,7 @@ public:
     }
 
 private:
-    typedef typename std::unordered_multimap<Vec2crd, ValueType, PointHash> map_type;
+    using map_type = typename std::unordered_multimap<Vec2crd, ValueType, PointHash>;
     PointAccessor m_point_accessor;
     map_type m_map;
     coord_t  m_search_radius;
@@ -386,7 +447,7 @@ template<class Tout = double,
          class = FloatingOnly<Tout>>
 inline constexpr Tout unscaled(const Tin &v) noexcept
 {
-    return Tout(v * Tout(SCALING_FACTOR));
+    return Tout(v) * Tout(SCALING_FACTOR);
 }
 
 // Unscaling for Eigen vectors. Input base type can be arithmetic, output base
@@ -400,9 +461,28 @@ template<class Tout = double,
 inline constexpr Eigen::Matrix<Tout, N, EigenArgs...>
 unscaled(const Eigen::Matrix<Tin, N, EigenArgs...> &v) noexcept
 {
-    return v.template cast<Tout>() * SCALING_FACTOR;
+    return v.template cast<Tout>() * Tout(SCALING_FACTOR);
 }
 
+// Align a coordinate to a grid. The coordinate may be negative,
+// the aligned value will never be bigger than the original one.
+inline coord_t align_to_grid(const coord_t coord, const coord_t spacing) {
+    // Current C++ standard defines the result of integer division to be rounded to zero,
+    // for both positive and negative numbers. Here we want to round down for negative
+    // numbers as well.
+    coord_t aligned = (coord < 0) ?
+            ((coord - spacing + 1) / spacing) * spacing :
+            (coord / spacing) * spacing;
+    assert(aligned <= coord);
+    return aligned;
+}
+inline Point   align_to_grid(Point   coord, Point   spacing) 
+    { return Point(align_to_grid(coord.x(), spacing.x()), align_to_grid(coord.y(), spacing.y())); }
+inline coord_t align_to_grid(coord_t coord, coord_t spacing, coord_t base) 
+    { return base + align_to_grid(coord - base, spacing); }
+inline Point   align_to_grid(Point   coord, Point   spacing, Point   base)
+    { return Point(align_to_grid(coord.x(), spacing.x(), base.x()), align_to_grid(coord.y(), spacing.y(), base.y())); }
+
 } // namespace Slic3r
 
 // start Boost
@@ -410,20 +490,20 @@ unscaled(const Eigen::Matrix<Tin, N, EigenArgs...> &v) noexcept
 #include <boost/polygon/polygon.hpp>
 namespace boost { namespace polygon {
     template <>
-    struct geometry_concept<Slic3r::Point> { typedef point_concept type; };
+    struct geometry_concept<Slic3r::Point> { using type = point_concept; };
    
     template <>
     struct point_traits<Slic3r::Point> {
-        typedef coord_t coordinate_type;
+        using coordinate_type = coord_t;
     
         static inline coordinate_type get(const Slic3r::Point& point, orientation_2d orient) {
-            return (coordinate_type)point((orient == HORIZONTAL) ? 0 : 1);
+            return static_cast<coordinate_type>(point((orient == HORIZONTAL) ? 0 : 1));
         }
     };
     
     template <>
     struct point_mutable_traits<Slic3r::Point> {
-        typedef coord_t coordinate_type;
+        using coordinate_type = coord_t;
         static inline void set(Slic3r::Point& point, orientation_2d orient, coord_t value) {
             point((orient == HORIZONTAL) ? 0 : 1) = value;
         }
@@ -451,4 +531,19 @@ namespace cereal {
 	template<class Archive> void save(Archive& archive, Slic3r::Matrix2f &m) { archive.saveBinary((char*)m.data(), sizeof(float) * 4); }
 }
 
+// To be able to use Vec<> and Mat<> in range based for loops:
+namespace Eigen {
+template<class T, int N, int M>
+T* begin(Slic3r::Mat<N, M, T> &mat) { return mat.data(); }
+
+template<class T, int N, int M>
+T* end(Slic3r::Mat<N, M, T> &mat) { return mat.data() + N * M; }
+
+template<class T, int N, int M>
+const T* begin(const Slic3r::Mat<N, M, T> &mat) { return mat.data(); }
+
+template<class T, int N, int M>
+const T* end(const Slic3r::Mat<N, M, T> &mat) { return mat.data() + N * M; }
+} // namespace Eigen
+
 #endif
