@@ -84,12 +84,13 @@ public:
         float                                   overhangs_area = 0.f;
         
         bool overlaps(const Structure &rhs) const { 
+            //FIXME ExPolygon::overlaps() shall be commutative, it is not!
             return this->bbox.overlap(rhs.bbox) && (this->polygon->overlaps(*rhs.polygon) || rhs.polygon->overlaps(*this->polygon)); 
         }
         float overlap_area(const Structure &rhs) const { 
             double out = 0.;
             if (this->bbox.overlap(rhs.bbox)) {
-                Polygons polys = intersection(to_polygons(*this->polygon), to_polygons(*rhs.polygon), false);
+                Polygons polys = intersection(*this->polygon, *rhs.polygon);
                 for (const Polygon &poly : polys)
                     out += poly.area();
             }
