@@ -1,6 +1,7 @@
 #include "Utils.hpp"
 #include "I18N.hpp"
 
+#include <atomic>
 #include <locale>
 #include <ctime>
 #include <cstdarg>
@@ -8,6 +9,7 @@
 
 #include "Platform.hpp"
 #include "Time.hpp"
+#include "libslic3r.h"
 
 #ifdef WIN32
 	#include <windows.h>
@@ -43,7 +45,23 @@
 #include <boost/nowide/convert.hpp>
 #include <boost/nowide/cstdio.hpp>
 
-#include <tbb/task_scheduler_init.h>
+// We are using quite an old TBB 2017 U7, which does not support global control API officially.
+// Before we update our build servers, let's use the old API, which is deprecated in up to date TBB.
+#include <tbb/tbb.h>
+#if ! defined(TBB_VERSION_MAJOR)
+    #include <tbb/version.h>
+#endif
+#if ! defined(TBB_VERSION_MAJOR)
+    static_assert(false, "TBB_VERSION_MAJOR not defined");
+#endif
+#if TBB_VERSION_MAJOR >= 2021
+    #define TBB_HAS_GLOBAL_CONTROL
+#endif
+#ifdef TBB_HAS_GLOBAL_CONTROL
+    #include <tbb/global_control.h>
+#else
+    #include <tbb/task_scheduler_init.h>
+#endif
 
 #if defined(__linux__) || defined(__GNUC__ )
 #include <strings.h>
@@ -118,9 +136,12 @@ void trace(unsigned int level, const char *message)
 void disable_multi_threading()
 {
     // Disable parallelization so the Shiny profiler works
-    static tbb::task_scheduler_init *tbb_init = nullptr;
-    if (tbb_init == nullptr)
-        tbb_init = new tbb::task_scheduler_init(1);
+#ifdef TBB_HAS_GLOBAL_CONTROL
+    tbb::global_control(tbb::global_control::max_allowed_parallelism, 1);
+#else // TBB_HAS_GLOBAL_CONTROL
+    static tbb::task_scheduler_init *tbb_init = new tbb::task_scheduler_init(1);
+    UNUSED(tbb_init);
+#endif // TBB_HAS_GLOBAL_CONTROL
 }
 
 static std::string g_var_dir;
@@ -165,6 +186,18 @@ const std::string& localization_dir()
 	return g_local_dir;
 }
 
+static std::string g_sys_shapes_dir;
+
+void set_sys_shapes_dir(const std::string &dir)
+{
+    g_sys_shapes_dir = dir;
+}
+
+const std::string& sys_shapes_dir()
+{
+	return g_sys_shapes_dir;
+}
+
 // Translate function callback, to call wxWidgets translate function to convert non-localized UTF8 string to a localized one.
 Slic3r::I18N::translate_fn_type Slic3r::I18N::translate_fn = nullptr;
 
@@ -180,6 +213,28 @@ const std::string& data_dir()
     return g_data_dir;
 }
 
+std::string custom_shapes_dir()
+{
+    return (boost::filesystem::path(g_data_dir) / "shapes").string();
+}
+
+static std::atomic<bool> debug_out_path_called(false);
+
+std::string debug_out_path(const char *name, ...)
+{
+	static constexpr const char *SLIC3R_DEBUG_OUT_PATH_PREFIX = "out/";
+    if (! debug_out_path_called.exchange(true)) {
+		std::string path = boost::filesystem::system_complete(SLIC3R_DEBUG_OUT_PATH_PREFIX).string();
+        printf("Debugging output files will be written to %s\n", path.c_str());
+    }
+	char buffer[2048];
+	va_list args;
+	va_start(args, name);
+	std::vsprintf(buffer, name, args);
+	va_end(args);
+	return std::string(SLIC3R_DEBUG_OUT_PATH_PREFIX) + std::string(buffer);
+}
+
 #ifdef _WIN32
 // The following helpers are borrowed from the LLVM project https://github.com/llvm
 namespace WindowsSupport
@@ -424,6 +479,53 @@ std::error_code rename_file(const std::string &from, const std::string &to)
 }
 
 #ifdef __linux__
+// Copied from boost::filesystem. 
+// Called by copy_file_linux() in case linux sendfile() API is not supported.
+int copy_file_linux_read_write(int infile, int outfile, uintmax_t file_size)
+{
+    std::vector<char> buf(
+	    // Prefer the buffer to be larger than the file size so that we don't have
+	    // to perform an extra read if the file fits in the buffer exactly.
+    	std::clamp<size_t>(file_size + (file_size < ~static_cast<uintmax_t >(0u)),
+		// Min and max buffer sizes are selected to minimize the overhead from system calls.
+		// The values are picked based on coreutils cp(1) benchmarking data described here:
+		// https://github.com/coreutils/coreutils/blob/d1b0257077c0b0f0ee25087efd46270345d1dd1f/src/ioblksize.h#L23-L72
+    			   		   8u * 1024u, 256u * 1024u),
+    	0);
+
+#if defined(POSIX_FADV_SEQUENTIAL)
+    ::posix_fadvise(infile, 0, 0, POSIX_FADV_SEQUENTIAL);
+#endif
+
+    // Don't use file size to limit the amount of data to copy since some filesystems, like procfs or sysfs,
+    // provide files with generated content and indicate that their size is zero or 4096. Just copy as much data
+    // as we can read from the input file.
+    while (true) {
+        ssize_t sz_read = ::read(infile, buf.data(), buf.size());
+        if (sz_read == 0)
+            break;
+        if (sz_read < 0) {
+            int err = errno;
+            if (err == EINTR)
+                continue;
+            return err;
+        }
+        // Allow for partial writes - see Advanced Unix Programming (2nd Ed.),
+        // Marc Rochkind, Addison-Wesley, 2004, page 94
+        for (ssize_t sz_wrote = 0; sz_wrote < sz_read;) {
+            ssize_t sz = ::write(outfile, buf.data() + sz_wrote, static_cast<std::size_t>(sz_read - sz_wrote));
+            if (sz < 0) {
+                int err = errno;
+                if (err == EINTR)
+                    continue;
+                return err;
+            }
+            sz_wrote += sz;
+        }
+    }
+    return 0;
+}
+
 // Copied from boost::filesystem, to support copying a file to a weird filesystem, which does not support changing file attributes,
 // for example ChromeOS Linux integration or FlashAIR WebDAV.
 // Copied and simplified from boost::filesystem::detail::copy_file() with option = overwrite_if_exists and with just the Linux path kept,
@@ -520,6 +622,19 @@ bool copy_file_linux(const boost::filesystem::path &from, const boost::filesyste
 			ssize_t sz = ::sendfile(outfile.fd, infile.fd, nullptr, size_to_copy);
 			if (sz < 0) {
 				err = errno;
+	            if (offset == 0u) {
+	                // sendfile may fail with EINVAL if the underlying filesystem does not support it.
+	                // See https://patchwork.kernel.org/project/linux-nfs/patch/20190411183418.4510-1-olga.kornievskaia@gmail.com/
+	                // https://bugzilla.redhat.com/show_bug.cgi?id=1783554.
+	                // https://github.com/boostorg/filesystem/commit/4b9052f1e0b2acf625e8247582f44acdcc78a4ce
+	                if (err == EINVAL || err == EOPNOTSUPP) {
+						err = copy_file_linux_read_write(infile.fd, outfile.fd, from_stat.st_size);
+						if (err < 0)
+							goto fail;
+						// Succeeded.
+	                	break;
+	                }
+	            }
 				if (err == EINTR)
 					continue;
 				if (err == 0)
@@ -674,6 +789,26 @@ bool is_gcode_file(const std::string &path)
 		   boost::iends_with(path, ".g")     || boost::iends_with(path, ".ngc");
 }
 
+bool is_img_file(const std::string &path)
+{
+	return boost::iends_with(path, ".png") || boost::iends_with(path, ".svg");
+}
+
+bool is_gallery_file(const boost::filesystem::directory_entry& dir_entry, char const* type)
+{
+	return is_plain_file(dir_entry) && strcasecmp(dir_entry.path().extension().string().c_str(), type) == 0;
+}
+
+bool is_gallery_file(const std::string &path, char const* type)
+{
+	return boost::iends_with(path, type);
+}
+
+bool is_shapes_dir(const std::string& dir)
+{
+	return dir == sys_shapes_dir() || dir == custom_shapes_dir();
+}
+
 } // namespace Slic3r
 
 #ifdef WIN32
@@ -764,7 +899,7 @@ std::string string_printf(const char *format, ...)
 
 std::string header_slic3r_generated()
 {
-	return std::string("generated by " SLIC3R_APP_NAME " " SLIC3R_VERSION " on " ) + Utils::utc_timestamp();
+	return std::string("generated by PrusaSlicer " SLIC3R_VERSION " on " ) + Utils::utc_timestamp();
 }
 
 std::string header_gcodeviewer_generated()
@@ -781,7 +916,8 @@ unsigned get_current_pid()
 #endif
 }
 
-std::string xml_escape(std::string text)
+//FIXME this has potentially O(n^2) time complexity!
+std::string xml_escape(std::string text, bool is_marked/* = false*/)
 {
     std::string::size_type pos = 0;
     for (;;)
@@ -796,8 +932,8 @@ std::string xml_escape(std::string text)
         case '\"': replacement = "&quot;"; break;
         case '\'': replacement = "&apos;"; break;
         case '&':  replacement = "&amp;";  break;
-        case '<':  replacement = "&lt;";   break;
-        case '>':  replacement = "&gt;";   break;
+        case '<':  replacement = is_marked ? "<" :"&lt;"; break;
+        case '>':  replacement = is_marked ? ">" :"&gt;"; break;
         default: break;
         }
 
@@ -859,16 +995,11 @@ std::string log_memory_info(bool ignore_loglevel)
         } PROCESS_MEMORY_COUNTERS_EX, *PPROCESS_MEMORY_COUNTERS_EX;
     #endif /* PROCESS_MEMORY_COUNTERS_EX */
 
-
-        HANDLE hProcess = ::OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, ::GetCurrentProcessId());
-        if (hProcess != nullptr) {
-            PROCESS_MEMORY_COUNTERS_EX pmc;
-            if (GetProcessMemoryInfo(hProcess, (PROCESS_MEMORY_COUNTERS*)&pmc, sizeof(pmc)))
-                out = " WorkingSet: " + format_memsize_MB(pmc.WorkingSetSize) + "; PrivateBytes: " + format_memsize_MB(pmc.PrivateUsage) + "; Pagefile(peak): " + format_memsize_MB(pmc.PagefileUsage) + "(" + format_memsize_MB(pmc.PeakPagefileUsage) + ")";
-            else
-                out += " Used memory: N/A";
-            CloseHandle(hProcess);
-        }
+        PROCESS_MEMORY_COUNTERS_EX pmc;
+        if (GetProcessMemoryInfo(GetCurrentProcess(), (PROCESS_MEMORY_COUNTERS*)&pmc, sizeof(pmc)))
+            out = " WorkingSet: " + format_memsize_MB(pmc.WorkingSetSize) + "; PrivateBytes: " + format_memsize_MB(pmc.PrivateUsage) + "; Pagefile(peak): " + format_memsize_MB(pmc.PagefileUsage) + "(" + format_memsize_MB(pmc.PeakPagefileUsage) + ")";
+        else
+            out += " Used memory: N/A";
 #elif defined(__linux__) or defined(__APPLE__)
         // Get current memory usage.
     #ifdef __APPLE__
