@@ -127,6 +127,13 @@ private:
 	std::vector<Snapshot>			m_snapshots;
 };
 
+// Take snapshot on SnapshotDB::singleton(). If taking snapshot fails, report an error and return nullptr.
+const Snapshot* take_config_snapshot_report_error(const AppConfig &app_config, Snapshot::Reason reason, const std::string &comment);
+
+// Take snapshot on SnapshotDB::singleton(). If taking snapshot fails, report "message", and present a "Continue" or "Abort" buttons to respond.
+// Return true on success and on "Continue" to continue with the process (for example installation of presets).
+bool take_config_snapshot_cancel_on_error(const AppConfig &app_config, Snapshot::Reason reason, const std::string &comment, const std::string &message, Snapshot const **psnapshot = nullptr);
+
 } // namespace Config
 } // namespace GUI
 } // namespace Slic3r
