@@ -7,6 +7,8 @@
 #include "2DBed.hpp"
 #include "I18N.hpp"
 
+#include <libslic3r/BuildVolume.hpp>
+
 #include <wx/dialog.h>
 #include <wx/choicebk.h>
 
@@ -19,11 +21,10 @@ using ConfigOptionsGroupShp = std::shared_ptr<ConfigOptionsGroup>;
 
 struct BedShape
 {
-    enum class Type {
-        Rectangular = 0,
-        Circular,
-        Custom,
-        Invalid
+    enum class PageType {
+        Rectangle,
+        Circle,
+        Custom
     };
 
     enum class Parameter {
@@ -34,22 +35,18 @@ struct BedShape
 
     BedShape(const ConfigOptionPoints& points);
 
-    bool            is_custom() { return m_type == Type::Custom; }
+    bool            is_custom() { return m_build_volume.type() == BuildVolume::Type::Convex || m_build_volume.type() == BuildVolume::Type::Custom; }
 
     static void     append_option_line(ConfigOptionsGroupShp optgroup, Parameter param);
-    static wxString get_name(Type type);
+    static wxString get_name(PageType type);
 
-    // convert Type to size_t
-    size_t          get_type();
+    PageType        get_page_type();
 
     wxString        get_full_name_with_params();
     void            apply_optgroup_values(ConfigOptionsGroupShp optgroup);
 
 private:
-    Type    m_type          {Type::Invalid};
-    Vec2d   m_rectSize      {200, 200};
-    Vec2d   m_rectOrigin    {0, 0};
-    double  m_diameter      {0};
+    BuildVolume m_build_volume;
 };
 
 class BedShapePanel : public wxPanel
