@@ -5,6 +5,7 @@
 #include "format.hpp"
 #include "libslic3r/Model.hpp"
 #include "libslic3r/PresetBundle.hpp"
+#include "MsgDialog.hpp"
 
 #include <wx/msgdlg.h>
 
@@ -45,8 +46,8 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
     // layer_height shouldn't be equal to zero
     if (config->opt_float("layer_height") < EPSILON)
     {
-        const wxString msg_text = _(L("Zero layer height is not valid.\n\nThe layer height will be reset to 0.01."));
-        wxMessageDialog dialog(nullptr, msg_text, _(L("Layer height")), wxICON_WARNING | wxOK);
+        const wxString msg_text = _(L("Layer height is not valid.\n\nThe layer height will be reset to 0.01."));
+        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("Layer height")), wxICON_WARNING | wxOK);
         DynamicPrintConfig new_conf = *config;
         is_msg_dlg_already_exist = true;
         dialog.ShowModal();
@@ -55,10 +56,10 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
         is_msg_dlg_already_exist = false;
     }
 
-    if (fabs(config->option<ConfigOptionFloatOrPercent>("first_layer_height")->value - 0) < EPSILON)
+    if (config->option<ConfigOptionFloatOrPercent>("first_layer_height")->value < EPSILON)
     {
-        const wxString msg_text = _(L("Zero first layer height is not valid.\n\nThe first layer height will be reset to 0.01."));
-        wxMessageDialog dialog(nullptr, msg_text, _(L("First layer height")), wxICON_WARNING | wxOK);
+        const wxString msg_text = _(L("First layer height is not valid.\n\nThe first layer height will be reset to 0.01."));
+        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("First layer height")), wxICON_WARNING | wxOK);
         DynamicPrintConfig new_conf = *config;
         is_msg_dlg_already_exist = true;
         dialog.ShowModal();
@@ -87,10 +88,11 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
                					"- Detect thin walls disabled"));
         if (is_global_config)
             msg_text += "\n\n" + _(L("Shall I adjust those settings in order to enable Spiral Vase?"));
-        wxMessageDialog dialog(nullptr, msg_text, _(L("Spiral Vase")),
+        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("Spiral Vase")),
                                wxICON_WARNING | (is_global_config ? wxYES | wxNO : wxOK));
         DynamicPrintConfig new_conf = *config;
         auto answer = dialog.ShowModal();
+        bool support = true;
         if (!is_global_config || answer == wxID_YES) {
             new_conf.set_key_value("perimeters", new ConfigOptionInt(1));
             new_conf.set_key_value("top_solid_layers", new ConfigOptionInt(0));
@@ -100,13 +102,17 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
             new_conf.set_key_value("ensure_vertical_shell_thickness", new ConfigOptionBool(true));
             new_conf.set_key_value("thin_walls", new ConfigOptionBool(false));            
             fill_density = 0;
+            support = false;
         }
         else {
             new_conf.set_key_value("spiral_vase", new ConfigOptionBool(false));
         }
         apply(config, &new_conf);
-        if (cb_value_change)
+        if (cb_value_change) {
             cb_value_change("fill_density", fill_density);
+            if (!support)
+                cb_value_change("support_material", false);
+        }
     }
 
     if (config->opt_bool("wipe_tower") && config->opt_bool("support_material") &&
@@ -117,7 +123,7 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
                                 "(both support_material_extruder and support_material_interface_extruder need to be set to 0)."));
         if (is_global_config)
             msg_text += "\n\n" + _(L("Shall I adjust those settings in order to enable the Wipe Tower?"));
-        wxMessageDialog dialog (nullptr, msg_text, _(L("Wipe Tower")),
+        MessageDialog dialog (m_msg_dlg_parent, msg_text, _(L("Wipe Tower")),
                                 wxICON_WARNING | (is_global_config ? wxYES | wxNO : wxOK));
         DynamicPrintConfig new_conf = *config;
         auto answer = dialog.ShowModal();
@@ -137,7 +143,7 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
                                 "need to be synchronized with the object layers."));
         if (is_global_config)
             msg_text += "\n\n" + _(L("Shall I synchronize support layers in order to enable the Wipe Tower?"));
-        wxMessageDialog dialog(nullptr, msg_text, _(L("Wipe Tower")),
+        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("Wipe Tower")),
                                wxICON_WARNING | (is_global_config ? wxYES | wxNO : wxOK));
         DynamicPrintConfig new_conf = *config;
         auto answer = dialog.ShowModal();
@@ -149,39 +155,30 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
         apply(config, &new_conf);
     }
 
-    static bool support_material_overhangs_queried = false;
-
-    if (config->opt_bool("support_material")) {
+    // Check "support_material" and "overhangs" relations only on global settings level
+    if (is_global_config && config->opt_bool("support_material")) {
         // Ask only once.
-        if (!support_material_overhangs_queried) {
-            support_material_overhangs_queried = true;
+        if (!m_support_material_overhangs_queried) {
+            m_support_material_overhangs_queried = true;
             if (!config->opt_bool("overhangs")/* != 1*/) {
                 wxString msg_text = _(L("Supports work better, if the following feature is enabled:\n"
                                         "- Detect bridging perimeters"));
                 if (is_global_config)
                     msg_text += "\n\n" + _(L("Shall I adjust those settings for supports?"));
-                wxMessageDialog dialog(nullptr, msg_text, _(L("Support Generator")),
-                                       wxICON_WARNING | (is_global_config ? wxYES | wxNO | wxCANCEL : wxOK));
+                MessageDialog dialog(m_msg_dlg_parent, msg_text, _L("Support Generator"), wxICON_WARNING | wxYES | wxNO);
                 DynamicPrintConfig new_conf = *config;
                 auto answer = dialog.ShowModal();
-                if (!is_global_config || answer == wxID_YES) {
+                if (answer == wxID_YES) {
                     // Enable "detect bridging perimeters".
                     new_conf.set_key_value("overhangs", new ConfigOptionBool(true));
                 }
-                else if (answer == wxID_NO) {
-                    // Do nothing, leave supports on and "detect bridging perimeters" off.
-                }
-                else if (answer == wxID_CANCEL) {
-                    // Disable supports.
-                    new_conf.set_key_value("support_material", new ConfigOptionBool(false));
-                    support_material_overhangs_queried = false;
-                }
+                //else Do nothing, leave supports on and "detect bridging perimeters" off.
                 apply(config, &new_conf);
             }
         }
     }
     else {
-        support_material_overhangs_queried = false;
+        m_support_material_overhangs_queried = false;
     }
 
     if (config->option<ConfigOptionPercent>("fill_density")->value == 100) {
@@ -199,7 +196,7 @@ void ConfigManipulation::update_print_fff_config(DynamicPrintConfig* config, con
                     _(fill_pattern_def->enum_labels[it_pattern - fill_pattern_def->enum_values.begin()]));
                 if (is_global_config)
                     msg_text += "\n\n" + _L("Shall I switch to rectilinear fill pattern?");
-                wxMessageDialog dialog(nullptr, msg_text, _L("Infill"),
+                MessageDialog dialog(m_msg_dlg_parent, msg_text, _L("Infill"),
                                                   wxICON_WARNING | (is_global_config ? wxYES | wxNO : wxOK) );
                 DynamicPrintConfig new_conf = *config;
                 auto answer = dialog.ShowModal();
@@ -255,7 +252,7 @@ void ConfigManipulation::toggle_print_fff_options(DynamicPrintConfig* config)
     toggle_field("gap_fill_speed", have_perimeters);
 
     for (auto el : { "top_infill_extrusion_width", "top_solid_infill_speed" })
-        toggle_field(el, has_top_solid_infill);
+        toggle_field(el, has_top_solid_infill || (has_spiral_vase && has_bottom_solid_infill));
 
     bool have_default_acceleration = config->opt_float("default_acceleration") > 0;
     for (auto el : { "perimeter_acceleration", "infill_acceleration",
@@ -263,11 +260,13 @@ void ConfigManipulation::toggle_print_fff_options(DynamicPrintConfig* config)
         toggle_field(el, have_default_acceleration);
 
     bool have_skirt = config->opt_int("skirts") > 0;
-    toggle_field("skirt_height", have_skirt && !config->opt_bool("draft_shield"));
+    toggle_field("skirt_height", have_skirt && config->opt_enum<DraftShield>("draft_shield") != dsEnabled);
     for (auto el : { "skirt_distance", "draft_shield", "min_skirt_length" })
         toggle_field(el, have_skirt);
 
-    bool have_brim = config->opt_float("brim_width") > 0;
+    bool have_brim = config->opt_enum<BrimType>("brim_type") != btNoBrim;
+    for (auto el : { "brim_width", "brim_separation" })
+        toggle_field(el, have_brim);
     // perimeter_extruder uses the same logic as in Print::extruders()
     toggle_field("perimeter_extruder", have_perimeters || have_brim);
 
@@ -276,14 +275,18 @@ void ConfigManipulation::toggle_print_fff_options(DynamicPrintConfig* config)
     bool have_support_material_auto = have_support_material && config->opt_bool("support_material_auto");
     bool have_support_interface = config->opt_int("support_material_interface_layers") > 0;
     bool have_support_soluble = have_support_material && config->opt_float("support_material_contact_distance") == 0;
-    for (auto el : { "support_material_pattern", "support_material_with_sheath",
-                    "support_material_spacing", "support_material_angle", "support_material_interface_layers",
+    auto support_material_style = config->opt_enum<SupportMaterialStyle>("support_material_style");
+    for (auto el : { "support_material_style", "support_material_pattern", "support_material_with_sheath",
+                    "support_material_spacing", "support_material_angle", 
+                    "support_material_interface_pattern", "support_material_interface_layers",
                     "dont_support_bridges", "support_material_extrusion_width", "support_material_contact_distance",
                     "support_material_xy_spacing" })
         toggle_field(el, have_support_material);
     toggle_field("support_material_threshold", have_support_material_auto);
+    toggle_field("support_material_bottom_contact_distance", have_support_material && ! have_support_soluble);
+    toggle_field("support_material_closing_radius", have_support_material && support_material_style == smsSnug);
 
-    for (auto el : { "support_material_interface_spacing", "support_material_interface_extruder",
+    for (auto el : { "support_material_bottom_interface_layers", "support_material_interface_spacing", "support_material_interface_extruder",
                     "support_material_interface_speed", "support_material_interface_contact_loops" })
         toggle_field(el, have_support_material && have_support_interface);
     toggle_field("support_material_synchronize_layers", have_support_soluble);
@@ -292,6 +295,10 @@ void ConfigManipulation::toggle_print_fff_options(DynamicPrintConfig* config)
     toggle_field("support_material_extruder", have_support_material || have_skirt);
     toggle_field("support_material_speed", have_support_material || have_brim || have_skirt);
 
+    toggle_field("raft_contact_distance", have_raft && !have_support_soluble);
+    for (auto el : { "raft_expansion", "first_layer_acceleration_over_raft", "first_layer_speed_over_raft" })
+        toggle_field(el, have_raft);
+
     bool has_ironing = config->opt_bool("ironing");
     for (auto el : { "ironing_type", "ironing_flowrate", "ironing_spacing", "ironing_speed" })
     	toggle_field(el, has_ironing);
@@ -304,7 +311,7 @@ void ConfigManipulation::toggle_print_fff_options(DynamicPrintConfig* config)
     toggle_field("standby_temperature_delta", have_ooze_prevention);
 
     bool have_wipe_tower = config->opt_bool("wipe_tower");
-    for (auto el : { "wipe_tower_x", "wipe_tower_y", "wipe_tower_width", "wipe_tower_rotation_angle",
+    for (auto el : { "wipe_tower_x", "wipe_tower_y", "wipe_tower_width", "wipe_tower_rotation_angle", "wipe_tower_brim_width",
                      "wipe_tower_bridging", "wipe_tower_no_sparse_layers", "single_extruder_multi_material_priming" })
         toggle_field(el, have_wipe_tower);
 
@@ -319,7 +326,7 @@ void ConfigManipulation::update_print_sla_config(DynamicPrintConfig* config, con
     if (head_penetration > head_width) {
         wxString msg_text = _(L("Head penetration should not be greater than the head width."));
 
-        wxMessageDialog dialog(nullptr, msg_text, _(L("Invalid Head penetration")), wxICON_WARNING | wxOK);
+        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("Invalid Head penetration")), wxICON_WARNING | wxOK);
         DynamicPrintConfig new_conf = *config;
         if (dialog.ShowModal() == wxID_OK) {
             new_conf.set_key_value("support_head_penetration", new ConfigOptionFloat(head_width));
@@ -332,7 +339,7 @@ void ConfigManipulation::update_print_sla_config(DynamicPrintConfig* config, con
     if (pinhead_d > pillar_d) {
         wxString msg_text = _(L("Pinhead diameter should be smaller than the pillar diameter."));
 
-        wxMessageDialog dialog(nullptr, msg_text, _(L("Invalid pinhead diameter")), wxICON_WARNING | wxOK);
+        MessageDialog dialog(m_msg_dlg_parent, msg_text, _(L("Invalid pinhead diameter")), wxICON_WARNING | wxOK);
 
         DynamicPrintConfig new_conf = *config;
         if (dialog.ShowModal() == wxID_OK) {
