@@ -37,39 +37,7 @@ using t_change = std::function<void(const t_config_option_key&, const boost::any
 using t_back_to_init = std::function<void(const std::string&)>;
 
 wxString double_to_string(double const value, const int max_precision = 4);
-
-class RevertButton : public ScalableButton
-{
-    bool hidden = false; // never show button if it's hidden ones
-public:
-// 	RevertButton() {} 
-// 	RevertButton(wxWindow* parent, wxWindowID id, const wxString& label = wxEmptyString,
-// 		const wxPoint& pos = wxDefaultPosition,
-// 		const wxSize& size = wxDefaultSize, long style = 0,
-// 		const wxValidator& validator = wxDefaultValidator,
-// 		const wxString& name = wxTextCtrlNameStr)
-// 	{
-// 		this->Create(parent, id, label, pos, size, style, validator, name);
-// 	}
-    RevertButton(
-        wxWindow *parent,
-        const std::string& icon_name = ""
-        ) :
-        ScalableButton(parent, wxID_ANY, icon_name) {}
-
-	// overridden from wxWindow base class
-	virtual bool
-		AcceptsFocusFromKeyboard() const { return false; }
-
-    void set_as_hidden() {
-        Hide();
-        hidden = true;
-	}
-
-    virtual bool Show(bool show = true) override {
-        return wxButton::Show(hidden ? false : show);
-	}
-};
+wxString get_thumbnails_string(const std::vector<Vec2d>& values);
 
 class Field {
 protected:
@@ -84,9 +52,18 @@ protected:
 	//! in another case we can't unfocused control at all
 	void			on_kill_focus();
     /// Call the attached on_change method. 
-    void			on_set_focus(wxEvent& event);
-    /// Call the attached on_change method. 
     void			on_change_field();
+
+    class EnterPressed {
+    public:
+        EnterPressed(Field* field) : 
+            m_parent(field){ m_parent->set_enter_pressed(true);  }
+        ~EnterPressed()    { m_parent->set_enter_pressed(false); }
+    private:
+        Field* m_parent;
+    };
+
+public:
     /// Call the attached m_back_to_initial_value method. 
 	void			on_back_to_initial_value();
     /// Call the attached m_back_to_sys_value method. 
@@ -99,9 +76,6 @@ public:
     /// Function object to store callback passed in from owning object.
 	t_kill_focus	m_on_kill_focus {nullptr};
 
-    /// Function object to store callback passed in from owning object.
-	t_kill_focus	m_on_set_focus {nullptr};
-
     /// Function object to store callback passed in from owning object.
 	t_change		m_on_change {nullptr};
 
@@ -119,6 +93,9 @@ public:
 	const t_config_option_key		m_opt_id;//! {""};
 	int								m_opt_idx = 0;
 
+	double							opt_height{ 0.0 };
+	bool							parent_is_custom_ctrl{ false };
+
     /// Sets a value for this control.
     /// subclasses should overload with a specific version
     /// Postcondition: Method does not fire the on_change event.
@@ -140,12 +117,9 @@ public:
 
     void				field_changed() { on_change_field(); }
 
-    // set icon to "UndoToSystemValue" button according to an inheritance of preset
-//	void				set_nonsys_btn_icon(const wxBitmap& icon);
-
-    Field(const ConfigOptionDef& opt, const t_config_option_key& id) : m_opt(opt), m_opt_id(id) {};
-    Field(wxWindow* parent, const ConfigOptionDef& opt, const t_config_option_key& id) : m_parent(parent), m_opt(opt), m_opt_id(id) {};
-    virtual ~Field() {}
+    Field(const ConfigOptionDef& opt, const t_config_option_key& id) : m_opt(opt), m_opt_id(id) {}
+    Field(wxWindow* parent, const ConfigOptionDef& opt, const t_config_option_key& id) : m_parent(parent), m_opt(opt), m_opt_id(id) {}
+    virtual ~Field();
 
     /// If you don't know what you are getting back, check both methods for nullptr. 
     virtual wxSizer*	getSizer()  { return nullptr; }
@@ -166,7 +140,6 @@ public:
     bool 	set_undo_bitmap(const ScalableBitmap *bmp) {
     	if (m_undo_bitmap != bmp) {
     		m_undo_bitmap = bmp;
-    		m_Undo_btn->SetBitmap_(*bmp);
     		return true;
     	}
     	return false;
@@ -175,33 +148,21 @@ public:
     bool 	set_undo_to_sys_bitmap(const ScalableBitmap *bmp) {
     	if (m_undo_to_sys_bitmap != bmp) {
     		m_undo_to_sys_bitmap = bmp;
-    		m_Undo_to_sys_btn->SetBitmap_(*bmp);
     		return true;
     	}
     	return false;
     }
 
 	bool	set_label_colour(const wxColour *clr) {
-		if (m_Label == nullptr) return false;
 		if (m_label_color != clr) {
 			m_label_color = clr;
-			m_Label->SetForegroundColour(*clr);
-			m_Label->Refresh(true);
 		}
 		return false;
 	}
 
-	bool	set_label_colour_force(const wxColour *clr) {
-		if (m_Label == nullptr) return false;
-		m_Label->SetForegroundColour(*clr);
-		m_Label->Refresh(true);
-		return false;
-	}
-
 	bool 	set_undo_tooltip(const wxString *tip) {
 		if (m_undo_tooltip != tip) {
 			m_undo_tooltip = tip;
-			m_Undo_btn->SetToolTip(*tip);
 			return true;
 		}
 		return false;
@@ -210,17 +171,17 @@ public:
 	bool 	set_undo_to_sys_tooltip(const wxString *tip) {
 		if (m_undo_to_sys_tooltip != tip) {
 			m_undo_to_sys_tooltip = tip;
-			m_Undo_to_sys_btn->SetToolTip(*tip);
 			return true;
 		}
 		return false;
 	}
 
-	void	set_side_text_ptr(wxStaticText* side_text) {
-		m_side_text = side_text;
+	bool*	get_blink_ptr() {
+		return &m_blink;
     }
 
-    virtual void msw_rescale(bool rescale_sidetext = false);
+    virtual void msw_rescale();
+    virtual void sys_color_changed();
 
     bool get_enter_pressed() const { return bEnterPressed; }
     void set_enter_pressed(bool pressed) { bEnterPressed = pressed; }
@@ -230,22 +191,26 @@ public:
 	static int def_width_wider()	;
 	static int def_width_thinner()	;
 
+	const ScalableBitmap*	undo_bitmap()			{ return m_undo_bitmap; }
+	const wxString*			undo_tooltip()			{ return m_undo_tooltip; }
+	const ScalableBitmap*	undo_to_sys_bitmap()	{ return m_undo_to_sys_bitmap; }
+	const wxString*			undo_to_sys_tooltip()	{ return m_undo_to_sys_tooltip; }
+	const wxColour*			label_color()			{ return m_label_color; }
+	const bool				blink()					{ return m_blink; }
+
 protected:
-	RevertButton*			m_Undo_btn = nullptr;
 	// Bitmap and Tooltip text for m_Undo_btn. The wxButton will be updated only if the new wxBitmap pointer differs from the currently rendered one.
 	const ScalableBitmap*   m_undo_bitmap = nullptr;
 	const wxString*         m_undo_tooltip = nullptr;
-	RevertButton*			m_Undo_to_sys_btn = nullptr;
 	// Bitmap and Tooltip text for m_Undo_to_sys_btn. The wxButton will be updated only if the new wxBitmap pointer differs from the currently rendered one.
     const ScalableBitmap*   m_undo_to_sys_bitmap = nullptr;
 	const wxString*		    m_undo_to_sys_tooltip = nullptr;
 
-	wxStaticText*		m_Label = nullptr;
+	bool					m_blink{ false };
+
 	// Color for Label. The wxColour will be updated only if the new wxColour pointer differs from the currently rendered one.
 	const wxColour*		m_label_color = nullptr;
 
-	wxStaticText*		m_side_text = nullptr;
-
 	// current value
 	boost::any			m_value;
     // last maeningful value
@@ -275,10 +240,6 @@ class TextCtrl : public Field {
     void    change_field_value(wxEvent& event);
 #endif //__WXGTK__
 
-#ifdef __WXOSX__
-	bool	bKilledFocus = false;
-#endif // __WXOSX__
-
 public:
 	TextCtrl(const ConfigOptionDef& opt, const t_config_option_key& id) : Field(opt,  id) {}
 	TextCtrl(wxWindow* parent, const ConfigOptionDef& opt, const t_config_option_key& id) : Field(parent, opt, id) {}
@@ -290,18 +251,18 @@ public:
     void propagate_value();
     wxWindow* window {nullptr};
 
-    virtual void	set_value(const std::string& value, bool change_event = false) {
+    void	set_value(const std::string& value, bool change_event = false) {
 		m_disable_change_event = !change_event;
         dynamic_cast<wxTextCtrl*>(window)->SetValue(wxString(value));
 		m_disable_change_event = false;
     }
-	virtual void	set_value(const boost::any& value, bool change_event = false) override;
-    virtual void    set_last_meaningful_value() override;
-    virtual void	set_na_value() override;
+	void	set_value(const boost::any& value, bool change_event = false) override;
+    void    set_last_meaningful_value() override;
+    void	set_na_value() override;
 
 	boost::any&		get_value() override;
 
-    void            msw_rescale(bool rescale_sidetext = false) override;
+    void            msw_rescale() override;
     
     void			enable() override;
     void			disable() override;
@@ -329,7 +290,7 @@ public:
 	void            set_na_value() override;
 	boost::any&		get_value() override;
 
-    void            msw_rescale(bool rescale_sidetext = false) override;
+    void            msw_rescale() override;
 
 	void			enable() override { dynamic_cast<wxCheckBox*>(window)->Enable(); }
 	void			disable() override { dynamic_cast<wxCheckBox*>(window)->Disable(); }
@@ -354,12 +315,12 @@ public:
     /// Propagate value from field to the OptionGroupe and Config after kill_focus/ENTER
     void	        propagate_value() ;
 
-	void			set_value(const std::string& value, bool change_event = false) {
+    void			set_value(const std::string& value, bool change_event = false) {
 		m_disable_change_event = !change_event;
 		dynamic_cast<wxSpinCtrl*>(window)->SetValue(value);
 		m_disable_change_event = false;
-	}
-	void			set_value(const boost::any& value, bool change_event = false) {
+    }
+    void			set_value(const boost::any& value, bool change_event = false) override {
 		m_disable_change_event = !change_event;
 		tmp_value = boost::any_cast<int>(value);
         m_value = value;
@@ -372,7 +333,7 @@ public:
 		return m_value = value;
 	}
 
-    void            msw_rescale(bool rescale_sidetext = false) override;
+    void            msw_rescale() override;
 
 	void			enable() override { dynamic_cast<wxSpinCtrl*>(window)->Enable(); }
 	void			disable() override { dynamic_cast<wxSpinCtrl*>(window)->Disable(); }
@@ -381,6 +342,7 @@ public:
 
 class Choice : public Field {
 	using Field::Field;
+
 public:
 	Choice(const ConfigOptionDef& opt, const t_config_option_key& id) : Field(opt, id) {}
 	Choice(wxWindow* parent, const ConfigOptionDef& opt, const t_config_option_key& id) : Field(parent, opt, id) {}
@@ -388,23 +350,31 @@ public:
 
 	wxWindow*		window{ nullptr };
 	void			BUILD() override;
+	// Propagate value from field to the OptionGroupe and Config after kill_focus/ENTER
+	void			propagate_value();
 
     /* Under OSX: wxBitmapComboBox->GetWindowStyle() returns some weard value, 
      * so let use a flag, which has TRUE value for a control without wxCB_READONLY style
      */
-    bool            m_is_editable { false };
+    bool            m_is_editable     { false };
+    bool            m_is_dropped      { false };
+    bool            m_suppress_scroll { false };
+    int             m_last_selected   { wxNOT_FOUND };
 
 	void			set_selection();
 	void			set_value(const std::string& value, bool change_event = false);
-	void			set_value(const boost::any& value, bool change_event = false);
+	void			set_value(const boost::any& value, bool change_event = false) override;
 	void			set_values(const std::vector<std::string> &values);
+	void			set_values(const wxArrayString &values);
 	boost::any&		get_value() override;
 
-    void            msw_rescale(bool rescale_sidetext = false) override;
+    void            msw_rescale() override;
 
-	void			enable() override { dynamic_cast<wxBitmapComboBox*>(window)->Enable(); };
-	void			disable() override{ dynamic_cast<wxBitmapComboBox*>(window)->Disable(); };
+	void			enable() override ;//{ dynamic_cast<wxBitmapComboBox*>(window)->Enable(); };
+	void			disable() override;//{ dynamic_cast<wxBitmapComboBox*>(window)->Disable(); };
 	wxWindow*		getWindow() override { return window; }
+
+    void            suppress_scroll();
 };
 
 class ColourPicker : public Field {
@@ -426,10 +396,11 @@ public:
 	 	}
 	void			set_value(const boost::any& value, bool change_event = false) override;
 	boost::any&		get_value() override;
-    void            msw_rescale(bool rescale_sidetext = false) override;
+    void            msw_rescale() override;
+    void            sys_color_changed() override;
 
-	void			enable() override { dynamic_cast<wxColourPickerCtrl*>(window)->Enable(); };
-	void			disable() override{ dynamic_cast<wxColourPickerCtrl*>(window)->Disable(); };
+    void			enable() override { dynamic_cast<wxColourPickerCtrl*>(window)->Enable(); }
+    void			disable() override{ dynamic_cast<wxColourPickerCtrl*>(window)->Disable(); }
 	wxWindow*		getWindow() override { return window; }
 };
 
@@ -445,13 +416,15 @@ public:
 	wxTextCtrl*		y_textctrl{ nullptr };
 
 	void			BUILD()  override;
+	bool			value_was_changed(wxTextCtrl* win);
     // Propagate value from field to the OptionGroupe and Config after kill_focus/ENTER
     void            propagate_value(wxTextCtrl* win);
 	void			set_value(const Vec2d& value, bool change_event = false);
-	void			set_value(const boost::any& value, bool change_event = false);
+	void			set_value(const boost::any& value, bool change_event = false) override;
 	boost::any&		get_value() override;
 
-    void            msw_rescale(bool rescale_sidetext = false) override;
+    void            msw_rescale() override;
+	void            sys_color_changed() override;
 
 	void			enable() override {
 		x_textctrl->Enable();
@@ -460,6 +433,7 @@ public:
 		x_textctrl->Disable();
 		y_textctrl->Disable(); }
 	wxSizer*		getSizer() override { return sizer; }
+	wxWindow*		getWindow() override { return dynamic_cast<wxWindow*>(x_textctrl); }
 };
 
 class StaticText : public Field {
@@ -477,7 +451,7 @@ public:
 		dynamic_cast<wxStaticText*>(window)->SetLabel(wxString::FromUTF8(value.data()));
 		m_disable_change_event = false;
 	}
-	void			set_value(const boost::any& value, bool change_event = false) {
+	void			set_value(const boost::any& value, bool change_event = false) override {
 		m_disable_change_event = !change_event;
 		dynamic_cast<wxStaticText*>(window)->SetLabel(boost::any_cast<wxString>(value));
 		m_disable_change_event = false;
@@ -485,10 +459,10 @@ public:
 
 	boost::any&		get_value()override { return m_value; }
 
-    void            msw_rescale(bool rescale_sidetext = false) override;
+    void            msw_rescale() override;
 
-	void			enable() override { dynamic_cast<wxStaticText*>(window)->Enable(); };
-	void			disable() override{ dynamic_cast<wxStaticText*>(window)->Disable(); };
+    void			enable() override { dynamic_cast<wxStaticText*>(window)->Enable(); }
+    void			disable() override{ dynamic_cast<wxStaticText*>(window)->Disable(); }
 	wxWindow*		getWindow() override { return window; }
 };
 
@@ -508,7 +482,7 @@ public:
 	void			BUILD()  override;
 
 	void			set_value(const int value, bool change_event = false);
-	void			set_value(const boost::any& value, bool change_event = false);
+	void			set_value(const boost::any& value, bool change_event = false) override;
 	boost::any&		get_value() override;
 
 	void			enable() override {
