@@ -15,6 +15,7 @@
 #include "MeshUtils.hpp"
 #include "libslic3r/GCode/GCodeProcessor.hpp"
 #include "GCodeViewer.hpp"
+#include "Camera.hpp"
 
 #include "libslic3r/Slicing.hpp"
 
@@ -37,9 +38,10 @@ class wxGLContext;
 
 namespace Slic3r {
 
-struct Camera;
 class BackgroundSlicingProcess;
+class BuildVolume;
 struct ThumbnailData;
+struct ThumbnailsParams;
 class ModelObject;
 class ModelInstance;
 class PrintObject;
@@ -49,6 +51,8 @@ namespace CustomGCode { struct Item; }
 
 namespace GUI {
 
+class Bed3D;
+
 #if ENABLE_RETINA_GL
 class RetinaHelper;
 #endif
@@ -92,6 +96,43 @@ private:
     wxTimer* m_timer;
 };
 
+class  ToolbarHighlighterTimerEvent : public wxEvent
+{
+public:
+    ToolbarHighlighterTimerEvent(wxEventType type, wxTimer& timer)
+        : wxEvent(timer.GetId(), type),
+        m_timer(&timer)
+    {
+        SetEventObject(timer.GetOwner());
+    }
+    int GetInterval() const { return m_timer->GetInterval(); }
+    wxTimer& GetTimer() const { return *m_timer; }
+
+    virtual wxEvent* Clone() const { return new ToolbarHighlighterTimerEvent(*this); }
+    virtual wxEventCategory GetEventCategory() const { return wxEVT_CATEGORY_TIMER; }
+private:
+    wxTimer* m_timer;
+};
+
+
+class  GizmoHighlighterTimerEvent : public wxEvent
+{
+public:
+    GizmoHighlighterTimerEvent(wxEventType type, wxTimer& timer)
+        : wxEvent(timer.GetId(), type),
+        m_timer(&timer)
+    {
+        SetEventObject(timer.GetOwner());
+    }
+    int GetInterval() const { return m_timer->GetInterval(); }
+    wxTimer& GetTimer() const { return *m_timer; }
+
+    virtual wxEvent* Clone() const { return new GizmoHighlighterTimerEvent(*this); }
+    virtual wxEventCategory GetEventCategory() const { return wxEVT_CATEGORY_TIMER; }
+private:
+    wxTimer* m_timer;
+};
+
 
 wxDECLARE_EVENT(EVT_GLCANVAS_OBJECT_SELECT, SimpleEvent);
 
@@ -120,15 +161,12 @@ wxDECLARE_EVENT(EVT_GLCANVAS_INSTANCE_SCALED, SimpleEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_WIPETOWER_ROTATED, Vec3dEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_ENABLE_ACTION_BUTTONS, Event<bool>);
 wxDECLARE_EVENT(EVT_GLCANVAS_UPDATE_GEOMETRY, Vec3dsEvent<2>);
+wxDECLARE_EVENT(EVT_GLCANVAS_MOUSE_DRAGGING_STARTED, SimpleEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_MOUSE_DRAGGING_FINISHED, SimpleEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_UPDATE_BED_SHAPE, SimpleEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_TAB, SimpleEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_RESETGIZMOS, SimpleEvent);
-#if ENABLE_ARROW_KEYS_WITH_SLIDERS
 wxDECLARE_EVENT(EVT_GLCANVAS_MOVE_SLIDERS, wxKeyEvent);
-#else
-wxDECLARE_EVENT(EVT_GLCANVAS_MOVE_LAYERS_SLIDER, wxKeyEvent);
-#endif // ENABLE_ARROW_KEYS_WITH_SLIDERS
 wxDECLARE_EVENT(EVT_GLCANVAS_EDIT_COLOR_CHANGE, wxKeyEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_JUMP_TO, wxKeyEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_UNDO, SimpleEvent);
@@ -139,6 +177,8 @@ wxDECLARE_EVENT(EVT_GLCANVAS_ADAPTIVE_LAYER_HEIGHT_PROFILE, Event<float>);
 wxDECLARE_EVENT(EVT_GLCANVAS_SMOOTH_LAYER_HEIGHT_PROFILE, HeightProfileSmoothEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_RELOAD_FROM_DISK, SimpleEvent);
 wxDECLARE_EVENT(EVT_GLCANVAS_RENDER_TIMER, wxTimerEvent/*RenderTimerEvent*/);
+wxDECLARE_EVENT(EVT_GLCANVAS_TOOLBAR_HIGHLIGHTER_TIMER, wxTimerEvent);
+wxDECLARE_EVENT(EVT_GLCANVAS_GIZMO_HIGHLIGHTER_TIMER, wxTimerEvent);
 
 class GLCanvas3D
 {
@@ -158,53 +198,50 @@ class GLCanvas3D
         static const float THICKNESS_BAR_WIDTH;
     private:
 
-        bool                        m_enabled;
-        unsigned int                m_z_texture_id;
+        bool                        m_enabled{ false };
+        unsigned int                m_z_texture_id{ 0 };
         // Not owned by LayersEditing.
-        const DynamicPrintConfig   *m_config;
+        const DynamicPrintConfig   *m_config{ nullptr };
         // ModelObject for the currently selected object (Model::objects[last_object_id]).
-        const ModelObject          *m_model_object;
+        const ModelObject          *m_model_object{ nullptr };
         // Maximum z of the currently selected object (Model::objects[last_object_id]).
-        float                       m_object_max_z;
+        float                       m_object_max_z{ 0.0f };
         // Owned by LayersEditing.
-        SlicingParameters          *m_slicing_parameters;
+        SlicingParameters           *m_slicing_parameters{ nullptr };
         std::vector<double>         m_layer_height_profile;
-        bool                        m_layer_height_profile_modified;
+        bool                        m_layer_height_profile_modified{ false };
 
-        mutable float               m_adaptive_quality;
+        mutable float               m_adaptive_quality{ 0.5f };
         mutable HeightProfileSmoothingParams m_smooth_params;
         
-        static float                s_overelay_window_width;
+        static float                s_overlay_window_width;
 
-        class LayersTexture
+        struct LayersTexture
         {
-        public:
-            LayersTexture() : width(0), height(0), levels(0), cells(0), valid(false) {}
-
             // Texture data
             std::vector<char>   data;
             // Width of the texture, top level.
-            size_t              width;
+            size_t              width{ 0 };
             // Height of the texture, top level.
-            size_t              height;
+            size_t              height{ 0 };
             // For how many levels of detail is the data allocated?
-            size_t              levels;
+            size_t              levels{ 0 };
             // Number of texture cells allocated for the height texture.
-            size_t              cells;
+            size_t              cells{ 0 };
             // Does it need to be refreshed?
-            bool                valid;
+            bool                valid{ false };
         };
         LayersTexture   m_layers_texture;
 
     public:
-        EState state;
-        float band_width;
-        float strength;
-        int last_object_id;
-        float last_z;
-        LayerHeightEditActionType last_action;
+        EState state{ Unknown };
+        float band_width{ 2.0f };
+        float strength{ 0.005f };
+        int last_object_id{ -1 };
+        float last_z{ 0.0f };
+        LayerHeightEditActionType last_action{ LAYER_HEIGHT_EDIT_ACTION_INCREASE };
 
-        LayersEditing();
+        LayersEditing() = default;
         ~LayersEditing();
 
         void init();
@@ -218,7 +255,7 @@ class GLCanvas3D
         void set_enabled(bool enabled);
 
         void render_overlay(const GLCanvas3D& canvas) const;
-        void render_volumes(const GLCanvas3D& canvas, const GLVolumeCollection& volumes) const;
+        void render_volumes(const GLCanvas3D& canvas, const GLVolumeCollection& volumes);
 
 		void adjust_layer_height_profile();
 		void accept_changes(GLCanvas3D& canvas);
@@ -230,7 +267,7 @@ class GLCanvas3D
         static bool bar_rect_contains(const GLCanvas3D& canvas, float x, float y);
         static Rect get_bar_rect_screen(const GLCanvas3D& canvas);
         static Rect get_bar_rect_viewport(const GLCanvas3D& canvas);
-        static float get_overlay_window_width() { return LayersEditing::s_overelay_window_width; }
+        static float get_overlay_window_width() { return LayersEditing::s_overlay_window_width; }
 
         float object_max_z() const { return m_object_max_z; }
 
@@ -302,64 +339,35 @@ class GLCanvas3D
         bool matches(double z) const { return this->z == z; }
     };
 
-    class WarningTexture : public GUI::GLTexture
-    {
-    public:
-        WarningTexture();
-
-        enum Warning {
-            ObjectOutside,
-            ToolpathOutside,
-            SlaSupportsOutside,
-            SomethingNotShown,
-            ObjectClashed
-        };
-
-        // Sets a warning of the given type to be active/inactive. If several warnings are active simultaneously,
-        // only the last one is shown (decided by the order in the enum above).
-        void activate(WarningTexture::Warning warning, bool state, const GLCanvas3D& canvas);
-        void render(const GLCanvas3D& canvas) const;
-
-        // function used to get an information for rescaling of the warning
-        void msw_rescale(const GLCanvas3D& canvas);
-
-    private:
-        static const unsigned char Background_Color[3];
-        static const unsigned char Opacity;
-
-        int m_original_width;
-        int m_original_height;
-
-        // information for rescaling of the warning legend
-        std::string     m_msg_text = "";
-        bool            m_is_colored_red{false};
-
-        // Information about which warnings are currently active.
-        std::vector<Warning> m_warnings;
-
-        // Generates the texture with given text.
-        bool generate(const std::string& msg, const GLCanvas3D& canvas, bool compress, bool red_colored = false);
+    enum class EWarning {
+        ObjectOutside,
+        ToolpathOutside,
+        SlaSupportsOutside,
+        SomethingNotShown,
+        ObjectClashed
     };
 
-#if ENABLE_RENDER_STATISTICS
     class RenderStats
     {
-        std::queue<std::pair<int64_t, int64_t>> m_frames;
-        int64_t m_curr_total{ 0 };
-
+    private:
+        std::chrono::time_point<std::chrono::high_resolution_clock> m_measuring_start;
+        int m_fps_out = -1;
+        int m_fps_running = 0;
     public:
-        void add_frame(int64_t frame) {
-            int64_t now = GLCanvas3D::timestamp_now();
-            if (!m_frames.empty() && now - m_frames.front().first > 1000) {
-                m_curr_total -= m_frames.front().second;
-                m_frames.pop();
+        void increment_fps_counter() { ++m_fps_running; }
+        int get_fps() { return m_fps_out; }
+        int get_fps_and_reset_if_needed() {
+            auto cur_time = std::chrono::high_resolution_clock::now();
+            int elapsed_ms = std::chrono::duration_cast<std::chrono::milliseconds>(cur_time-m_measuring_start).count();
+            if (elapsed_ms > 1000  || m_fps_out == -1) {
+                m_measuring_start = cur_time;
+                m_fps_out = int (1000. * m_fps_running / elapsed_ms);
+                m_fps_running = 0;
             }
-            m_curr_total += frame;
-            m_frames.push({ now, frame });
+            return m_fps_out;
         }
-        int64_t get_average() const { return m_frames.empty() ? 0 : m_curr_total / m_frames.size(); }
+
     };
-#endif // ENABLE_RENDER_STATISTICS
 
     class Labels
     {
@@ -385,7 +393,7 @@ class GLCanvas3D
     public:
         bool is_empty() const { return m_text.empty(); }
         void set_text(const std::string& text);
-        void render(const Vec2d& mouse_position, GLCanvas3D& canvas) const;
+        void render(const Vec2d& mouse_position, GLCanvas3D& canvas);
         // Indicates that the mouse is inside an ImGUI dialog, therefore the tooltip should be suppressed.
         void set_in_imgui(bool b) { m_in_imgui = b; }
         bool is_in_imgui() const { return m_in_imgui; }
@@ -394,12 +402,9 @@ class GLCanvas3D
     class Slope
     {
         bool m_enabled{ false };
-        bool m_dialog_shown{ false };
-        GLCanvas3D& m_canvas;
         GLVolumeCollection& m_volumes;
-        static float s_window_width;
     public:
-        Slope(GLCanvas3D& canvas, GLVolumeCollection& volumes) : m_canvas(canvas), m_volumes(volumes) {}
+        Slope(GLVolumeCollection& volumes) : m_volumes(volumes) {}
 
         void enable(bool enable) { m_enabled = enable; }
         bool is_enabled() const { return m_enabled; }
@@ -408,7 +413,6 @@ class GLCanvas3D
         void set_normal_angle(float angle_in_deg) const {
             m_volumes.set_slope_normal_z(-::cos(Geometry::deg2rad(90.0f - angle_in_deg)));
         }
-        static float get_window_width() { return s_window_width; };
     };
 
     class RenderTimer : public wxTimer {
@@ -416,6 +420,16 @@ class GLCanvas3D
         virtual void Notify() override;
     };
 
+    class ToolbarHighlighterTimer : public wxTimer {
+    private:
+        virtual void Notify() override;
+    };
+
+    class GizmoHighlighterTimer : public wxTimer {
+    private:
+        virtual void Notify() override;
+    };
+
 public:
     enum ECursorType : unsigned char
     {
@@ -435,44 +449,44 @@ public:
 private:
     wxGLCanvas* m_canvas;
     wxGLContext* m_context;
+    Bed3D &m_bed;
 #if ENABLE_RETINA_GL
     std::unique_ptr<RetinaHelper> m_retina_helper;
 #endif
     bool m_in_render;
-    WarningTexture m_warning_texture;
     wxTimer m_timer;
     LayersEditing m_layers_editing;
     Mouse m_mouse;
-    mutable GLGizmosManager m_gizmos;
-    mutable GLToolbar m_main_toolbar;
-    mutable GLToolbar m_undoredo_toolbar;
-    ClippingPlane m_clipping_planes[2];
-    mutable ClippingPlane m_camera_clipping_plane;
+    GLGizmosManager m_gizmos;
+    GLToolbar m_main_toolbar;
+    GLToolbar m_undoredo_toolbar;
+    std::array<ClippingPlane, 2> m_clipping_planes;
+    ClippingPlane m_camera_clipping_plane;
     bool m_use_clipping_planes;
-    mutable SlaCap m_sla_caps[2];
+    SlaCap m_sla_caps[2];
     std::string m_sidebar_field;
     // when true renders an extra frame by not resetting m_dirty to false
     // see request_extra_frame()
-    bool m_extra_frame_requested; 
-    int  m_extra_frame_requested_delayed { std::numeric_limits<int>::max() };
+    bool m_extra_frame_requested;
     bool m_event_handlers_bound{ false };
 
-    mutable GLVolumeCollection m_volumes;
+    GLVolumeCollection m_volumes;
     GCodeViewer m_gcode_viewer;
 
     RenderTimer m_render_timer;
-    int64_t     m_render_timer_start;
 
     Selection m_selection;
     const DynamicPrintConfig* m_config;
     Model* m_model;
     BackgroundSlicingProcess *m_process;
+    bool m_requires_check_outside_state{ false };
+
+    std::array<unsigned int, 2> m_old_size{ 0, 0 };
 
     // Screen is only refreshed from the OnIdle handler if it is dirty.
     bool m_dirty;
     bool m_initialized;
     bool m_apply_zoom_to_volumes_filter;
-    mutable std::vector<int> m_hover_volume_idxs;
     bool m_picking_enabled;
     bool m_moving_enabled;
     bool m_dynamic_background_enabled;
@@ -481,6 +495,7 @@ private:
     bool m_tab_down;
     ECursorType m_cursor_type;
     GLSelectionRectangle m_rectangle_selection;
+    std::vector<int> m_hover_volume_idxs;
 
     // Following variable is obsolete and it should be safe to remove it.
     // I just don't want to do it now before a release (Lukas Matena 24.3.2019)
@@ -494,17 +509,15 @@ private:
     bool m_show_picking_texture;
 #endif // ENABLE_RENDER_PICKING_PASS
 
-#if ENABLE_RENDER_STATISTICS
     RenderStats m_render_stats;
-#endif // ENABLE_RENDER_STATISTICS
 
-    mutable int m_imgui_undo_redo_hovered_pos{ -1 };
-    mutable int m_mouse_wheel {0};
+    int m_imgui_undo_redo_hovered_pos{ -1 };
+    int m_mouse_wheel{ 0 };
     int m_selected_extruder;
 
     Labels m_labels;
-    mutable Tooltip m_tooltip;
-    mutable bool m_tooltip_enabled{ true };
+    Tooltip m_tooltip;
+    bool m_tooltip_enabled{ true };
     Slope m_slope;
 
     ArrangeSettings m_arrange_settings_fff, m_arrange_settings_sla,
@@ -513,8 +526,7 @@ private:
     PrinterTechnology current_printer_technology() const;
 
     template<class Self>
-    static auto & get_arrange_settings(Self *self)
-    {
+    static auto & get_arrange_settings(Self *self) {
         PrinterTechnology ptech = self->current_printer_technology();
 
         auto *ptr = &self->m_arrange_settings_fff;
@@ -523,11 +535,10 @@ private:
             ptr = &self->m_arrange_settings_sla;
         } else if (ptech == ptFFF) {
             auto co_opt = self->m_config->template option<ConfigOptionBool>("complete_objects");
-            if (co_opt && co_opt->value) {
+            if (co_opt && co_opt->value)
                 ptr = &self->m_arrange_settings_fff_seq_print;
-            } else {
+            else
                 ptr = &self->m_arrange_settings_fff;
-            }
         }
 
         return *ptr;
@@ -537,8 +548,61 @@ private:
 
     void load_arrange_settings();
 
+    class SequentialPrintClearance
+    {
+        GLModel m_fill;
+        GLModel m_perimeter;
+        bool m_render_fill{ true };
+        bool m_visible{ false };
+
+        std::vector<Pointf3s> m_hull_2d_cache;
+
+    public:
+        void set_polygons(const Polygons& polygons);
+        void set_render_fill(bool render_fill) { m_render_fill = render_fill; }
+        void set_visible(bool visible) { m_visible = visible; }
+        void render();
+
+        friend class GLCanvas3D;
+    };
+
+    SequentialPrintClearance m_sequential_print_clearance;
+    bool m_sequential_print_clearance_first_displacement{ true };
+
+    struct ToolbarHighlighter
+    {
+        void set_timer_owner(wxEvtHandler* owner, int timerid = wxID_ANY);
+        void init(GLToolbarItem* toolbar_item, GLCanvas3D* canvas);
+        void blink();
+        void invalidate();
+        bool                    m_render_arrow{ false };
+        GLToolbarItem*          m_toolbar_item{ nullptr };
+    private:
+        GLCanvas3D*             m_canvas{ nullptr };
+        int				        m_blink_counter{ 0 };
+        ToolbarHighlighterTimer m_timer;       
+    }
+    m_toolbar_highlighter;
+
+    struct GizmoHighlighter
+    {
+        void set_timer_owner(wxEvtHandler* owner, int timerid = wxID_ANY);
+        void init(GLGizmosManager* manager, GLGizmosManager::EType gizmo, GLCanvas3D* canvas);
+        void blink();
+        void invalidate();
+        bool                    m_render_arrow{ false };
+        GLGizmosManager::EType  m_gizmo_type;
+    private:
+        GLGizmosManager*        m_gizmo_manager{ nullptr };
+        GLCanvas3D*             m_canvas{ nullptr };
+        int				        m_blink_counter{ 0 };
+        GizmoHighlighterTimer   m_timer;
+
+    }
+    m_gizmo_highlighter;
+
 public:
-    explicit GLCanvas3D(wxGLCanvas* canvas);
+    explicit GLCanvas3D(wxGLCanvas* canvas, Bed3D &bed);
     ~GLCanvas3D();
 
     bool is_initialized() const { return m_initialized; }
@@ -552,20 +616,22 @@ public:
     void post_event(wxEvent &&event);
 
     void set_as_dirty();
+    void requires_check_outside_state() { m_requires_check_outside_state = true; }
 
     unsigned int get_volumes_count() const;
     const GLVolumeCollection& get_volumes() const { return m_volumes; }
     void reset_volumes();
-    int check_volumes_outside_state() const;
+    ModelInstanceEPrintVolumeState check_volumes_outside_state() const;
 
+    void init_gcode_viewer() { m_gcode_viewer.init(); }
     void reset_gcode_toolpaths() { m_gcode_viewer.reset(); }
     const GCodeViewer::SequentialView& get_gcode_sequential_view() const { return m_gcode_viewer.get_sequential_view(); }
     void update_gcode_sequential_view_current(unsigned int first, unsigned int last) { m_gcode_viewer.update_sequential_view_current(first, last); }
 
     void toggle_sla_auxiliaries_visibility(bool visible, const ModelObject* mo = nullptr, int instance_idx = -1);
-    void toggle_model_objects_visibility(bool visible, const ModelObject* mo = nullptr, int instance_idx = -1);
+    void toggle_model_objects_visibility(bool visible, const ModelObject* mo = nullptr, int instance_idx = -1, const ModelVolume* mv = nullptr);
     void update_instance_printable_state_for_object(size_t obj_idx);
-    void update_instance_printable_state_for_objects(std::vector<size_t>& object_idxs);
+    void update_instance_printable_state_for_objects(const std::vector<size_t>& object_idxs);
 
     void set_config(const DynamicPrintConfig* config);
     void set_process(BackgroundSlicingProcess* process);
@@ -580,10 +646,8 @@ public:
 
     void bed_shape_changed();
 
-    void set_clipping_plane(unsigned int id, const ClippingPlane& plane)
-    {
-        if (id < 2)
-        {
+    void set_clipping_plane(unsigned int id, const ClippingPlane& plane) {
+        if (id < 2) {
             m_clipping_planes[id] = plane;
             m_sla_caps[id].reset();
         }
@@ -591,6 +655,9 @@ public:
     void reset_clipping_planes_cache() { m_sla_caps[0].triangles.clear(); m_sla_caps[1].triangles.clear(); }
     void set_use_clipping_planes(bool use) { m_use_clipping_planes = use; }
 
+    bool                                get_use_clipping_planes() const { return m_use_clipping_planes; }
+    const std::array<ClippingPlane, 2> &get_clipping_planes() const { return m_clipping_planes; };
+
     void set_color_by(const std::string& value);
 
     void refresh_camera_scene_box();
@@ -634,12 +701,13 @@ public:
     void render();
     // printable_only == false -> render also non printable volumes as grayed
     // parts_only == false -> render also sla support and pad
-    void render_thumbnail(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, bool printable_only, bool parts_only, bool show_bed, bool transparent_background) const;
+    void render_thumbnail(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, const ThumbnailsParams& thumbnail_params, Camera::EType camera_type);
+    void render_thumbnail(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, const ThumbnailsParams& thumbnail_params, const GLVolumeCollection& volumes, Camera::EType camera_type);
 
     void select_all();
     void deselect_all();
     void delete_selected();
-    void ensure_on_bed(unsigned int object_idx);
+    void ensure_on_bed(unsigned int object_idx, bool allow_negative_z);
 
     bool is_gcode_legend_enabled() const { return m_gcode_viewer.is_legend_enabled(); }
     GCodeViewer::EViewType get_gcode_view_type() const { return m_gcode_viewer.get_view_type(); }
@@ -652,6 +720,8 @@ public:
     void set_toolpath_view_type(GCodeViewer::EViewType type);
     void set_volumes_z_range(const std::array<double, 2>& range);
     void set_toolpaths_z_range(const std::array<unsigned int, 2>& range);
+    std::vector<CustomGCode::Item>& get_custom_gcode_per_print_z() { return m_gcode_viewer.get_custom_gcode_per_print_z(); }
+    size_t get_gcode_extruders_count() { return m_gcode_viewer.get_extruders_count(); }
 
     std::vector<int> load_object(const ModelObject& model_object, int obj_idx, std::vector<int> instance_idxs);
     std::vector<int> load_object(const Model& model, int obj_idx);
@@ -660,11 +730,8 @@ public:
 
     void reload_scene(bool refresh_immediately, bool force_full_scene_refresh = false);
 
-    void load_gcode_preview(const GCodeProcessor::Result& gcode_result);
-    void refresh_gcode_preview(const GCodeProcessor::Result& gcode_result, const std::vector<std::string>& str_tool_colors);
-#if ENABLE_RENDER_PATH_REFRESH_AFTER_OPTIONS_CHANGE
+    void load_gcode_preview(const GCodeProcessorResult& gcode_result, const std::vector<std::string>& str_tool_colors);
     void refresh_gcode_preview_render_paths();
-#endif // ENABLE_RENDER_PATH_REFRESH_AFTER_OPTIONS_CHANGE
     void set_gcode_view_preview_type(GCodeViewer::EViewType type) { return m_gcode_viewer.set_view_type(type); }
     GCodeViewer::EViewType get_gcode_view_preview_type() const { return m_gcode_viewer.get_view_type(); }
     void load_sla_preview();
@@ -686,7 +753,7 @@ public:
     Size get_canvas_size() const;
     Vec2d get_local_mouse_position() const;
 
-    void set_tooltip(const std::string& tooltip) const;
+    void set_tooltip(const std::string& tooltip);
 
     // the following methods add a snapshot to the undo/redo stack, unless the given string is empty
     void do_move(const std::string& snapshot_type);
@@ -713,10 +780,9 @@ public:
         double m_rotation = 0.;
         BoundingBoxf m_bb;
         friend class GLCanvas3D;
-    public:
-        
-        inline operator bool() const
-        {
+
+    public:        
+        inline operator bool() const {
             return !std::isnan(m_pos.x()) && !std::isnan(m_pos.y());
         }
         
@@ -741,8 +807,10 @@ public:
     void msw_rescale();
 
     void request_extra_frame() { m_extra_frame_requested = true; }
-    void request_extra_frame_delayed(int miliseconds);
+    
+    void schedule_extra_frame(int miliseconds);
 
+    float get_main_toolbar_height() { return m_main_toolbar.get_height(); }
     int get_main_toolbar_item_id(const std::string& name) const { return m_main_toolbar.get_item_id(name); }
     void force_main_toolbar_left_action(int item_id) { m_main_toolbar.force_left_action(item_id, *this); }
     void force_main_toolbar_right_action(int item_id) { m_main_toolbar.force_right_action(item_id, *this); }
@@ -760,8 +828,10 @@ public:
     void use_slope(bool use) { m_slope.use(use); }
     void set_slope_normal_angle(float angle_in_deg) { m_slope.set_normal_angle(angle_in_deg); }
 
-    ArrangeSettings get_arrange_settings() const
-    {
+    void highlight_toolbar_item(const std::string& item_name);
+    void highlight_gizmo(const std::string& gizmo_name);
+
+    ArrangeSettings get_arrange_settings() const {
         const ArrangeSettings &settings = get_arrange_settings(this);
         ArrangeSettings ret = settings;
         if (&settings == &m_arrange_settings_fff_seq_print) {
@@ -783,6 +853,33 @@ public:
 #endif
     }
 
+    void reset_sequential_print_clearance() {
+        m_sequential_print_clearance.set_visible(false);
+        m_sequential_print_clearance.set_render_fill(false);
+        m_sequential_print_clearance.set_polygons(Polygons());
+    }
+
+    void set_sequential_print_clearance_visible(bool visible) {
+        m_sequential_print_clearance.set_visible(visible);
+    }
+
+    void set_sequential_print_clearance_render_fill(bool render_fill) {
+        m_sequential_print_clearance.set_render_fill(render_fill);
+    }
+
+    void set_sequential_print_clearance_polygons(const Polygons& polygons) {
+        m_sequential_print_clearance.set_polygons(polygons);
+    }
+
+    void update_sequential_clearance();
+
+    const Print* fff_print() const;
+    const SLAPrint* sla_print() const;
+
+    void reset_old_size() { m_old_size = { 0, 0 }; }
+
+    bool is_object_sinking(int object_idx) const;
+
 private:
     bool _is_shown_on_screen() const;
 
@@ -802,43 +899,44 @@ private:
 
     void _refresh_if_shown_on_screen();
 
-    void _picking_pass() const;
-    void _rectangular_selection_picking_pass() const;
+    void _picking_pass();
+    void _rectangular_selection_picking_pass();
     void _render_background() const;
-    void _render_bed(bool bottom, bool show_axes) const;
-    void _render_objects() const;
-    void _render_gcode() const;
+    void _render_bed(bool bottom, bool show_axes);
+    void _render_bed_for_picking(bool bottom);
+    void _render_objects(GLVolumeCollection::ERenderType type);
+    void _render_gcode();
     void _render_selection() const;
+    void _render_sequential_clearance();
 #if ENABLE_RENDER_SELECTION_CENTER
     void _render_selection_center() const;
 #endif // ENABLE_RENDER_SELECTION_CENTER
-    void _check_and_update_toolbar_icon_scale() const;
-    void _render_overlays() const;
-    void _render_warning_texture() const;
+    void _check_and_update_toolbar_icon_scale();
+    void _render_overlays();
     void _render_volumes_for_picking() const;
     void _render_current_gizmo() const;
-    void _render_gizmos_overlay() const;
-    void _render_main_toolbar() const;
-    void _render_undoredo_toolbar() const;
+    void _render_gizmos_overlay();
+    void _render_main_toolbar();
+    void _render_undoredo_toolbar();
     void _render_collapse_toolbar() const;
     void _render_view_toolbar() const;
 #if ENABLE_SHOW_CAMERA_TARGET
     void _render_camera_target() const;
 #endif // ENABLE_SHOW_CAMERA_TARGET
-    void _render_sla_slices() const;
+    void _render_sla_slices();
     void _render_selection_sidebar_hints() const;
-    bool _render_undo_redo_stack(const bool is_undo, float pos_x) const;
-    bool _render_search_list(float pos_x) const;
+    bool _render_undo_redo_stack(const bool is_undo, float pos_x);
+    bool _render_search_list(float pos_x);
     bool _render_arrange_menu(float pos_x);
-    void _render_thumbnail_internal(ThumbnailData& thumbnail_data, bool printable_only, bool parts_only, bool show_bed, bool transparent_background) const;
+    void _render_thumbnail_internal(ThumbnailData& thumbnail_data, const ThumbnailsParams& thumbnail_params, const GLVolumeCollection& volumes, Camera::EType camera_type);
     // render thumbnail using an off-screen framebuffer
-    void _render_thumbnail_framebuffer(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, bool printable_only, bool parts_only, bool show_bed, bool transparent_background) const;
+    void _render_thumbnail_framebuffer(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, const ThumbnailsParams& thumbnail_params, const GLVolumeCollection& volumes, Camera::EType camera_type);
     // render thumbnail using an off-screen framebuffer when GLEW_EXT_framebuffer_object is supported
-    void _render_thumbnail_framebuffer_ext(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, bool printable_only, bool parts_only, bool show_bed, bool transparent_background) const;
+    void _render_thumbnail_framebuffer_ext(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, const ThumbnailsParams& thumbnail_params, const GLVolumeCollection& volumes, Camera::EType camera_type);
     // render thumbnail using the default framebuffer
-    void _render_thumbnail_legacy(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, bool printable_only, bool parts_only, bool show_bed, bool transparent_background) const;
+    void _render_thumbnail_legacy(ThumbnailData& thumbnail_data, unsigned int w, unsigned int h, const ThumbnailsParams& thumbnail_params, const GLVolumeCollection& volumes, Camera::EType camera_type);
 
-    void _update_volumes_hover_state() const;
+    void _update_volumes_hover_state();
 
     void _perform_layer_editing_action(wxMouseEvent* evt = nullptr);
 
@@ -853,24 +951,23 @@ private:
     void _stop_timer();
 
     // Create 3D thick extrusion lines for a skirt and brim.
-    // Adds a new Slic3r::GUI::3DScene::Volume to volumes.
-    void _load_print_toolpaths();
+    // Adds a new Slic3r::GUI::3DScene::Volume to volumes, updates collision with the build_volume.
+    void _load_print_toolpaths(const BuildVolume &build_volume);
     // Create 3D thick extrusion lines for object forming extrusions.
     // Adds a new Slic3r::GUI::3DScene::Volume to $self->volumes,
-    // one for perimeters, one for infill and one for supports.
-    void _load_print_object_toolpaths(const PrintObject& print_object, const std::vector<std::string>& str_tool_colors,
-                                      const std::vector<CustomGCode::Item>& color_print_values);
-    // Create 3D thick extrusion lines for wipe tower extrusions
-    void _load_wipe_tower_toolpaths(const std::vector<std::string>& str_tool_colors);
+    // one for perimeters, one for infill and one for supports, updates collision with the build_volume.
+    void _load_print_object_toolpaths(const PrintObject& print_object, const BuildVolume &build_volume,
+        const std::vector<std::string>& str_tool_colors, const std::vector<CustomGCode::Item>& color_print_values);
+    // Create 3D thick extrusion lines for wipe tower extrusions, updates collision with the build_volume.
+    void _load_wipe_tower_toolpaths(const BuildVolume &build_volume, const std::vector<std::string>& str_tool_colors);
 
     // Load SLA objects and support structures for objects, for which the slaposSliceSupports step has been finished.
 	void _load_sla_shells();
-    void _update_toolpath_volumes_outside_state();
     void _update_sla_shells_outside_state();
-    void _show_warning_texture_if_needed(WarningTexture::Warning warning);
+    void _set_warning_notification_if_needed(EWarning warning);
 
-    // generates a warning texture containing the given message
-    void _set_warning_texture(WarningTexture::Warning warning, bool state);
+    // generates a warning notification containing the given message
+    void _set_warning_notification(EWarning warning, bool state);
 
     bool _is_any_volume_outside() const;
 
@@ -885,11 +982,7 @@ private:
 
     float get_overlay_window_width() { return LayersEditing::get_overlay_window_width(); }
 
-    static std::vector<float> _parse_colors(const std::vector<std::string>& colors);
-
-public:
-    const Print* fff_print() const;
-    const SLAPrint* sla_print() const;
+    static std::vector<std::array<float, 4>> _parse_colors(const std::vector<std::string>& colors);
 };
 
 } // namespace GUI
