@@ -1,10 +1,14 @@
 #include "GUI.hpp"
 #include "GUI_App.hpp"
+#include "format.hpp"
 #include "I18N.hpp"
 
+#include "libslic3r/LocalesUtils.hpp"
+
 #include <string>
 
 #include <boost/algorithm/string.hpp>
+#include <boost/algorithm/string/predicate.hpp>
 #include <boost/any.hpp>
 
 #if __APPLE__
@@ -18,6 +22,7 @@
 
 #include "AboutDialog.hpp"
 #include "MsgDialog.hpp"
+#include "format.hpp"
 
 #include "libslic3r/Print.hpp"
 
@@ -35,8 +40,8 @@ void disable_screensaver()
 {
     #if __APPLE__
     CFStringRef reasonForActivity = CFSTR("Slic3r");
-    IOReturn success = IOPMAssertionCreateWithName(kIOPMAssertionTypeNoDisplaySleep, 
-        kIOPMAssertionLevelOn, reasonForActivity, &assertionID); 
+    [[maybe_unused]]IOReturn success = IOPMAssertionCreateWithName(kIOPMAssertionTypeNoDisplaySleep,
+        kIOPMAssertionLevelOn, reasonForActivity, &assertionID);
     // ignore result: success == kIOReturnSuccess
     #elif _WIN32
     SetThreadExecutionState(ES_DISPLAY_REQUIRED | ES_CONTINUOUS);
@@ -46,7 +51,7 @@ void disable_screensaver()
 void enable_screensaver()
 {
     #if __APPLE__
-    IOReturn success = IOPMAssertionRelease(assertionID);
+    IOPMAssertionRelease(assertionID);
     #elif _WIN32
     SetThreadExecutionState(ES_CONTINUOUS);
     #endif
@@ -71,7 +76,7 @@ void break_to_debugger()
 
 const std::string& shortkey_ctrl_prefix()
 {
-	static const std::string str = 
+	static const std::string str =
 #ifdef __APPLE__
 		"⌘"
 #else
@@ -83,7 +88,7 @@ const std::string& shortkey_ctrl_prefix()
 
 const std::string& shortkey_alt_prefix()
 {
-	static const std::string str = 
+	static const std::string str =
 #ifdef __APPLE__
 		"⌥"
 #else
@@ -104,7 +109,8 @@ void change_opt_value(DynamicPrintConfig& config, const t_config_option_key& opt
             return;
         }
 
-		switch (config.def()->get(opt_key)->type) {
+        const ConfigOptionDef *opt_def = config.def()->get(opt_key);
+		switch (opt_def->type) {
 		case coFloatOrPercent:{
 			std::string str = boost::any_cast<std::string>(value);
 			bool percent = false;
@@ -112,7 +118,7 @@ void change_opt_value(DynamicPrintConfig& config, const t_config_option_key& opt
 				str.pop_back();
 				percent = true;
 			}
-			double val = stod(str);
+            double val = std::stod(str); // locale-dependent (on purpose - the input is the actual content of the field)
 			config.set_key_value(opt_key, new ConfigOptionFloatOrPercent(val, percent));
 			break;}
 		case coPercent:
@@ -132,13 +138,13 @@ void change_opt_value(DynamicPrintConfig& config, const t_config_option_key& opt
 			ConfigOptionFloats* vec_new = new ConfigOptionFloats{ boost::any_cast<double>(value) };
 			config.option<ConfigOptionFloats>(opt_key)->set_at(vec_new, opt_index, opt_index);
  			break;
-		}			
+		}
 		case coString:
 			config.set_key_value(opt_key, new ConfigOptionString(boost::any_cast<std::string>(value)));
 			break;
 		case coStrings:{
 			if (opt_key == "compatible_prints" || opt_key == "compatible_printers") {
-				config.option<ConfigOptionStrings>(opt_key)->values = 
+				config.option<ConfigOptionStrings>(opt_key)->values =
 					boost::any_cast<std::vector<std::string>>(value);
 			}
 			else if (config.def()->get(opt_key)->gui_flags.compare("serialized") == 0) {
@@ -176,28 +182,9 @@ void change_opt_value(DynamicPrintConfig& config, const t_config_option_key& opt
 			}
 			break;
 		case coEnum:{
-			if (opt_key == "top_fill_pattern" ||
-				opt_key == "bottom_fill_pattern" ||
-				opt_key == "fill_pattern")
-				config.set_key_value(opt_key, new ConfigOptionEnum<InfillPattern>(boost::any_cast<InfillPattern>(value))); 
-			else if (opt_key.compare("ironing_type") == 0)
-				config.set_key_value(opt_key, new ConfigOptionEnum<IroningType>(boost::any_cast<IroningType>(value))); 
-			else if (opt_key.compare("gcode_flavor") == 0)
-				config.set_key_value(opt_key, new ConfigOptionEnum<GCodeFlavor>(boost::any_cast<GCodeFlavor>(value))); 
-			else if (opt_key.compare("machine_limits_usage") == 0)
-				config.set_key_value(opt_key, new ConfigOptionEnum<MachineLimitsUsage>(boost::any_cast<MachineLimitsUsage>(value))); 
-			else if (opt_key.compare("support_material_pattern") == 0)
-				config.set_key_value(opt_key, new ConfigOptionEnum<SupportMaterialPattern>(boost::any_cast<SupportMaterialPattern>(value)));
-			else if (opt_key.compare("seam_position") == 0)
-				config.set_key_value(opt_key, new ConfigOptionEnum<SeamPosition>(boost::any_cast<SeamPosition>(value)));
-			else if (opt_key.compare("host_type") == 0)
-				config.set_key_value(opt_key, new ConfigOptionEnum<PrintHostType>(boost::any_cast<PrintHostType>(value)));
-			else if (opt_key.compare("display_orientation") == 0)
-				config.set_key_value(opt_key, new ConfigOptionEnum<SLADisplayOrientation>(boost::any_cast<SLADisplayOrientation>(value)));
-            else if(opt_key.compare("support_pillar_connection_mode") == 0)
-                config.set_key_value(opt_key, new ConfigOptionEnum<SLAPillarConnectionMode>(boost::any_cast<SLAPillarConnectionMode>(value)));
-            else if(opt_key == "printhost_authorization_type")
-                config.set_key_value(opt_key, new ConfigOptionEnum<AuthorizationType>(boost::any_cast<AuthorizationType>(value)));
+			auto *opt = opt_def->default_value.get()->clone();
+			opt->setInt(boost::any_cast<int>(value));
+			config.set_key_value(opt_key, opt);
 			}
 			break;
 		case coPoints:{
@@ -215,9 +202,9 @@ void change_opt_value(DynamicPrintConfig& config, const t_config_option_key& opt
 			break;
 		}
 	}
-	catch (const std::exception & /* e */)
+	catch (const std::exception &e)
 	{
-		// int i = 0;//no reason, just experiment
+		wxLogError(format_wxstr("Internal error when changing value for %1%: %2%", opt_key, e.what()));
 	}
 }
 
@@ -241,7 +228,8 @@ void show_error_id(int id, const std::string& message)
 
 void show_info(wxWindow* parent, const wxString& message, const wxString& title)
 {
-	wxMessageDialog msg_wingow(parent, message, wxString(SLIC3R_APP_NAME " - ") + (title.empty() ? _L("Notice") : title), wxOK | wxICON_INFORMATION);
+	//wxMessageDialog msg_wingow(parent, message, wxString(SLIC3R_APP_NAME " - ") + (title.empty() ? _L("Notice") : title), wxOK | wxICON_INFORMATION);
+	MessageDialog msg_wingow(parent, message, wxString(SLIC3R_APP_NAME " - ") + (title.empty() ? _L("Notice") : title), wxOK | wxICON_INFORMATION);
 	msg_wingow.ShowModal();
 }
 
@@ -253,7 +241,129 @@ void show_info(wxWindow* parent, const char* message, const char* title)
 
 void warning_catcher(wxWindow* parent, const wxString& message)
 {
-	wxMessageDialog msg(parent, message, _L("Warning"), wxOK | wxICON_WARNING);
+	//wxMessageDialog msg(parent, message, _L("Warning"), wxOK | wxICON_WARNING);
+	MessageDialog msg(parent, message, _L("Warning"), wxOK | wxICON_WARNING);
+	msg.ShowModal();
+}
+
+static wxString bold(const wxString& str)
+{
+	return wxString::Format("<b>%s</b>", str);
+};
+
+static wxString bold_string(const wxString& str) 
+{ 
+	return wxString::Format("<b>\"%s\"</b>", str); 
+};
+
+static void add_config_substitutions(const ConfigSubstitutions& conf_substitutions, wxString& changes)
+{
+	changes += "<table>";
+	for (const ConfigSubstitution& conf_substitution : conf_substitutions) {
+		wxString new_val;
+		const ConfigOptionDef* def = conf_substitution.opt_def;
+		if (!def)
+			continue;
+		switch (def->type) {
+		case coEnum:
+		{
+			const std::vector<std::string>& labels = def->enum_labels;
+			const std::vector<std::string>& values = def->enum_values;
+			int val = conf_substitution.new_value->getInt();
+
+			bool is_infill = def->opt_key == "top_fill_pattern"	   ||
+							 def->opt_key == "bottom_fill_pattern" ||
+							 def->opt_key == "fill_pattern";
+
+			// Each infill doesn't use all list of infill declared in PrintConfig.hpp.
+			// So we should "convert" val to the correct one
+			if (is_infill) {
+				for (const auto& key_val : *def->enum_keys_map)
+					if ((int)key_val.second == val) {
+						auto it = std::find(values.begin(), values.end(), key_val.first);
+						if (it == values.end())
+							break;
+						auto idx = it - values.begin();
+						new_val = wxString("\"") + values[idx] + "\"" + " (" + from_u8(_utf8(labels[idx])) + ")";
+						break;
+					}
+				if (new_val.IsEmpty()) {
+					assert(false);
+					new_val = _L("Undefined");
+				}
+			}
+			else
+				new_val = wxString("\"") + values[val] + "\"" + " (" + from_u8(_utf8(labels[val])) + ")";
+			break;
+		}
+		case coBool:
+			new_val = conf_substitution.new_value->getBool() ? "true" : "false";
+			break;
+		case coBools:
+			if (conf_substitution.new_value->nullable())
+				for (const char v : static_cast<const ConfigOptionBoolsNullable*>(conf_substitution.new_value.get())->values)
+					new_val += std::string(v == ConfigOptionBoolsNullable::nil_value() ? "nil" : v ? "true" : "false") + ", ";
+			else
+				for (const char v : static_cast<const ConfigOptionBools*>(conf_substitution.new_value.get())->values)
+					new_val += std::string(v ? "true" : "false") + ", ";
+			if (! new_val.empty())
+				new_val.erase(new_val.begin() + new_val.size() - 2, new_val.end());
+			break;
+		default:
+			assert(false);
+		}
+
+		changes += format_wxstr("<tr><td><b>\"%1%\" (%2%)</b></td><td>: ", def->opt_key, _(def->label)) +
+				   format_wxstr(_L("%1% was substituted with %2%"), bold_string(conf_substitution.old_value), bold(new_val)) + 
+				   "</td></tr>";
+	}
+	changes += "</table>";
+}
+
+static wxString substitution_message(const wxString& changes)
+{
+	return
+		_L("Most likely the configuration was produced by a newer version of PrusaSlicer or by some PrusaSlicer fork.") + " " +
+		_L("The following values were substituted:") + "\n" + changes + "\n\n" +
+		_L("Review the substitutions and adjust them if needed.");
+}
+
+void show_substitutions_info(const PresetsConfigSubstitutions& presets_config_substitutions) 
+{
+	wxString changes;
+
+	auto preset_type_name = [](Preset::Type type) {
+		switch (type) {
+			case Preset::TYPE_PRINT:			return _L("Print settings");
+			case Preset::TYPE_SLA_PRINT:		return _L("SLA print settings");
+			case Preset::TYPE_FILAMENT:			return _L("Filament");
+			case Preset::TYPE_SLA_MATERIAL:		return _L("SLA material");
+			case Preset::TYPE_PRINTER: 			return _L("Printer");
+			case Preset::TYPE_PHYSICAL_PRINTER:	return _L("Physical Printer");
+			default: assert(false);				return wxString();
+		}
+	};
+
+	for (const PresetConfigSubstitutions& substitution : presets_config_substitutions) {
+		changes += "\n\n" + format_wxstr("%1% : %2%", preset_type_name(substitution.preset_type), bold_string(substitution.preset_name));
+		if (!substitution.preset_file.empty())
+			changes += format_wxstr(" (%1%)", substitution.preset_file);
+
+		add_config_substitutions(substitution.substitutions, changes);
+	}
+
+	InfoDialog msg(nullptr, _L("Configuration bundle was loaded, however some configuration values were not recognized."), substitution_message(changes), true);
+	msg.ShowModal();
+}
+
+void show_substitutions_info(const ConfigSubstitutions& config_substitutions, const std::string& filename)
+{
+	wxString changes = "\n";
+	add_config_substitutions(config_substitutions, changes);
+
+	InfoDialog msg(nullptr, 
+		format_wxstr(_L("Configuration file \"%1%\" was loaded, however some configuration values were not recognized."), from_u8(filename)), 
+		substitution_message(changes), true);
 	msg.ShowModal();
 }
 
@@ -261,6 +371,7 @@ void create_combochecklist(wxComboCtrl* comboCtrl, const std::string& text, cons
 {
     if (comboCtrl == nullptr)
         return;
+    wxGetApp().UpdateDarkUI(comboCtrl);
 
     wxCheckListBoxComboPopup* popup = new wxCheckListBoxComboPopup;
     if (popup != nullptr) {
@@ -272,6 +383,9 @@ void create_combochecklist(wxComboCtrl* comboCtrl, const std::string& text, cons
 
 		// the following line messes up the popup size the first time it is shown on wxWidgets 3.1.3
 //		comboCtrl->EnablePopupAnimation(false);
+#ifdef _WIN32
+		popup->SetFont(comboCtrl->GetFont());
+#endif // _WIN32
 		comboCtrl->SetPopupControl(popup);
 		wxString title = from_u8(text);
 		max_width = std::max(max_width, 60 + comboCtrl->GetTextExtent(title).x);
@@ -295,6 +409,7 @@ void create_combochecklist(wxComboCtrl* comboCtrl, const std::string& text, cons
 		}
 
 		comboCtrl->SetMinClientSize(wxSize(max_width, -1));
+        wxGetApp().UpdateDarkUI(popup);
 	}
 }
 
