@@ -7,10 +7,10 @@ namespace boost::filesystem { class path; }
 #include <wx/string.h>
 
 #include "libslic3r/Config.hpp"
+#include "libslic3r/Preset.hpp"
 
 class wxWindow;
 class wxMenuBar;
-class wxNotebook;
 class wxComboCtrl;
 class wxFileDialog;
 class wxTopLevelWindow;
@@ -49,6 +49,8 @@ void show_info(wxWindow* parent, const wxString& message, const wxString& title
 void show_info(wxWindow* parent, const char* message, const char* title = nullptr);
 inline void show_info(wxWindow* parent, const std::string& message,const std::string& title = std::string()) { show_info(parent, message.c_str(), title.c_str()); }
 void warning_catcher(wxWindow* parent, const wxString& message);
+void show_substitutions_info(const PresetsConfigSubstitutions& presets_config_substitutions);
+void show_substitutions_info(const ConfigSubstitutions& config_substitutions, const std::string& filename);
 
 // Creates a wxCheckListBoxComboPopup inside the given wxComboCtrl, filled with the given text and items.
 // Items data must be separated by '|', and contain the item name to be shown followed by its initial value (0 for false, 1 for true).
