@@ -1,6 +1,7 @@
 #ifndef slic3r_GUI_Init_hpp_
 #define slic3r_GUI_Init_hpp_
 
+#include <libslic3r/Preset.hpp>
 #include <libslic3r/PrintConfig.hpp>
 
 namespace Slic3r {
@@ -12,6 +13,9 @@ struct GUI_InitParams
 	int		                    argc;
 	char	                  **argv;
 
+	// Substitutions of unknown configuration values done during loading of user presets.
+	PresetsConfigSubstitutions  preset_substitutions;
+
     std::vector<std::string>    load_configs;
     DynamicPrintConfig          extra_config;
     std::vector<std::string>    input_files;
