@@ -1,4 +1,6 @@
+//#include "stdlib.h"
 #include "libslic3r/libslic3r.h"
+#include "libslic3r/Layer.hpp"
 #include "GUI_Preview.hpp"
 #include "GUI_App.hpp"
 #include "GUI.hpp"
@@ -11,39 +13,44 @@
 #include "DoubleSlider.hpp"
 #include "Plater.hpp"
 #include "MainFrame.hpp"
+#include "format.hpp"
 
+#include <wx/listbook.h>
 #include <wx/notebook.h>
 #include <wx/glcanvas.h>
 #include <wx/sizer.h>
 #include <wx/stattext.h>
 #include <wx/choice.h>
 #include <wx/combo.h>
+#include <wx/combobox.h>
 #include <wx/checkbox.h>
 
 // this include must follow the wxWidgets ones or it won't compile on Windows -> see http://trac.wxwidgets.org/ticket/2421
 #include "libslic3r/Print.hpp"
 #include "libslic3r/SLAPrint.hpp"
+#include "NotificationManager.hpp"
+
+#ifdef _WIN32
+#include "BitmapComboBox.hpp"
+#endif
 
 namespace Slic3r {
 namespace GUI {
 
-View3D::View3D(wxWindow* parent, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process)
+View3D::View3D(wxWindow* parent, Bed3D& bed, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process)
     : m_canvas_widget(nullptr)
     , m_canvas(nullptr)
 {
-    init(parent, model, config, process);
+    init(parent, bed, model, config, process);
 }
 
 View3D::~View3D()
 {
-    if (m_canvas != nullptr)
-        delete m_canvas;
-
-    if (m_canvas_widget != nullptr)
-        delete m_canvas_widget;
+    delete m_canvas;
+    delete m_canvas_widget;
 }
 
-bool View3D::init(wxWindow* parent, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process)
+bool View3D::init(wxWindow* parent, Bed3D& bed, Model* model, DynamicPrintConfig* config, BackgroundSlicingProcess* process)
 {
     if (!Create(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0 /* disable wxTAB_TRAVERSAL */))
         return false;
@@ -52,7 +59,7 @@ bool View3D::init(wxWindow* parent, Model* model, DynamicPrintConfig* config, Ba
     if (m_canvas_widget == nullptr)
         return false;
 
-    m_canvas = new GLCanvas3D(m_canvas_widget);
+    m_canvas = new GLCanvas3D(m_canvas_widget, bed);
     m_canvas->set_context(wxGetApp().init_glcontext(*m_canvas_widget));
 
     m_canvas->allow_multisample(OpenGLManager::can_multisample());
@@ -122,11 +129,6 @@ void View3D::mirror_selection(Axis axis)
         m_canvas->mirror_selection(axis);
 }
 
-int View3D::check_volumes_outside_state() const
-{
-    return (m_canvas != nullptr) ? m_canvas->check_volumes_outside_state() : false;
-}
-
 bool View3D::is_layers_editing_enabled() const
 {
     return (m_canvas != nullptr) ? m_canvas->is_layers_editing_enabled() : false;
@@ -167,28 +169,25 @@ void View3D::render()
 }
 
 Preview::Preview(
-    wxWindow* parent, Model* model, DynamicPrintConfig* config,
-    BackgroundSlicingProcess* process, GCodeProcessor::Result* gcode_result, std::function<void()> schedule_background_process_func)
+    wxWindow* parent, Bed3D& bed, Model* model, DynamicPrintConfig* config,
+    BackgroundSlicingProcess* process, GCodeProcessorResult* gcode_result, std::function<void()> schedule_background_process_func)
     : m_config(config)
     , m_process(process)
     , m_gcode_result(gcode_result)
-#if !ENABLE_PREVIEW_TYPE_CHANGE
-    , m_preferred_color_mode("feature")
-#endif // !ENABLE_PREVIEW_TYPE_CHANGE
     , m_schedule_background_process(schedule_background_process_func)
 {
-    if (init(parent, model))
+    if (init(parent, bed, model))
         load_print();
 }
 
-bool Preview::init(wxWindow* parent, Model* model)
+bool Preview::init(wxWindow* parent, Bed3D& bed, Model* model)
 {
     if (!Create(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0 /* disable wxTAB_TRAVERSAL */))
         return false;
 
     // to match the background of the sliders
 #ifdef _WIN32 
-    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
+    wxGetApp().UpdateDarkUI(this);
 #else
     SetBackgroundColour(GetParent()->GetBackgroundColour());
 #endif // _WIN32 
@@ -197,7 +196,7 @@ bool Preview::init(wxWindow* parent, Model* model)
     if (m_canvas_widget == nullptr)
         return false;
 
-    m_canvas = new GLCanvas3D(m_canvas_widget);
+    m_canvas = new GLCanvas3D(m_canvas_widget, bed);
     m_canvas->set_context(wxGetApp().init_glcontext(*m_canvas_widget));
     m_canvas->allow_multisample(OpenGLManager::can_multisample());
     m_canvas->set_config(m_config);
@@ -208,14 +207,19 @@ bool Preview::init(wxWindow* parent, Model* model)
 
     m_layers_slider_sizer = create_layers_slider_sizer();
 
-    m_bottom_toolbar_panel = new wxPanel(this);
+    wxGetApp().UpdateDarkUI(m_bottom_toolbar_panel = new wxPanel(this));
     m_label_view_type = new wxStaticText(m_bottom_toolbar_panel, wxID_ANY, _L("View"));
-    m_choice_view_type = new wxChoice(m_bottom_toolbar_panel, wxID_ANY);
+#ifdef _WIN32
+    wxGetApp().UpdateDarkUI(m_choice_view_type = new BitmapComboBox(m_bottom_toolbar_panel, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY));
+#else
+    m_choice_view_type = new wxComboBox(m_bottom_toolbar_panel, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY);
+#endif
     m_choice_view_type->Append(_L("Feature type"));
     m_choice_view_type->Append(_L("Height"));
     m_choice_view_type->Append(_L("Width"));
     m_choice_view_type->Append(_L("Speed"));
     m_choice_view_type->Append(_L("Fan speed"));
+    m_choice_view_type->Append(_L("Temperature"));
     m_choice_view_type->Append(_L("Volumetric flow rate"));
     m_choice_view_type->Append(_L("Tool"));
     m_choice_view_type->Append(_L("Color Print"));
@@ -223,8 +227,13 @@ bool Preview::init(wxWindow* parent, Model* model)
 
     m_label_show = new wxStaticText(m_bottom_toolbar_panel, wxID_ANY, _L("Show"));
 
+#ifdef _WIN32
+    long combo_style = wxCB_READONLY | wxBORDER_SIMPLE; //set border allows use default color instead of theme color wich is allways light under MSW
+#else
+    long combo_style = wxCB_READONLY;
+#endif
     m_combochecklist_features = new wxComboCtrl();
-    m_combochecklist_features->Create(m_bottom_toolbar_panel, wxID_ANY, _L("Feature types"), wxDefaultPosition, wxDefaultSize, wxCB_READONLY);
+    m_combochecklist_features->Create(m_bottom_toolbar_panel, wxID_ANY, _L("Feature types"), wxDefaultPosition, wxDefaultSize, combo_style);
     std::string feature_items = GUI::into_u8(
         _L("Unknown") + "|1|" +
         _L("Perimeter") + "|1|" +
@@ -236,7 +245,7 @@ bool Preview::init(wxWindow* parent, Model* model)
         _L("Ironing") + "|1|" +
         _L("Bridge infill") + "|1|" +
         _L("Gap fill") + "|1|" +
-        _L("Skirt") + "|1|" +
+        _L("Skirt/Brim") + "|1|" +
         _L("Support material") + "|1|" +
         _L("Support material interface") + "|1|" +
         _L("Wipe tower") + "|1|" +
@@ -245,12 +254,13 @@ bool Preview::init(wxWindow* parent, Model* model)
     Slic3r::GUI::create_combochecklist(m_combochecklist_features, GUI::into_u8(_L("Feature types")), feature_items);
 
     m_combochecklist_options = new wxComboCtrl();
-    m_combochecklist_options->Create(m_bottom_toolbar_panel, wxID_ANY, _L("Options"), wxDefaultPosition, wxDefaultSize, wxCB_READONLY);
+    m_combochecklist_options->Create(m_bottom_toolbar_panel, wxID_ANY, _L("Options"), wxDefaultPosition, wxDefaultSize, combo_style);
     std::string options_items = GUI::into_u8(
         get_option_type_string(OptionType::Travel) + "|0|" +
         get_option_type_string(OptionType::Wipe) + "|0|" +
         get_option_type_string(OptionType::Retractions) + "|0|" +
         get_option_type_string(OptionType::Unretractions) + "|0|" +
+        get_option_type_string(OptionType::Seams) + "|0|" +
         get_option_type_string(OptionType::ToolChanges) + "|0|" +
         get_option_type_string(OptionType::ColorChanges) + "|0|" +
         get_option_type_string(OptionType::PausePrints) + "|0|" +
@@ -318,22 +328,6 @@ void Preview::set_as_dirty()
         m_canvas->set_as_dirty();
 }
 
-#if !ENABLE_PREVIEW_TYPE_CHANGE
-void Preview::set_number_extruders(unsigned int number_extruders)
-{
-    if (m_number_extruders != number_extruders) {
-        m_number_extruders = number_extruders;
-        int tool_idx = m_choice_view_type->FindString(_(L("Tool")));
-        int type = (number_extruders > 1) ? tool_idx /* color by a tool number */  : 0; // color by a feature type
-        m_choice_view_type->SetSelection(type);
-        if (0 <= type && (type < static_cast<int>(GCodeViewer::EViewType::Count)))
-            m_canvas->set_gcode_view_preview_type(static_cast<GCodeViewer::EViewType>(type));
-
-        m_preferred_color_mode = (type == tool_idx) ? "tool_or_feature" : "feature";
-    }
-}
-#endif // !ENABLE_PREVIEW_TYPE_CHANGE
-
 void Preview::bed_shape_changed()
 {
     if (m_canvas != nullptr)
@@ -402,6 +396,10 @@ void Preview::refresh_print()
 
 void Preview::msw_rescale()
 {
+#ifdef _WIN32
+    m_choice_view_type->Rescale();
+    m_choice_view_type->SetMinSize(m_choice_view_type->GetSize());
+#endif
     // rescale slider
     if (m_layers_slider != nullptr) m_layers_slider->msw_rescale();
     if (m_moves_slider != nullptr) m_moves_slider->msw_rescale();
@@ -415,6 +413,17 @@ void Preview::msw_rescale()
 
 void Preview::sys_color_changed()
 {
+#ifdef _WIN32
+    wxWindowUpdateLocker noUpdates(this);
+
+    wxGetApp().UpdateAllStaticTextDarkUI(m_bottom_toolbar_panel);
+    wxGetApp().UpdateDarkUI(m_choice_view_type);
+    wxGetApp().UpdateDarkUI(m_combochecklist_features);
+    wxGetApp().UpdateDarkUI(static_cast<wxCheckListBoxComboPopup*>(m_combochecklist_features->GetPopupControl()));
+    wxGetApp().UpdateDarkUI(m_combochecklist_options);
+    wxGetApp().UpdateDarkUI(static_cast<wxCheckListBoxComboPopup*>(m_combochecklist_options->GetPopupControl()));
+#endif
+
     if (m_layers_slider != nullptr)
         m_layers_slider->sys_color_changed();
 }
@@ -437,7 +446,7 @@ void Preview::edit_layers_slider(wxKeyEvent& evt)
 void Preview::bind_event_handlers()
 {
     this->Bind(wxEVT_SIZE, &Preview::on_size, this);
-    m_choice_view_type->Bind(wxEVT_CHOICE, &Preview::on_choice_view_type, this);
+    m_choice_view_type->Bind(wxEVT_COMBOBOX, &Preview::on_choice_view_type, this);
     m_combochecklist_features->Bind(wxEVT_CHECKLISTBOX, &Preview::on_combochecklist_features, this);
     m_combochecklist_options->Bind(wxEVT_CHECKLISTBOX, &Preview::on_combochecklist_options, this);
     m_moves_slider->Bind(wxEVT_SCROLL_CHANGED, &Preview::on_moves_slider_scroll_changed, this);
@@ -446,18 +455,16 @@ void Preview::bind_event_handlers()
 void Preview::unbind_event_handlers()
 {
     this->Unbind(wxEVT_SIZE, &Preview::on_size, this);
-    m_choice_view_type->Unbind(wxEVT_CHOICE, &Preview::on_choice_view_type, this);
+    m_choice_view_type->Unbind(wxEVT_COMBOBOX, &Preview::on_choice_view_type, this);
     m_combochecklist_features->Unbind(wxEVT_CHECKLISTBOX, &Preview::on_combochecklist_features, this);
     m_combochecklist_options->Unbind(wxEVT_CHECKLISTBOX, &Preview::on_combochecklist_options, this);
     m_moves_slider->Unbind(wxEVT_SCROLL_CHANGED, &Preview::on_moves_slider_scroll_changed, this);
 }
 
-#if ENABLE_ARROW_KEYS_WITH_SLIDERS
 void Preview::move_moves_slider(wxKeyEvent& evt)
 {
     if (m_moves_slider != nullptr) m_moves_slider->OnKeyDown(evt);
 }
-#endif // ENABLE_ARROW_KEYS_WITH_SLIDERS
 
 void Preview::hide_layers_slider()
 {
@@ -473,20 +480,11 @@ void Preview::on_size(wxSizeEvent& evt)
 
 void Preview::on_choice_view_type(wxCommandEvent& evt)
 {
-#if !ENABLE_PREVIEW_TYPE_CHANGE
-    m_preferred_color_mode = (m_choice_view_type->GetStringSelection() == L("Tool")) ? "tool" : "feature";
-#endif // !ENABLE_PREVIEW_TYPE_CHANGE
     int selection = m_choice_view_type->GetCurrentSelection();
-#if ENABLE_PREVIEW_TYPE_CHANGE
     if (0 <= selection && selection < static_cast<int>(GCodeViewer::EViewType::Count)) {
         m_canvas->set_toolpath_view_type(static_cast<GCodeViewer::EViewType>(selection));
         m_keep_current_preview_type = true;
     }
-#else
-    if (0 <= selection && selection < static_cast<int>(GCodeViewer::EViewType::Count))
-        m_canvas->set_toolpath_view_type(static_cast<GCodeViewer::EViewType>(selection));
-#endif // ENABLE_PREVIEW_TYPE_CHANGE
-
     refresh_print();
 }
 
@@ -499,56 +497,24 @@ void Preview::on_combochecklist_features(wxCommandEvent& evt)
 
 void Preview::on_combochecklist_options(wxCommandEvent& evt)
 {
-    unsigned int curr_flags = m_canvas->get_gcode_options_visibility_flags();
-    unsigned int new_flags = Slic3r::GUI::combochecklist_get_flags(m_combochecklist_options);
+    const unsigned int curr_flags = m_canvas->get_gcode_options_visibility_flags();
+    const unsigned int new_flags = Slic3r::GUI::combochecklist_get_flags(m_combochecklist_options);
     if (curr_flags == new_flags)
         return;
 
     m_canvas->set_gcode_options_visibility_from_flags(new_flags);
-
-#if ENABLE_RENDER_PATH_REFRESH_AFTER_OPTIONS_CHANGE
-    m_canvas->refresh_gcode_preview_render_paths();
-#else
-    auto xored = [](unsigned int flags1, unsigned int flags2, unsigned int flag) {
-        auto is_flag_set = [](unsigned int flags, unsigned int flag) {
-            return (flags & (1 << flag)) != 0;
-        };
-        return !is_flag_set(flags1, flag) != !is_flag_set(flags2, flag);
-    };
-
-    bool skip_refresh = xored(curr_flags, new_flags, static_cast<unsigned int>(OptionType::Shells)) ||
-        xored(curr_flags, new_flags, static_cast<unsigned int>(OptionType::ToolMarker));
-
-    if (!skip_refresh)
-        refresh_print();
-    else
-        m_canvas->set_as_dirty();
-#endif // ENABLE_RENDER_PATH_REFRESH_AFTER_OPTIONS_CHANGE
-}
-
-#if !ENABLE_PREVIEW_TYPE_CHANGE
-void Preview::update_view_type(bool keep_volumes)
-{
-    const DynamicPrintConfig& config = wxGetApp().preset_bundle->project_config;
-
-    const wxString& choice = !wxGetApp().plater()->model().custom_gcode_per_print_z.gcodes.empty() /*&&
-                             (wxGetApp().extruders_edited_cnt()==1 || !slice_completed) */? 
-                                _L("Color Print") :
-                                config.option<ConfigOptionFloats>("wiping_volumes_matrix")->values.size() > 1 ?
-                                    _L("Tool") : 
-                                    _L("Feature type");
-
-    int type = m_choice_view_type->FindString(choice);
-    if (m_choice_view_type->GetSelection() != type) {
-        m_choice_view_type->SetSelection(type);
-        if (0 <= type && type < static_cast<int>(GCodeViewer::EViewType::Count))
-            m_canvas->set_gcode_view_preview_type(static_cast<GCodeViewer::EViewType>(type));
-        m_preferred_color_mode = "feature";
+    if (m_canvas->get_gcode_view_type() == GCodeViewer::EViewType::Feedrate) {
+        const unsigned int diff_flags = curr_flags ^ new_flags;
+        if ((diff_flags & (1 << static_cast<unsigned int>(Preview::OptionType::Travel))) != 0)
+            refresh_print();
+        else
+            m_canvas->refresh_gcode_preview_render_paths();
     }
+    else
+        m_canvas->refresh_gcode_preview_render_paths();
 
-    reload_print(keep_volumes);
+    update_moves_slider();
 }
-#endif // !ENABLE_PREVIEW_TYPE_CHANGE
 
 void Preview::update_bottom_toolbar()
 {
@@ -605,12 +571,8 @@ wxBoxSizer* Preview::create_layers_slider_sizer()
         model.custom_gcode_per_print_z = m_layers_slider->GetTicksValues();
         m_schedule_background_process();
 
-#if ENABLE_PREVIEW_TYPE_CHANGE
         m_keep_current_preview_type = false;
         reload_print(false);
-#else
-        update_view_type(false);
-#endif // ENABLE_PREVIEW_TYPE_CHANGE
         });
 
     return sizer;
@@ -680,9 +642,17 @@ void Preview::update_layers_slider(const std::vector<double>& layers_z, bool kee
     update_layers_slider_mode();
 
     Plater* plater = wxGetApp().plater();
-    CustomGCode::Info& ticks_info_from_model = plater->model().custom_gcode_per_print_z;
+    CustomGCode::Info ticks_info_from_model;
+    if (wxGetApp().is_editor())
+        ticks_info_from_model = plater->model().custom_gcode_per_print_z;
+    else {
+        ticks_info_from_model.mode = CustomGCode::Mode::SingleExtruder;
+        ticks_info_from_model.gcodes = m_canvas->get_custom_gcode_per_print_z();
+    }
     check_layers_slider_values(ticks_info_from_model.gcodes, layers_z);
 
+    //first of all update extruder colors to avoid crash, when we are switching printer preset from MM to SM
+    m_layers_slider->SetExtruderColors(plater->get_extruder_colors_from_plater_config(wxGetApp().is_editor() ? nullptr : m_gcode_result));
     m_layers_slider->SetSliderValues(layers_z);
     assert(m_layers_slider->GetMinValue() == 0);
     m_layers_slider->SetMaxValue(layers_z.empty() ? 0 : layers_z.size() - 1);
@@ -707,11 +677,69 @@ void Preview::update_layers_slider(const std::vector<double>& layers_z, bool kee
     bool sla_print_technology = plater->printer_technology() == ptSLA;
     bool sequential_print = wxGetApp().preset_bundle->prints.get_edited_preset().config.opt_bool("complete_objects");
     m_layers_slider->SetDrawMode(sla_print_technology, sequential_print);
-    m_layers_slider->SetExtruderColors(plater->get_extruder_colors_from_plater_config());
     if (sla_print_technology)
         m_layers_slider->SetLayersTimes(plater->sla_print().print_statistics().layers_times);
-    else
-        m_layers_slider->SetLayersTimes(m_gcode_result->time_statistics.modes.front().layers_times);
+    else {
+        auto print_mode_stat = m_gcode_result->print_statistics.modes.front();
+        m_layers_slider->SetLayersTimes(print_mode_stat.layers_times, print_mode_stat.time);
+    }
+
+    // Suggest the auto color change, if model looks like sign
+    if (m_layers_slider->IsNewPrint())
+    {
+        const Print& print = wxGetApp().plater()->fff_print();
+
+        //bool is_possible_auto_color_change = false;
+        for (auto object : print.objects()) {
+            double object_x = double(object->size().x());
+            double object_y = double(object->size().y());
+
+            // if it's sign, than object have not to be a too height
+            double height = object->height();
+            coord_t longer_side = std::max(object_x, object_y);
+            auto   num_layers = int(object->layers().size());
+            if (height / longer_side > 0.3 || num_layers < 2)
+                continue;
+
+            const ExPolygons& bottom = object->get_layer(0)->lslices;
+            double bottom_area = area(bottom);
+
+            // at least 25% of object's height have to be a solid 
+            int  i, min_solid_height = int(0.25 * num_layers);
+            for (i = 1; i <= min_solid_height; ++ i) {
+                double cur_area = area(object->get_layer(i)->lslices);
+                if (!DoubleSlider::equivalent_areas(bottom_area, cur_area)) {
+                    // but due to the elephant foot compensation, the first layer may be slightly smaller than the others
+                    if (i == 1 && fabs(cur_area - bottom_area) / bottom_area < 0.1) {
+                        // So, let process this case and use second layer as a bottom 
+                        bottom_area = cur_area;
+                        continue;
+                    }
+                    break;
+                }
+            }
+            if (i < min_solid_height)
+                continue;
+
+            if (DoubleSlider::check_color_change(object, i, num_layers, true, [this, object](Layer*) {
+                NotificationManager* notif_mngr = wxGetApp().plater()->get_notification_manager();
+                notif_mngr->push_notification(
+                    NotificationType::SignDetected, NotificationManager::NotificationLevel::PrintInfoNotificationLevel,
+                    _u8L("NOTE:") + "\n" +
+                    format(_u8L("Sliced object \"%1%\" looks like a logo or a sign"), object->model_object()->name) + "\n",
+                    _u8L("Apply color change automatically"),
+                    [this](wxEvtHandler*) {
+                        m_layers_slider->auto_color_change();
+                        return true;
+                    });
+
+                notif_mngr->apply_in_preview();
+                return true;
+            }) )
+                // first object with color chnages is found
+                break;
+        }
+    }
 
     m_layers_slider_sizer->Show((size_t)0);
     Layout();
@@ -745,11 +773,11 @@ void Preview::update_layers_slider_mode()
                         object->config.option("extruder")->getInt() != extruder)
                         return false;
 
-                    if (object->volumes.size() > 1)
-                        for (ModelVolume* volume : object->volumes)
-                            if (volume->config.has("extruder") &&
-                                volume->config.option("extruder")->getInt() != extruder)
-                                return false;
+                    for (ModelVolume* volume : object->volumes)
+                        if ((volume->config.has("extruder") &&
+                            volume->config.option("extruder")->getInt() != extruder) ||
+                            !volume->mmu_segmentation_facets.empty())
+                            return false;
 
                     for (const auto& range : object->layer_config_ranges)
                         if (range.second.has("extruder") &&
@@ -784,26 +812,17 @@ void Preview::update_layers_slider_from_canvas(wxKeyEvent& event)
 
     const auto key = event.GetKeyCode();
 
-#if ENABLE_ARROW_KEYS_WITH_SLIDERS
     if (key == 'S' || key == 'W') {
         const int new_pos = key == 'W' ? m_layers_slider->GetHigherValue() + 1 : m_layers_slider->GetHigherValue() - 1;
-#else
-    if (key == 'U' || key == 'D') {
-        const int new_pos = key == 'U' ? m_layers_slider->GetHigherValue() + 1 : m_layers_slider->GetHigherValue() - 1;
-#endif // ENABLE_ARROW_KEYS_WITH_SLIDERS
         m_layers_slider->SetHigherValue(new_pos);
         if (event.ShiftDown() || m_layers_slider->is_one_layer()) m_layers_slider->SetLowerValue(m_layers_slider->GetHigherValue());
     }
-#if ENABLE_ARROW_KEYS_WITH_SLIDERS
     else if (key == 'A' || key == 'D') {
         const int new_pos = key == 'D' ? m_moves_slider->GetHigherValue() + 1 : m_moves_slider->GetHigherValue() - 1;
         m_moves_slider->SetHigherValue(new_pos);
         if (event.ShiftDown() || m_moves_slider->is_one_layer()) m_moves_slider->SetLowerValue(m_moves_slider->GetHigherValue());
     }
     else if (key == 'X')
-#else
-    else if (key == 'S')
-#endif // ENABLE_ARROW_KEYS_WITH_SLIDERS
         m_layers_slider->ChangeOneLayerLock();
     else if (key == WXK_SHIFT)
         m_layers_slider->UseDefaultColors(false);
@@ -819,12 +838,17 @@ void Preview::update_moves_slider()
         return;
 
     std::vector<double> values(view.endpoints.last - view.endpoints.first + 1);
+    std::vector<double> alternate_values(view.endpoints.last - view.endpoints.first + 1);
     unsigned int count = 0;
     for (unsigned int i = view.endpoints.first; i <= view.endpoints.last; ++i) {
-        values[count++] = static_cast<double>(i + 1);
+        values[count] = static_cast<double>(i + 1);
+        if (view.gcode_ids[i] > 0)
+            alternate_values[count] = static_cast<double>(view.gcode_ids[i]);
+        ++count;
     }
 
     m_moves_slider->SetSliderValues(values);
+    m_moves_slider->SetSliderAlternateValues(alternate_values);
     m_moves_slider->SetMaxValue(view.endpoints.last - view.endpoints.first);
     m_moves_slider->SetSelectionSpan(view.current.first - view.endpoints.first, view.current.last - view.endpoints.first);
 }
@@ -876,23 +900,9 @@ void Preview::load_print_as_fff(bool keep_z_range)
         return;
     }
 
-#if !ENABLE_PREVIEW_TYPE_CHANGE
-    if (m_preferred_color_mode == "tool_or_feature") {
-        // It is left to Slic3r to decide whether the print shall be colored by the tool or by the feature.
-        // Color by feature if it is a single extruder print.
-        unsigned int number_extruders = (unsigned int)print->extruders().size();
-        int tool_idx = m_choice_view_type->FindString(_L("Tool"));
-        int type = (number_extruders > 1) ? tool_idx /* color by a tool number */ : 0; // color by a feature type
-        m_choice_view_type->SetSelection(type);
-        if (0 <= type && type < static_cast<int>(GCodeViewer::EViewType::Count))
-            m_canvas->set_gcode_view_preview_type(static_cast<GCodeViewer::EViewType>(type));
-        // If the->SetSelection changed the following line, revert it to "decide yourself".
-        m_preferred_color_mode = "tool_or_feature";
-    }
-#endif // !ENABLE_PREVIEW_TYPE_CHANGE
-
     GCodeViewer::EViewType gcode_view_type = m_canvas->get_gcode_view_preview_type();
     bool gcode_preview_data_valid = !m_gcode_result->moves.empty();
+
     // Collect colors per extruder.
     std::vector<std::string> colors;
     std::vector<CustomGCode::Item> color_print_values = {};
@@ -901,7 +911,10 @@ void Preview::load_print_as_fff(bool keep_z_range)
         colors = wxGetApp().plater()->get_colors_for_color_print(m_gcode_result);
 
         if (!gcode_preview_data_valid) {
-            color_print_values = wxGetApp().plater()->model().custom_gcode_per_print_z.gcodes;
+            if (wxGetApp().is_editor())
+                color_print_values = wxGetApp().plater()->model().custom_gcode_per_print_z.gcodes;
+            else
+                color_print_values = m_canvas->get_custom_gcode_per_print_z();
             colors.push_back("#808080"); // gray color for pause print or custom G-code 
         }
     }
@@ -910,20 +923,20 @@ void Preview::load_print_as_fff(bool keep_z_range)
         color_print_values.clear();
     }
 
-    if (IsShown()) {
-        std::vector<double> zs;
+    std::vector<double> zs;
 
+    if (IsShown()) {
         m_canvas->set_selected_extruder(0);
         if (gcode_preview_data_valid) {
             // Load the real G-code preview.
-            m_canvas->load_gcode_preview(*m_gcode_result);
-            m_canvas->refresh_gcode_preview(*m_gcode_result, colors);
+            m_canvas->load_gcode_preview(*m_gcode_result, colors);
             m_left_sizer->Show(m_bottom_toolbar_panel);
             m_left_sizer->Layout();
             Refresh();
             zs = m_canvas->get_gcode_layers_zs();
             m_loaded = true;
-        } else {
+        }
+        else if (wxGetApp().is_editor()) {
             // Load the initial preview based on slices, not the final G-code.
             m_canvas->load_preview(colors, color_print_values);
             m_left_sizer->Hide(m_bottom_toolbar_panel);
@@ -931,6 +944,30 @@ void Preview::load_print_as_fff(bool keep_z_range)
             Refresh();
             zs = m_canvas->get_volumes_print_zs(true);
         }
+
+        if (!zs.empty() && !m_keep_current_preview_type) {
+            unsigned int number_extruders = wxGetApp().is_editor() ?
+                (unsigned int)print->extruders().size() :
+                m_canvas->get_gcode_extruders_count();
+            std::vector<Item> gcodes = wxGetApp().is_editor() ?
+                wxGetApp().plater()->model().custom_gcode_per_print_z.gcodes :
+                m_canvas->get_custom_gcode_per_print_z();
+            const wxString choice = !gcodes.empty() ?
+                _L("Color Print") :
+                (number_extruders > 1) ? _L("Tool") : _L("Feature type");
+
+            int type = m_choice_view_type->FindString(choice);
+            if (m_choice_view_type->GetSelection() != type) {
+                if (0 <= type && type < static_cast<int>(GCodeViewer::EViewType::Count)) {
+                    m_choice_view_type->SetSelection(type);
+                    m_canvas->set_gcode_view_preview_type(static_cast<GCodeViewer::EViewType>(type));
+                    if (wxGetApp().is_gcode_viewer())
+                        m_keep_current_preview_type = true;
+                    refresh_print();
+                }
+            }
+        }
+
         if (zs.empty()) {
             // all layers filtered out
             hide_layers_slider();
@@ -938,24 +975,6 @@ void Preview::load_print_as_fff(bool keep_z_range)
         } else
             update_layers_slider(zs, keep_z_range);
     }
-
-#if ENABLE_PREVIEW_TYPE_CHANGE
-    unsigned int number_extruders = (unsigned int)print->extruders().size();
-
-    if (!m_keep_current_preview_type) {
-        const wxString choice = !wxGetApp().plater()->model().custom_gcode_per_print_z.gcodes.empty() ?
-            _L("Color Print") :
-            (number_extruders > 1) ? _L("Tool") : _L("Feature type");
-
-        int type = m_choice_view_type->FindString(choice);
-        if (m_choice_view_type->GetSelection() != type) {
-            if (0 <= type && type < static_cast<int>(GCodeViewer::EViewType::Count)) {
-                m_choice_view_type->SetSelection(type);
-                m_canvas->set_gcode_view_preview_type(static_cast<GCodeViewer::EViewType>(type));
-            }
-        }
-    }
-#endif // ENABLE_PREVIEW_TYPE_CHANGE
 }
 
 void Preview::load_print_as_sla()
@@ -977,6 +996,7 @@ void Preview::load_print_as_sla()
     sort_remove_duplicates(zs);
 
     m_canvas->reset_clipping_planes_cache();
+    m_canvas->set_use_clipping_planes(true);
 
     n_layers = (unsigned int)zs.size();
     if (n_layers == 0) {
@@ -1010,7 +1030,6 @@ void Preview::on_layers_slider_scroll_changed(wxCommandEvent& event)
         else if (tech == ptSLA) {
             m_canvas->set_clipping_plane(0, ClippingPlane(Vec3d::UnitZ(), -m_layers_slider->GetLowerValueD()));
             m_canvas->set_clipping_plane(1, ClippingPlane(-Vec3d::UnitZ(), m_layers_slider->GetHigherValueD()));
-            m_canvas->set_use_clipping_planes(m_layers_slider->GetHigherValue() != 0);
             m_canvas->render();
         }
     }
@@ -1030,6 +1049,7 @@ wxString Preview::get_option_type_string(OptionType type) const
     case OptionType::Wipe:          { return _L("Wipe"); }
     case OptionType::Retractions:   { return _L("Retractions"); }
     case OptionType::Unretractions: { return _L("Deretractions"); }
+    case OptionType::Seams:         { return _L("Seams"); }
     case OptionType::ToolChanges:   { return _L("Tool changes"); }
     case OptionType::ColorChanges:  { return _L("Color changes"); }
     case OptionType::PausePrints:   { return _L("Print pauses"); }
