@@ -3,9 +3,7 @@
 
 #include "GLGizmoPainterBase.hpp"
 
-namespace Slic3r {
-
-namespace GUI {
+namespace Slic3r::GUI {
 
 class GLGizmoFdmSupports : public GLGizmoPainterBase
 {
@@ -19,6 +17,13 @@ protected:
     void on_render_input_window(float x, float y, float bottom_limit) override;
     std::string on_get_name() const override;
 
+    wxString handle_snapshot_action_name(bool shift_down, Button button_down) const override;
+
+    std::string get_gizmo_entering_text() const override { return _u8L("Entering Paint-on supports"); }
+    std::string get_gizmo_leaving_text() const override { return _u8L("Leaving Paint-on supports"); }
+    std::string get_action_snapshot_name() override { return _u8L("Paint-on supports editing"); }
+
+
 private:
     bool on_init() override;
 
@@ -30,8 +35,6 @@ private:
     PainterGizmoType get_painter_type() const override;
 
     void select_facets_by_angle(float threshold, bool block);
-    float m_angle_threshold_deg = 45.f;
-    bool m_setting_angle = false;
 
     // This map holds all translated description texts, so they can be easily referenced during layout calculations
     // etc. When language changes, GUI is recreated and this class constructed again, so the change takes effect.
@@ -40,8 +43,7 @@ private:
 
 
 
-} // namespace GUI
-} // namespace Slic3r
+} // namespace Slic3r::GUI
 
 
 #endif // slic3r_GLGizmoFdmSupports_hpp_
