@@ -19,20 +19,10 @@ namespace GUI {
 
 GLGizmoHollow::GLGizmoHollow(GLCanvas3D& parent, const std::string& icon_filename, unsigned int sprite_id)
     : GLGizmoBase(parent, icon_filename, sprite_id)
-    , m_quadric(nullptr)
 {
-    m_quadric = ::gluNewQuadric();
-    if (m_quadric != nullptr)
-        // using GLU_FILL does not work when the instance's transformation
-        // contains mirroring (normals are reverted)
-        ::gluQuadricDrawStyle(m_quadric, GLU_FILL);
+    m_vbo_cylinder.init_from(its_make_cylinder(1., 1.));
 }
 
-GLGizmoHollow::~GLGizmoHollow()
-{
-    if (m_quadric != nullptr)
-        ::gluDeleteQuadric(m_quadric);
-}
 
 bool GLGizmoHollow::on_init()
 {
@@ -71,7 +61,7 @@ void GLGizmoHollow::set_sla_support_data(ModelObject*, const Selection&)
 
 
 
-void GLGizmoHollow::on_render() const
+void GLGizmoHollow::on_render()
 {
     const Selection& selection = m_parent.get_selection();
     const CommonGizmosDataObjects::SelectionInfo* sel_info = m_c->selection_info();
@@ -87,7 +77,7 @@ void GLGizmoHollow::on_render() const
     glsafe(::glEnable(GL_BLEND));
     glsafe(::glEnable(GL_DEPTH_TEST));
 
-    if (m_quadric != nullptr && selection.is_from_single_instance())
+    if (selection.is_from_single_instance())
         render_points(selection, false);
 
     m_selection_rectangle.render(m_parent);
@@ -98,12 +88,12 @@ void GLGizmoHollow::on_render() const
 }
 
 
-void GLGizmoHollow::on_render_for_picking() const
+void GLGizmoHollow::on_render_for_picking()
 {
     const Selection& selection = m_parent.get_selection();
-#if ENABLE_RENDER_PICKING_PASS
-    m_z_shift = selection.get_volume(*selection.get_volume_idxs().begin())->get_sla_shift_z();
-#endif
+//#if ENABLE_RENDER_PICKING_PASS
+//    m_z_shift = selection.get_volume(*selection.get_volume_idxs().begin())->get_sla_shift_z();
+//#endif
 
     glsafe(::glEnable(GL_DEPTH_TEST));
     render_points(selection, true);
@@ -111,8 +101,10 @@ void GLGizmoHollow::on_render_for_picking() const
 
 void GLGizmoHollow::render_points(const Selection& selection, bool picking) const
 {
-    if (!picking)
-        glsafe(::glEnable(GL_LIGHTING));
+    GLShaderProgram* shader = picking ? nullptr : wxGetApp().get_shader("gouraud_light");
+    if (shader)
+        shader->start_using();
+    ScopeGuard guard([shader]() { if (shader) shader->stop_using(); });
 
     const GLVolume* vol = selection.get_volume(*selection.get_volume_idxs().begin());
     const Transform3d& instance_scaling_matrix_inverse = vol->get_instance_transformation().get_matrix(true, true, false, true).inverse();
@@ -122,11 +114,11 @@ void GLGizmoHollow::render_points(const Selection& selection, bool picking) cons
     glsafe(::glTranslated(0.0, 0.0, m_c->selection_info()->get_sla_shift()));
     glsafe(::glMultMatrixd(instance_matrix.data()));
 
-    float render_color[4];
+    std::array<float, 4> render_color;
     const sla::DrainHoles& drain_holes = m_c->selection_info()->model_object()->sla_drain_holes;
     size_t cache_size = drain_holes.size();
-    for (size_t i = 0; i < cache_size; ++i)
-    {
+
+    for (size_t i = 0; i < cache_size; ++i) {
         const sla::DrainHole& drain_hole = drain_holes[i];
         const bool& point_selected = m_selected[i];
 
@@ -136,28 +128,27 @@ void GLGizmoHollow::render_points(const Selection& selection, bool picking) cons
         // First decide about the color of the point.
         if (picking) {
             std::array<float, 4> color = picking_color_component(i);
-            render_color[0] = color[0];
-            render_color[1] = color[1];
-            render_color[2] = color[2];
-            render_color[3] = color[3];
+            render_color = color;
         }
         else {
-            render_color[3] = 1.f;
             if (size_t(m_hover_id) == i) {
-                render_color[0] = 0.f;
-                render_color[1] = 1.0f;
-                render_color[2] = 1.0f;
+                render_color = {0.f, 1.f, 1.f, 1.f};
+            }
+            else if (m_c->hollowed_mesh() &&
+                       i < m_c->hollowed_mesh()->get_drainholes().size() &&
+                       m_c->hollowed_mesh()->get_drainholes()[i].failed) {
+                render_color = {1.f, 0.f, 0.f, .5f};
             }
             else { // neigher hover nor picking
-                render_color[0] = point_selected ? 1.0f : 0.7f;
-                render_color[1] = point_selected ? 0.3f : 0.7f;
-                render_color[2] = point_selected ? 0.3f : 0.7f;
+
+                render_color[0] = point_selected ? 1.0f : 1.f;
+                render_color[1] = point_selected ? 0.3f : 1.f;
+                render_color[2] = point_selected ? 0.3f : 1.f;
                 render_color[3] = 0.5f;
             }
         }
-        glsafe(::glColor4fv(render_color));
-        float render_color_emissive[4] = { 0.5f * render_color[0], 0.5f * render_color[1], 0.5f * render_color[2], 1.f};
-        glsafe(::glMaterialfv(GL_FRONT, GL_EMISSION, render_color_emissive));
+
+        const_cast<GLModel*>(&m_vbo_cylinder)->set_color(-1, render_color);
 
         // Inverse matrix of the instance scaling is applied so that the mark does not scale with the object.
         glsafe(::glPushMatrix());
@@ -174,12 +165,8 @@ void GLGizmoHollow::render_points(const Selection& selection, bool picking) cons
         glsafe(::glRotated(aa.angle() * (180. / M_PI), aa.axis()(0), aa.axis()(1), aa.axis()(2)));
         glsafe(::glPushMatrix());
         glsafe(::glTranslated(0., 0., -drain_hole.height));
-        ::gluCylinder(m_quadric, drain_hole.radius, drain_hole.radius, drain_hole.height + sla::HoleStickOutLength, 24, 1);
-        glsafe(::glTranslated(0., 0., drain_hole.height + sla::HoleStickOutLength));
-        ::gluDisk(m_quadric, 0.0, drain_hole.radius, 24, 1);
-        glsafe(::glTranslated(0., 0., -drain_hole.height - sla::HoleStickOutLength));
-        glsafe(::glRotatef(180.f, 1.f, 0.f, 0.f));
-        ::gluDisk(m_quadric, 0.0, drain_hole.radius, 24, 1);
+        glsafe(::glScaled(drain_hole.radius, drain_hole.radius, drain_hole.height + sla::HoleStickOutLength));
+        m_vbo_cylinder.render();
         glsafe(::glPopMatrix());
 
         if (vol->is_left_handed())
@@ -187,15 +174,6 @@ void GLGizmoHollow::render_points(const Selection& selection, bool picking) cons
         glsafe(::glPopMatrix());
     }
 
-    {
-        // Reset emissive component to zero (the default value)
-        float render_color_emissive[4] = { 0.f, 0.f, 0.f, 1.f };
-        glsafe(::glMaterialfv(GL_FRONT, GL_EMISSION, render_color_emissive));
-    }
-
-    if (!picking)
-        glsafe(::glDisable(GL_LIGHTING));
-
     glsafe(::glPopMatrix());
 }
 
@@ -527,23 +505,26 @@ RENDER_AGAIN:
     y = std::min(y, bottom_limit - approx_height);
     m_imgui->set_next_window_pos(x, y, ImGuiCond_Always);
 
-    m_imgui->begin(on_get_name(), ImGuiWindowFlags_NoMove | ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse);
+    m_imgui->begin(get_name(), ImGuiWindowFlags_NoMove | ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse);
 
     // First calculate width of all the texts that are could possibly be shown. We will decide set the dialog width based on that:
+    const float clipping_slider_left = std::max(m_imgui->calc_text_size(m_desc.at("clipping_of_view")).x,
+                                                m_imgui->calc_text_size(m_desc.at("reset_direction")).x) + m_imgui->scaled(0.5f);
+
     const float settings_sliders_left =
-      std::max({m_imgui->calc_text_size(m_desc.at("offset")).x,
-               m_imgui->calc_text_size(m_desc.at("quality")).x,
-               m_imgui->calc_text_size(m_desc.at("closing_distance")).x,
-               m_imgui->calc_text_size(m_desc.at("hole_diameter")).x,
-               m_imgui->calc_text_size(m_desc.at("hole_depth")).x})
-           + m_imgui->scaled(1.f);
+        std::max(std::max({m_imgui->calc_text_size(m_desc.at("offset")).x,
+                           m_imgui->calc_text_size(m_desc.at("quality")).x,
+                           m_imgui->calc_text_size(m_desc.at("closing_distance")).x,
+                           m_imgui->calc_text_size(m_desc.at("hole_diameter")).x,
+                           m_imgui->calc_text_size(m_desc.at("hole_depth")).x}) + m_imgui->scaled(0.5f), clipping_slider_left);
 
-    const float clipping_slider_left = std::max(m_imgui->calc_text_size(m_desc.at("clipping_of_view")).x, m_imgui->calc_text_size(m_desc.at("reset_direction")).x) + m_imgui->scaled(1.5f);
     const float diameter_slider_left = settings_sliders_left; //m_imgui->calc_text_size(m_desc.at("hole_diameter")).x + m_imgui->scaled(1.f);
     const float minimal_slider_width = m_imgui->scaled(4.f);
 
+    const float button_preview_width = m_imgui->calc_button_size(m_desc.at("preview")).x;
+
     float window_width = minimal_slider_width + std::max({settings_sliders_left, clipping_slider_left, diameter_slider_left});
-    window_width = std::max(window_width, m_imgui->calc_text_size(m_desc.at("preview")).x);
+    window_width = std::max(window_width, button_preview_width);
 
     if (m_imgui->button(m_desc["preview"]))
         hollow_mesh();
@@ -564,51 +545,42 @@ RENDER_AGAIN:
 
     m_imgui->disabled_begin(! m_enable_hollowing);
     float max_tooltip_width = ImGui::GetFontSize() * 20.0f;
+    ImGui::AlignTextToFramePadding();
     m_imgui->text(m_desc.at("offset"));
-    ImGui::SameLine(settings_sliders_left);
+    ImGui::SameLine(settings_sliders_left, m_imgui->get_item_spacing().x);
     ImGui::PushItemWidth(window_width - settings_sliders_left);
-    ImGui::SliderFloat("   ", &offset, offset_min, offset_max, "%.1f mm");
-    if (ImGui::IsItemHovered()) {
-        ImGui::BeginTooltip();
-        ImGui::PushTextWrapPos(max_tooltip_width);
-        ImGui::TextUnformatted((_utf8(opts[0].second->tooltip)).c_str());
-        ImGui::PopTextWrapPos();
-        ImGui::EndTooltip();
-    }
-    bool slider_clicked = ImGui::IsItemClicked(); // someone clicked the slider
-    bool slider_edited = ImGui::IsItemEdited(); // someone is dragging the slider
-    bool slider_released = ImGui::IsItemDeactivatedAfterEdit(); // someone has just released the slider
+    m_imgui->slider_float("##offset", &offset, offset_min, offset_max, "%.1f mm");
+    if (m_imgui->get_last_slider_status().hovered)
+        m_imgui->tooltip((_utf8(opts[0].second->tooltip)).c_str(), max_tooltip_width);
+
+    bool slider_clicked = m_imgui->get_last_slider_status().clicked; // someone clicked the slider
+    bool slider_edited =m_imgui->get_last_slider_status().edited; // someone is dragging the slider
+    bool slider_released =m_imgui->get_last_slider_status().deactivated_after_edit; // someone has just released the slider
 
     if (current_mode >= quality_mode) {
+        ImGui::AlignTextToFramePadding();
         m_imgui->text(m_desc.at("quality"));
-        ImGui::SameLine(settings_sliders_left);
-        ImGui::SliderFloat("    ", &quality, quality_min, quality_max, "%.1f");
-        if (ImGui::IsItemHovered()) {
-            ImGui::BeginTooltip();
-            ImGui::PushTextWrapPos(max_tooltip_width);
-            ImGui::TextUnformatted((_utf8(opts[1].second->tooltip)).c_str());
-            ImGui::PopTextWrapPos();
-            ImGui::EndTooltip();
-        }
-        slider_clicked |= ImGui::IsItemClicked();
-        slider_edited |= ImGui::IsItemEdited();
-        slider_released |= ImGui::IsItemDeactivatedAfterEdit();
+        ImGui::SameLine(settings_sliders_left, m_imgui->get_item_spacing().x);
+        m_imgui->slider_float("##quality", &quality, quality_min, quality_max, "%.1f");
+        if (m_imgui->get_last_slider_status().hovered)
+            m_imgui->tooltip((_utf8(opts[1].second->tooltip)).c_str(), max_tooltip_width);
+
+        slider_clicked |= m_imgui->get_last_slider_status().clicked;
+        slider_edited |= m_imgui->get_last_slider_status().edited;
+        slider_released |= m_imgui->get_last_slider_status().deactivated_after_edit;
     }
 
     if (current_mode >= closing_d_mode) {
+        ImGui::AlignTextToFramePadding();
         m_imgui->text(m_desc.at("closing_distance"));
-        ImGui::SameLine(settings_sliders_left);
-        ImGui::SliderFloat("      ", &closing_d, closing_d_min, closing_d_max, "%.1f mm");
-        if (ImGui::IsItemHovered()) {
-            ImGui::BeginTooltip();
-            ImGui::PushTextWrapPos(max_tooltip_width);
-            ImGui::TextUnformatted((_utf8(opts[2].second->tooltip)).c_str());
-            ImGui::PopTextWrapPos();
-            ImGui::EndTooltip();
-        }
-        slider_clicked |= ImGui::IsItemClicked();
-        slider_edited |= ImGui::IsItemEdited();
-        slider_released |= ImGui::IsItemDeactivatedAfterEdit();
+        ImGui::SameLine(settings_sliders_left, m_imgui->get_item_spacing().x);
+        m_imgui->slider_float("##closing_distance", &closing_d, closing_d_min, closing_d_max, "%.1f mm");
+        if (m_imgui->get_last_slider_status().hovered)
+            m_imgui->tooltip((_utf8(opts[2].second->tooltip)).c_str(), max_tooltip_width);
+
+        slider_clicked |= m_imgui->get_last_slider_status().clicked;
+        slider_edited |= m_imgui->get_last_slider_status().edited;
+        slider_released |= m_imgui->get_last_slider_status().deactivated_after_edit;
     }
 
     if (slider_clicked) {
@@ -640,27 +612,35 @@ RENDER_AGAIN:
 
     ImGui::Separator();
 
-    float diameter_upper_cap = 15.;
-    if (m_new_hole_radius > diameter_upper_cap)
-        m_new_hole_radius = diameter_upper_cap;
+    float diameter_upper_cap = 60.;
+    if (m_new_hole_radius * 2.f > diameter_upper_cap)
+        m_new_hole_radius = diameter_upper_cap / 2.f;
+    ImGui::AlignTextToFramePadding();
     m_imgui->text(m_desc.at("hole_diameter"));
-    ImGui::SameLine(diameter_slider_left);
+    ImGui::SameLine(diameter_slider_left, m_imgui->get_item_spacing().x);
     ImGui::PushItemWidth(window_width - diameter_slider_left);
 
     float diam = 2.f * m_new_hole_radius;
-    ImGui::SliderFloat("", &diam, 1.f, diameter_upper_cap, "%.1f mm");
+    m_imgui->slider_float("##hole_diameter", &diam, 1.f, 25.f, "%.1f mm", 1.f, false);
+    // Let's clamp the value (which could have been entered by keyboard) to a larger range
+    // than the slider. This allows entering off-scale values and still protects against
+    //complete non-sense.
+    diam = std::clamp(diam, 0.1f, diameter_upper_cap);
     m_new_hole_radius = diam / 2.f;
-    bool clicked = ImGui::IsItemClicked();
-    bool edited = ImGui::IsItemEdited();
-    bool deactivated = ImGui::IsItemDeactivatedAfterEdit();
+    bool clicked = m_imgui->get_last_slider_status().clicked;
+    bool edited = m_imgui->get_last_slider_status().edited;
+    bool deactivated = m_imgui->get_last_slider_status().deactivated_after_edit;
 
+    ImGui::AlignTextToFramePadding();
     m_imgui->text(m_desc["hole_depth"]);
-    ImGui::SameLine(diameter_slider_left);
-    ImGui::SliderFloat("  ", &m_new_hole_height, 0.f, 10.f, "%.1f mm");
+    ImGui::SameLine(diameter_slider_left, m_imgui->get_item_spacing().x);
+    m_imgui->slider_float("##hole_depth", &m_new_hole_height, 0.f, 10.f, "%.1f mm", 1.f, false);
+    // Same as above:
+    m_new_hole_height = std::clamp(m_new_hole_height, 0.f, 100.f);
 
-    clicked |= ImGui::IsItemClicked();
-    edited |= ImGui::IsItemEdited();
-    deactivated |= ImGui::IsItemDeactivatedAfterEdit();
+    clicked |= m_imgui->get_last_slider_status().clicked;
+    edited |= m_imgui->get_last_slider_status().edited;
+    deactivated |= m_imgui->get_last_slider_status().deactivated_after_edit;;
 
     // Following is a nasty way to:
     //  - save the initial value of the slider before one starts messing with it
@@ -708,8 +688,10 @@ RENDER_AGAIN:
     // Following is rendered in both editing and non-editing mode:
    // m_imgui->text("");
     ImGui::Separator();
-    if (m_c->object_clipper()->get_position() == 0.f)
+    if (m_c->object_clipper()->get_position() == 0.f) {
+        ImGui::AlignTextToFramePadding();
         m_imgui->text(m_desc.at("clipping_of_view"));
+    }
     else {
         if (m_imgui->button(m_desc.at("reset_direction"))) {
             wxGetApp().CallAfter([this](){
@@ -718,10 +700,10 @@ RENDER_AGAIN:
         }
     }
 
-    ImGui::SameLine(clipping_slider_left);
-    ImGui::PushItemWidth(window_width - clipping_slider_left);
+    ImGui::SameLine(settings_sliders_left, m_imgui->get_item_spacing().x);
+    ImGui::PushItemWidth(window_width - settings_sliders_left);
     float clp_dist = m_c->object_clipper()->get_position();
-    if (ImGui::SliderFloat("     ", &clp_dist, 0.f, 1.f, "%.2f"))
+    if (m_imgui->slider_float("##clp_dist", &clp_dist, 0.f, 1.f, "%.2f"))
         m_c->object_clipper()->set_position(clp_dist, true);
 
     // make sure supports are shown/hidden as appropriate
@@ -753,7 +735,7 @@ RENDER_AGAIN:
 
     if (force_refresh)
         m_parent.set_as_dirty();
-    
+
     if (config_changed)
         m_parent.post_event(SimpleEvent(EVT_GLCANVAS_FORCE_UPDATE));
 }
@@ -782,7 +764,7 @@ bool GLGizmoHollow::on_is_selectable() const
 
 std::string GLGizmoHollow::on_get_name() const
 {
-    return (_(L("Hollow and drill")) + " [H]").ToUTF8().data();
+    return _u8L("Hollow and drill");
 }
 
 
