@@ -4,6 +4,7 @@
 #include "slic3r/GUI/Camera.hpp"
 #include "slic3r/GUI/Gizmos/GLGizmosCommon.hpp"
 #include "slic3r/GUI/MainFrame.hpp"
+#include "slic3r/Utils/UndoRedo.hpp"
 
 #include <GL/glew.h>
 
@@ -16,6 +17,8 @@
 #include "slic3r/GUI/GUI_ObjectSettings.hpp"
 #include "slic3r/GUI/GUI_ObjectList.hpp"
 #include "slic3r/GUI/Plater.hpp"
+#include "slic3r/GUI/NotificationManager.hpp"
+#include "slic3r/GUI/MsgDialog.hpp"
 #include "libslic3r/PresetBundle.hpp"
 #include "libslic3r/SLAPrint.hpp"
 
@@ -25,37 +28,26 @@ namespace GUI {
 
 GLGizmoSlaSupports::GLGizmoSlaSupports(GLCanvas3D& parent, const std::string& icon_filename, unsigned int sprite_id)
     : GLGizmoBase(parent, icon_filename, sprite_id)
-    , m_quadric(nullptr)
-{    
-    m_quadric = ::gluNewQuadric();
-    if (m_quadric != nullptr)
-        // using GLU_FILL does not work when the instance's transformation
-        // contains mirroring (normals are reverted)
-        ::gluQuadricDrawStyle(m_quadric, GLU_FILL);
+{
 }
 
-GLGizmoSlaSupports::~GLGizmoSlaSupports()
-{
-    if (m_quadric != nullptr)
-        ::gluDeleteQuadric(m_quadric);
-}
 
 bool GLGizmoSlaSupports::on_init()
 {
     m_shortcut_key = WXK_CONTROL_L;
 
-    m_desc["head_diameter"]    = _(L("Head diameter")) + ": ";
-    m_desc["lock_supports"]    = _(L("Lock supports under new islands"));
-    m_desc["remove_selected"]  = _(L("Remove selected points"));
-    m_desc["remove_all"]       = _(L("Remove all points"));
-    m_desc["apply_changes"]    = _(L("Apply changes"));
-    m_desc["discard_changes"]  = _(L("Discard changes"));
-    m_desc["minimal_distance"] = _(L("Minimal points distance")) + ": ";
-    m_desc["points_density"]   = _(L("Support points density")) + ": ";
-    m_desc["auto_generate"]    = _(L("Auto-generate points"));
-    m_desc["manual_editing"]   = _(L("Manual editing"));
-    m_desc["clipping_of_view"] = _(L("Clipping of view"))+ ": ";
-    m_desc["reset_direction"]  = _(L("Reset direction"));
+    m_desc["head_diameter"]    = _L("Head diameter") + ": ";
+    m_desc["lock_supports"]    = _L("Lock supports under new islands");
+    m_desc["remove_selected"]  = _L("Remove selected points");
+    m_desc["remove_all"]       = _L("Remove all points");
+    m_desc["apply_changes"]    = _L("Apply changes");
+    m_desc["discard_changes"]  = _L("Discard changes");
+    m_desc["minimal_distance"] = _L("Minimal points distance") + ": ";
+    m_desc["points_density"]   = _L("Support points density") + ": ";
+    m_desc["auto_generate"]    = _L("Auto-generate points");
+    m_desc["manual_editing"]   = _L("Manual editing");
+    m_desc["clipping_of_view"] = _L("Clipping of view")+ ": ";
+    m_desc["reset_direction"]  = _L("Reset direction");
 
     return true;
 }
@@ -83,7 +75,7 @@ void GLGizmoSlaSupports::set_sla_support_data(ModelObject* model_object, const S
 
 
 
-void GLGizmoSlaSupports::on_render() const
+void GLGizmoSlaSupports::on_render()
 {
     ModelObject* mo = m_c->selection_info()->model_object();
     const Selection& selection = m_parent.get_selection();
@@ -99,7 +91,7 @@ void GLGizmoSlaSupports::on_render() const
     glsafe(::glEnable(GL_BLEND));
     glsafe(::glEnable(GL_DEPTH_TEST));
 
-    if (m_quadric != nullptr && selection.is_from_single_instance())
+    if (selection.is_from_single_instance())
         render_points(selection, false);
 
     m_selection_rectangle.render(m_parent);
@@ -110,17 +102,31 @@ void GLGizmoSlaSupports::on_render() const
 }
 
 
-void GLGizmoSlaSupports::on_render_for_picking() const
+void GLGizmoSlaSupports::on_render_for_picking()
 {
     const Selection& selection = m_parent.get_selection();
-    glsafe(::glEnable(GL_DEPTH_TEST));
+    //glsafe(::glEnable(GL_DEPTH_TEST));
     render_points(selection, true);
 }
 
 void GLGizmoSlaSupports::render_points(const Selection& selection, bool picking) const
 {
-    if (!picking)
-        glsafe(::glEnable(GL_LIGHTING));
+    size_t cache_size = m_editing_mode ? m_editing_cache.size() : m_normal_cache.size();
+
+    bool has_points = (cache_size != 0);
+    bool has_holes = (! m_c->hollowed_mesh()->get_hollowed_mesh()
+                   && ! m_c->selection_info()->model_object()->sla_drain_holes.empty());
+
+    if (! has_points && ! has_holes)
+        return;
+
+    GLShaderProgram* shader = picking ? nullptr : wxGetApp().get_shader("gouraud_light");
+    if (shader != nullptr)
+        shader->start_using();
+    ScopeGuard guard([shader]() {
+        if (shader != nullptr)
+            shader->stop_using();
+    });
 
     const GLVolume* vol = selection.get_volume(*selection.get_volume_idxs().begin());
     const Transform3d& instance_scaling_matrix_inverse = vol->get_instance_transformation().get_matrix(true, true, false, true).inverse();
@@ -131,10 +137,8 @@ void GLGizmoSlaSupports::render_points(const Selection& selection, bool picking)
     glsafe(::glTranslated(0.0, 0.0, z_shift));
     glsafe(::glMultMatrixd(instance_matrix.data()));
 
-    float render_color[4];
-    size_t cache_size = m_editing_mode ? m_editing_cache.size() : m_normal_cache.size();
-    for (size_t i = 0; i < cache_size; ++i)
-    {
+    std::array<float, 4> render_color;
+    for (size_t i = 0; i < cache_size; ++i) {
         const sla::SupportPoint& support_point = m_editing_mode ? m_editing_cache[i].support_point : m_normal_cache[i];
         const bool& point_selected = m_editing_mode ? m_editing_cache[i].selected : false;
 
@@ -142,34 +146,31 @@ void GLGizmoSlaSupports::render_points(const Selection& selection, bool picking)
             continue;
 
         // First decide about the color of the point.
-        if (picking) {
-            std::array<float, 4> color = picking_color_component(i);
-            render_color[0] = color[0];
-            render_color[1] = color[1];
-            render_color[2] = color[2];
-	        render_color[3] = color[3];
-        }
+        if (picking)
+            render_color = picking_color_component(i);
         else {
-            render_color[3] = 1.f;
-            if ((size_t(m_hover_id) == i && m_editing_mode)) { // ignore hover state unless editing mode is active
-                render_color[0] = 0.f;
-                render_color[1] = 1.0f;
-                render_color[2] = 1.0f;
-            }
+            if (size_t(m_hover_id) == i && m_editing_mode) // ignore hover state unless editing mode is active
+                render_color = { 0.f, 1.f, 1.f, 1.f };
             else { // neigher hover nor picking
                 bool supports_new_island = m_lock_unique_islands && support_point.is_new_island;
                 if (m_editing_mode) {
-                    render_color[0] = point_selected ? 1.0f : (supports_new_island ? 0.3f : 0.7f);
-                    render_color[1] = point_selected ? 0.3f : (supports_new_island ? 0.3f : 0.7f);
-                    render_color[2] = point_selected ? 0.3f : (supports_new_island ? 1.0f : 0.7f);
+                    if (point_selected)
+                        render_color = { 1.f, 0.3f, 0.3f, 1.f};
+                    else
+                        if (supports_new_island)
+                            render_color = { 0.3f, 0.3f, 1.f, 1.f };
+                        else
+                            render_color = { 0.7f, 0.7f, 0.7f, 1.f };
                 }
                 else
-                    for (unsigned char i=0; i<3; ++i) render_color[i] = 0.5f;
+                    render_color = { 0.5f, 0.5f, 0.5f, 1.f };
             }
         }
-        glsafe(::glColor4fv(render_color));
-        float render_color_emissive[4] = { 0.5f * render_color[0], 0.5f * render_color[1], 0.5f * render_color[2], 1.f};
-        glsafe(::glMaterialfv(GL_FRONT, GL_EMISSION, render_color_emissive));
+
+        const_cast<GLModel*>(&m_cone)->set_color(-1, render_color);
+        const_cast<GLModel*>(&m_sphere)->set_color(-1, render_color);
+        if (shader && !picking)
+            shader->set_uniform("emission_factor", 0.5f);
 
         // Inverse matrix of the instance scaling is applied so that the mark does not scale with the object.
         glsafe(::glPushMatrix());
@@ -194,33 +195,37 @@ void GLGizmoSlaSupports::render_points(const Selection& selection, bool picking)
             const double cone_radius = 0.25; // mm
             const double cone_height = 0.75;
             glsafe(::glPushMatrix());
-            glsafe(::glTranslatef(0.f, 0.f, support_point.head_front_radius * RenderPointScale));
-            ::gluCylinder(m_quadric, 0., cone_radius, cone_height, 24, 1);
+            glsafe(::glTranslatef(0.f, 0.f, cone_height + support_point.head_front_radius * RenderPointScale));
+            glsafe(::glPushMatrix());
+            glsafe(::glRotated(180., 1., 0., 0.));
+            glsafe(::glScaled(cone_radius, cone_radius, cone_height));
+            m_cone.render();
+            glsafe(::glPopMatrix());
             glsafe(::glTranslatef(0.f, 0.f, cone_height));
-            ::gluDisk(m_quadric, 0.0, cone_radius, 24, 1);
             glsafe(::glPopMatrix());
         }
-        ::gluSphere(m_quadric, (double)support_point.head_front_radius * RenderPointScale, 24, 12);
+
+        glsafe(::glPushMatrix());
+        double radius = (double)support_point.head_front_radius * RenderPointScale;
+        glsafe(::glScaled(radius, radius, radius));
+        m_sphere.render();
+        glsafe(::glPopMatrix());
+
         if (vol->is_left_handed())
             glFrontFace(GL_CCW);
 
         glsafe(::glPopMatrix());
     }
 
-    {
-        // Reset emissive component to zero (the default value)
-        float render_color_emissive[4] = { 0.f, 0.f, 0.f, 1.f };
-        glsafe(::glMaterialfv(GL_FRONT, GL_EMISSION, render_color_emissive));
-    }
-
     // Now render the drain holes:
-    //if (! m_c->has_drilled_mesh()) {
-    if (! m_c->hollowed_mesh()->get_hollowed_mesh()) {
+    if (has_holes && ! picking) {
         render_color[0] = 0.7f;
         render_color[1] = 0.7f;
         render_color[2] = 0.7f;
         render_color[3] = 0.7f;
-        glsafe(::glColor4fv(render_color));
+        const_cast<GLModel*>(&m_cylinder)->set_color(-1, render_color);
+        if (shader)
+            shader->set_uniform("emission_factor", 0.5f);
         for (const sla::DrainHole& drain_hole : m_c->selection_info()->model_object()->sla_drain_holes) {
             if (is_mesh_point_clipped(drain_hole.pos.cast<double>()))
                 continue;
@@ -241,12 +246,8 @@ void GLGizmoSlaSupports::render_points(const Selection& selection, bool picking)
             glsafe(::glRotated(aa.angle() * (180. / M_PI), aa.axis()(0), aa.axis()(1), aa.axis()(2)));
             glsafe(::glPushMatrix());
             glsafe(::glTranslated(0., 0., -drain_hole.height));
-            ::gluCylinder(m_quadric, drain_hole.radius, drain_hole.radius, drain_hole.height + sla::HoleStickOutLength, 24, 1);
-            glsafe(::glTranslated(0., 0., drain_hole.height + sla::HoleStickOutLength));
-            ::gluDisk(m_quadric, 0.0, drain_hole.radius, 24, 1);
-            glsafe(::glTranslated(0., 0., -drain_hole.height - sla::HoleStickOutLength));
-            glsafe(::glRotatef(180.f, 1.f, 0.f, 0.f));
-            ::gluDisk(m_quadric, 0.0, drain_hole.radius, 24, 1);
+            glsafe(::glScaled(drain_hole.radius, drain_hole.radius, drain_hole.height + sla::HoleStickOutLength));
+            m_cylinder.render();
             glsafe(::glPopMatrix());
 
             if (vol->is_left_handed())
@@ -255,9 +256,6 @@ void GLGizmoSlaSupports::render_points(const Selection& selection, bool picking)
         }
     }
 
-    if (!picking)
-        glsafe(::glDisable(GL_LIGHTING));
-
     glsafe(::glPopMatrix());
 }
 
@@ -371,7 +369,7 @@ bool GLGizmoSlaSupports::gizmo_event(SLAGizmoEventType action, const Vec2d& mous
             if (m_selection_empty) {
                 std::pair<Vec3f, Vec3f> pos_and_normal;
                 if (unproject_on_mesh(mouse_position, pos_and_normal)) { // we got an intersection
-                    Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("Add support point")));
+                    Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Add support point"));
                     m_editing_cache.emplace_back(sla::SupportPoint(pos_and_normal.first, m_new_point_head_diameter/2.f, false), false, pos_and_normal.second);
                     m_parent.set_as_dirty();
                     m_wait_for_up_event = true;
@@ -403,11 +401,20 @@ bool GLGizmoSlaSupports::gizmo_event(SLAGizmoEventType action, const Vec2d& mous
             for (size_t idx : points_idxs)
                 points_inside.push_back(points[idx].cast<float>());
 
-            // Only select/deselect points that are actually visible
+            // Only select/deselect points that are actually visible. We want to check not only
+            // the point itself, but also the center of base of its cone, so the points don't hide
+            // under every miniature irregularity on the model. Remember the actual number and
+            // append the cone bases.
+            size_t orig_pts_num = points_inside.size();
+            for (size_t idx : points_idxs)
+                points_inside.emplace_back((trafo.get_matrix().cast<float>() * (m_editing_cache[idx].support_point.pos + m_editing_cache[idx].normal)).cast<float>());
+
             for (size_t idx : m_c->raycaster()->raycaster()->get_unobscured_idxs(
                      trafo, wxGetApp().plater()->get_camera(), points_inside,
                      m_c->object_clipper()->get_clipping_plane()))
             {
+                if (idx >= orig_pts_num) // this is a cone-base, get index of point it belongs to
+                    idx -= orig_pts_num;
                 if (rectangle_status == GLSelectionRectangle::Deselect)
                     unselect_point(points_idxs[idx]);
                 else
@@ -450,7 +457,8 @@ bool GLGizmoSlaSupports::gizmo_event(SLAGizmoEventType action, const Vec2d& mous
         }
 
         if (action ==  SLAGizmoEventType::DiscardChanges) {
-            editing_mode_discard_changes();
+            ask_about_changes_call_after([this](){ editing_mode_apply_changes(); },
+                                         [this](){ editing_mode_discard_changes(); });
             return true;
         }
 
@@ -511,7 +519,7 @@ void GLGizmoSlaSupports::delete_selected_points(bool force)
         std::abort();
     }
 
-    Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("Delete support point")));
+    Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Delete support point"));
 
     for (unsigned int idx=0; idx<m_editing_cache.size(); ++idx) {
         if (m_editing_cache[idx].selected && (!m_editing_cache[idx].support_point.is_new_island || !m_lock_unique_islands || force)) {
@@ -627,7 +635,7 @@ RENDER_AGAIN:
     //ImGui::SetNextWindowPos(ImVec2(x, y - std::max(0.f, y+window_size.y-bottom_limit) ));
     //ImGui::SetNextWindowSize(ImVec2(window_size));
 
-    m_imgui->begin(on_get_name(), ImGuiWindowFlags_NoMove | ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse);
+    m_imgui->begin(get_name(), ImGuiWindowFlags_NoMove | ImGuiWindowFlags_AlwaysAutoResize | ImGuiWindowFlags_NoCollapse);
 
     // adjust window position to avoid overlap the view toolbar
     float win_h = ImGui::GetWindowHeight();
@@ -636,7 +644,11 @@ RENDER_AGAIN:
     if ((last_h != win_h) || (last_y != y))
     {
         // ask canvas for another frame to render the window in the correct position
+#if ENABLE_ENHANCED_IMGUI_SLIDER_FLOAT
+        m_imgui->set_requires_extra_frame();
+#else
         m_parent.request_extra_frame();
+#endif // ENABLE_ENHANCED_IMGUI_SLIDER_FLOAT
         if (last_h != win_h)
             last_h = win_h;
         if (last_y != y)
@@ -674,24 +686,24 @@ RENDER_AGAIN:
         //  - keep updating the head radius during sliding so it is continuosly refreshed in 3D scene
         //  - take correct undo/redo snapshot after the user is done with moving the slider
         float initial_value = m_new_point_head_diameter;
-        ImGui::SliderFloat("", &m_new_point_head_diameter, 0.1f, diameter_upper_cap, "%.1f");
-        if (ImGui::IsItemClicked()) {
+        m_imgui->slider_float("##head_diameter", &m_new_point_head_diameter, 0.1f, diameter_upper_cap, "%.1f");
+        if (m_imgui->get_last_slider_status().clicked) {
             if (m_old_point_head_diameter == 0.f)
                 m_old_point_head_diameter = initial_value;
         }
-        if (ImGui::IsItemEdited()) {
+        if (m_imgui->get_last_slider_status().edited) {
             for (auto& cache_entry : m_editing_cache)
                 if (cache_entry.selected)
                     cache_entry.support_point.head_front_radius = m_new_point_head_diameter / 2.f;
         }
-        if (ImGui::IsItemDeactivatedAfterEdit()) {
+        if (m_imgui->get_last_slider_status().deactivated_after_edit) {
             // momentarily restore the old value to take snapshot
             for (auto& cache_entry : m_editing_cache)
                 if (cache_entry.selected)
                     cache_entry.support_point.head_front_radius = m_old_point_head_diameter / 2.f;
             float backup = m_new_point_head_diameter;
             m_new_point_head_diameter = m_old_point_head_diameter;
-            Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("Change point head diameter")));
+            Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Change point head diameter"));
             m_new_point_head_diameter = backup;
             for (auto& cache_entry : m_editing_cache)
                 if (cache_entry.selected)
@@ -734,19 +746,19 @@ RENDER_AGAIN:
         float density = static_cast<const ConfigOptionInt*>(opts[0])->value;
         float minimal_point_distance = static_cast<const ConfigOptionFloat*>(opts[1])->value;
 
-        ImGui::SliderFloat("", &minimal_point_distance, 0.f, 20.f, "%.f mm");
-        bool slider_clicked = ImGui::IsItemClicked(); // someone clicked the slider
-        bool slider_edited = ImGui::IsItemEdited(); // someone is dragging the slider
-        bool slider_released = ImGui::IsItemDeactivatedAfterEdit(); // someone has just released the slider
+        m_imgui->slider_float("##minimal_point_distance", &minimal_point_distance, 0.f, 20.f, "%.f mm");
+        bool slider_clicked = m_imgui->get_last_slider_status().clicked; // someone clicked the slider
+        bool slider_edited = m_imgui->get_last_slider_status().edited; // someone is dragging the slider
+        bool slider_released = m_imgui->get_last_slider_status().deactivated_after_edit; // someone has just released the slider
 
         ImGui::AlignTextToFramePadding();
         m_imgui->text(m_desc.at("points_density"));
         ImGui::SameLine(settings_sliders_left);
 
-        ImGui::SliderFloat(" ", &density, 0.f, 200.f, "%.f %%");
-        slider_clicked |= ImGui::IsItemClicked();
-        slider_edited |= ImGui::IsItemEdited();
-        slider_released |= ImGui::IsItemDeactivatedAfterEdit();
+        m_imgui->slider_float("##points_density", &density, 0.f, 200.f, "%.f %%");
+        slider_clicked |= m_imgui->get_last_slider_status().clicked;
+        slider_edited |= m_imgui->get_last_slider_status().edited;
+        slider_released |= m_imgui->get_last_slider_status().deactivated_after_edit;
 
         if (slider_clicked) { // stash the values of the settings so we know what to revert to after undo
             m_minimal_point_distance_stash = minimal_point_distance;
@@ -759,7 +771,7 @@ RENDER_AGAIN:
         if (slider_released) {
             mo->config.set("support_points_minimal_distance", m_minimal_point_distance_stash);
             mo->config.set("support_points_density_relative", (int)m_density_stash);
-            Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("Support parameter change")));
+            Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Support parameter change"));
             mo->config.set("support_points_minimal_distance", minimal_point_distance);
             mo->config.set("support_points_density_relative", (int)density);
             wxGetApp().obj_list()->update_and_show_object_settings_item();
@@ -788,8 +800,7 @@ RENDER_AGAIN:
 
     // Following is rendered in both editing and non-editing mode:
     ImGui::Separator();
-    if (m_c->object_clipper()->get_position() == 0.f)
-    {
+    if (m_c->object_clipper()->get_position() == 0.f) {
         ImGui::AlignTextToFramePadding();
         m_imgui->text(m_desc.at("clipping_of_view"));
     }
@@ -804,7 +815,7 @@ RENDER_AGAIN:
     ImGui::SameLine(clipping_slider_left);
     ImGui::PushItemWidth(window_width - clipping_slider_left);
     float clp_dist = m_c->object_clipper()->get_position();
-    if (ImGui::SliderFloat("  ", &clp_dist, 0.f, 1.f, "%.2f"))
+    if (m_imgui->slider_float("##clp_dist", &clp_dist, 0.f, 1.f, "%.2f"))
         m_c->object_clipper()->set_position(clp_dist, true);
 
 
@@ -866,10 +877,9 @@ bool GLGizmoSlaSupports::on_is_selectable() const
 
 std::string GLGizmoSlaSupports::on_get_name() const
 {
-    return (_(L("SLA Support Points")) + " [L]").ToUTF8().data();
+    return _u8L("SLA Support Points");
 }
 
-
 CommonGizmosDataID GLGizmoSlaSupports::on_get_requirements() const
 {
     return CommonGizmosDataID(
@@ -883,46 +893,43 @@ CommonGizmosDataID GLGizmoSlaSupports::on_get_requirements() const
 
 
 
+void GLGizmoSlaSupports::ask_about_changes_call_after(std::function<void()> on_yes, std::function<void()> on_no)
+{
+    wxGetApp().CallAfter([on_yes, on_no]() {
+        // Following is called through CallAfter, because otherwise there was a problem
+        // on OSX with the wxMessageDialog being shown several times when clicked into.
+        MessageDialog dlg(GUI::wxGetApp().mainframe, _L("Do you want to save your manually "
+            "edited support points?") + "\n",_L("Save support points?"), wxICON_QUESTION | wxYES | wxNO | wxCANCEL );
+        int ret = dlg.ShowModal();
+            if (ret == wxID_YES)
+                on_yes();
+            else if (ret == wxID_NO)
+                on_no();
+    });
+}
+
+
 void GLGizmoSlaSupports::on_set_state()
 {
     if (m_state == m_old_state)
         return;
 
     if (m_state == On && m_old_state != On) { // the gizmo was just turned on
-        if (! m_parent.get_gizmos_manager().is_serializing()) {
-            // Only take the snapshot when the USER opens the gizmo. Common gizmos
-            // data are not yet available, the CallAfter will postpone taking the
-            // snapshot until they are. No, it does not feel right.
-            wxGetApp().CallAfter([this]() {
-                Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("SLA gizmo turned on")));
-            });
-        }
-
         // Set default head diameter from config.
         const DynamicPrintConfig& cfg = wxGetApp().preset_bundle->sla_prints.get_edited_preset().config;
         m_new_point_head_diameter = static_cast<const ConfigOptionFloat*>(cfg.option("support_head_front_diameter"))->value;
     }
     if (m_state == Off && m_old_state != Off) { // the gizmo was just turned Off
-        bool will_ask = m_editing_mode && unsaved_changes();
+        bool will_ask = m_editing_mode && unsaved_changes() && on_is_activable();
         if (will_ask) {
-            wxGetApp().CallAfter([this]() {
-                // Following is called through CallAfter, because otherwise there was a problem
-                // on OSX with the wxMessageDialog being shown several times when clicked into.
-                wxMessageDialog dlg(GUI::wxGetApp().mainframe, _(L("Do you want to save your manually "
-                    "edited support points?")) + "\n",_(L("Save changes?")), wxICON_QUESTION | wxYES | wxNO);
-                    if (dlg.ShowModal() == wxID_YES)
-                        editing_mode_apply_changes();
-                    else
-                        editing_mode_discard_changes();
-            });
+            ask_about_changes_call_after([this](){ editing_mode_apply_changes(); },
+                                         [this](){ editing_mode_discard_changes(); });
             // refuse to be turned off so the gizmo is active when the CallAfter is executed
             m_state = m_old_state;
         }
         else {
             // we are actually shutting down
             disable_editing_mode(); // so it is not active next time the gizmo opens
-            Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("SLA gizmo turned off")));
-            m_normal_cache.clear();
             m_old_mo_id = -1;
         }
     }
@@ -952,7 +959,7 @@ void GLGizmoSlaSupports::on_stop_dragging()
          && backup.support_point.pos != m_point_before_drag.support_point.pos) // and it was moved, not just selected
         {
             m_editing_cache[m_hover_id] = m_point_before_drag;
-            Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("Move support point")));
+            Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Move support point"));
             m_editing_cache[m_hover_id] = backup;
         }
     }
@@ -1045,7 +1052,7 @@ void GLGizmoSlaSupports::editing_mode_apply_changes()
     disable_editing_mode(); // this leaves the editing mode undo/redo stack and must be done before the snapshot is taken
 
     if (unsaved_changes()) {
-        Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("Support points edit")));
+        Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Support points edit"));
 
         m_normal_cache.clear();
         for (const CacheEntry& ce : m_editing_cache)
@@ -1123,15 +1130,16 @@ void GLGizmoSlaSupports::get_data_from_backend()
 
 void GLGizmoSlaSupports::auto_generate()
 {
-    wxMessageDialog dlg(GUI::wxGetApp().plater(), 
-                        _(L("Autogeneration will erase all manually edited points.")) + "\n\n" +
-                        _(L("Are you sure you want to do it?")) + "\n",
-                        _(L("Warning")), wxICON_WARNING | wxYES | wxNO);
+    //wxMessageDialog dlg(GUI::wxGetApp().plater(), 
+    MessageDialog dlg(GUI::wxGetApp().plater(), 
+                        _L("Autogeneration will erase all manually edited points.") + "\n\n" +
+                        _L("Are you sure you want to do it?") + "\n",
+                        _L("Warning"), wxICON_WARNING | wxYES | wxNO);
 
     ModelObject* mo = m_c->selection_info()->model_object();
 
     if (mo->sla_points_status != sla::PointsStatus::UserModified || m_normal_cache.empty() || dlg.ShowModal() == wxID_YES) {
-        Plater::TakeSnapshot snapshot(wxGetApp().plater(), _(L("Autogenerate support points")));
+        Plater::TakeSnapshot snapshot(wxGetApp().plater(), _L("Autogenerate support points"));
         wxGetApp().CallAfter([this]() { reslice_SLA_supports(); });
         mo->sla_points_status = sla::PointsStatus::Generating;
     }
@@ -1161,6 +1169,7 @@ void GLGizmoSlaSupports::disable_editing_mode()
         m_c->instances_hider()->show_supports(true);
         m_parent.set_as_dirty();
     }
+    wxGetApp().plater()->get_notification_manager()->close_notification_of_type(NotificationType::QuitSLAManualMode);
 }
 
 
@@ -1178,7 +1187,7 @@ bool GLGizmoSlaSupports::unsaved_changes() const
 }
 
 SlaGizmoHelpDialog::SlaGizmoHelpDialog()
-: wxDialog(nullptr, wxID_ANY, _(L("SLA gizmo keyboard shortcuts")), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
+: wxDialog(nullptr, wxID_ANY, _L("SLA gizmo keyboard shortcuts"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
 {
     SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
     const wxString ctrl = GUI::shortkey_ctrl_prefix();
@@ -1189,7 +1198,7 @@ SlaGizmoHelpDialog::SlaGizmoHelpDialog()
     const wxFont& font = wxGetApp().small_font();
     const wxFont& bold_font = wxGetApp().bold_font();
 
-    auto note_text = new wxStaticText(this, wxID_ANY, _(L("Note: some shortcuts work in (non)editing mode only.")));
+    auto note_text = new wxStaticText(this, wxID_ANY, _L("Note: some shortcuts work in (non)editing mode only."));
     note_text->SetFont(font);
 
     auto vsizer    = new wxBoxSizer(wxVERTICAL);
@@ -1207,21 +1216,21 @@ SlaGizmoHelpDialog::SlaGizmoHelpDialog()
     vsizer->AddSpacer(20);
 
     std::vector<std::pair<wxString, wxString>> shortcuts;
-    shortcuts.push_back(std::make_pair(_(L("Left click")),          _(L("Add point"))));
-    shortcuts.push_back(std::make_pair(_(L("Right click")),         _(L("Remove point"))));
-    shortcuts.push_back(std::make_pair(_(L("Drag")),                _(L("Move point"))));
-    shortcuts.push_back(std::make_pair(ctrl+_(L("Left click")),     _(L("Add point to selection"))));
-    shortcuts.push_back(std::make_pair(alt+_(L("Left click")),      _(L("Remove point from selection"))));
-    shortcuts.push_back(std::make_pair(wxString("Shift+")+_(L("Drag")), _(L("Select by rectangle"))));
-    shortcuts.push_back(std::make_pair(alt+_(L("Drag")),            _(L("Deselect by rectangle"))));
-    shortcuts.push_back(std::make_pair(ctrl+"A",                    _(L("Select all points"))));
-    shortcuts.push_back(std::make_pair("Delete",                    _(L("Remove selected points"))));
-    shortcuts.push_back(std::make_pair(ctrl+_(L("Mouse wheel")),    _(L("Move clipping plane"))));
-    shortcuts.push_back(std::make_pair("R",                         _(L("Reset clipping plane"))));
-    shortcuts.push_back(std::make_pair("Enter",                     _(L("Apply changes"))));
-    shortcuts.push_back(std::make_pair("Esc",                       _(L("Discard changes"))));
-    shortcuts.push_back(std::make_pair("M",                         _(L("Switch to editing mode"))));
-    shortcuts.push_back(std::make_pair("A",                         _(L("Auto-generate points"))));
+    shortcuts.push_back(std::make_pair(_L("Left click"),              _L("Add point")));
+    shortcuts.push_back(std::make_pair(_L("Right click"),             _L("Remove point")));
+    shortcuts.push_back(std::make_pair(_L("Drag"),                    _L("Move point")));
+    shortcuts.push_back(std::make_pair(ctrl+_L("Left click"),         _L("Add point to selection")));
+    shortcuts.push_back(std::make_pair(alt+_L("Left click"),          _L("Remove point from selection")));
+    shortcuts.push_back(std::make_pair(wxString("Shift+")+_L("Drag"), _L("Select by rectangle")));
+    shortcuts.push_back(std::make_pair(alt+_(L("Drag")),              _L("Deselect by rectangle")));
+    shortcuts.push_back(std::make_pair(ctrl+"A",                      _L("Select all points")));
+    shortcuts.push_back(std::make_pair("Delete",                      _L("Remove selected points")));
+    shortcuts.push_back(std::make_pair(ctrl+_L("Mouse wheel"),        _L("Move clipping plane")));
+    shortcuts.push_back(std::make_pair("R",                           _L("Reset clipping plane")));
+    shortcuts.push_back(std::make_pair("Enter",                       _L("Apply changes")));
+    shortcuts.push_back(std::make_pair("Esc",                         _L("Discard changes")));
+    shortcuts.push_back(std::make_pair("M",                           _L("Switch to editing mode")));
+    shortcuts.push_back(std::make_pair("A",                           _L("Auto-generate points")));
 
     for (const auto& pair : shortcuts) {
         auto shortcut = new wxStaticText(this, wxID_ANY, pair.first);
