@@ -12,9 +12,6 @@
 
 namespace Slic3r {
 
-class TriangleMesh;
-class TriangleMeshSlicer;
-
 namespace GUI {
 
 struct Camera;
@@ -33,11 +30,8 @@ public:
 
     ClippingPlane(const Vec3d& direction, double offset)
     {
-        Vec3d norm_dir = direction.normalized();
-        m_data[0] = norm_dir(0);
-        m_data[1] = norm_dir(1);
-        m_data[2] = norm_dir(2);
-        m_data[3] = offset;
+        set_normal(direction);
+        set_offset(offset);
     }
 
     bool operator==(const ClippingPlane& cp) const {
@@ -51,8 +45,15 @@ public:
     }
 
     bool is_point_clipped(const Vec3d& point) const { return distance(point) < 0.; }
-    void set_normal(const Vec3d& normal) { for (size_t i=0; i<3; ++i) m_data[i] = normal(i); }
+    void set_normal(const Vec3d& normal)
+    {
+        const Vec3d norm_dir = normal.normalized();
+        m_data[0] = norm_dir.x();
+        m_data[1] = norm_dir.y();
+        m_data[2] = norm_dir.z();
+    }
     void set_offset(double offset) { m_data[3] = offset; }
+    double get_offset() const { return m_data[3]; }
     Vec3d get_normal() const { return Vec3d(m_data[0], m_data[1], m_data[2]); }
     bool is_active() const { return m_data[3] != DBL_MAX; }
     static ClippingPlane ClipsNothing() { return ClippingPlane(Vec3d(0., 0., 1.), DBL_MAX); }
@@ -74,10 +75,17 @@ public:
     // This is supposed to be in world coordinates.
     void set_plane(const ClippingPlane& plane);
 
+    // In case the object is clipped by two planes (e.g. in case of sinking
+    // objects), this will be used to clip the triagnulated cut.
+    // Pass ClippingPlane::ClipsNothing to turn this off.
+    void set_limiting_plane(const ClippingPlane& plane);
+
     // Which mesh to cut. MeshClipper remembers const * to it, caller
     // must make sure that it stays valid.
     void set_mesh(const TriangleMesh& mesh);
 
+    void set_negative_mesh(const TriangleMesh &mesh);
+
     // Inform the MeshClipper about the transformation that transforms the mesh
     // into world coordinates.
     void set_transformation(const Geometry::Transformation& trafo);
@@ -91,11 +99,12 @@ private:
 
     Geometry::Transformation m_trafo;
     const TriangleMesh* m_mesh = nullptr;
+    const TriangleMesh* m_negative_mesh = nullptr;
     ClippingPlane m_plane;
+    ClippingPlane m_limiting_plane = ClippingPlane::ClipsNothing();
     std::vector<Vec2f> m_triangles2d;
     GLIndexedVertexArray m_vertex_array;
     bool m_triangles_valid = false;
-    std::unique_ptr<TriangleMeshSlicer> m_tms;
 };
 
 
@@ -107,11 +116,9 @@ public:
     // The class references extern TriangleMesh, which must stay alive
     // during MeshRaycaster existence.
     MeshRaycaster(const TriangleMesh& mesh)
-        : m_emesh(mesh)
+        : m_emesh(mesh, true) // calculate epsilon for triangle-ray intersection from an average edge length
+        , m_normals(its_face_normals(mesh.its))
     {
-        m_normals.reserve(mesh.stl.facet_start.size());
-        for (const stl_facet& facet : mesh.stl.facet_start)
-            m_normals.push_back(facet.normal);
     }
 
     void line_from_mouse_pos(const Vec2d& mouse_pos, const Transform3d& trafo, const Camera& camera,
@@ -144,6 +151,9 @@ public:
 
     Vec3f get_closest_point(const Vec3f& point, Vec3f* normal = nullptr) const;
 
+    // Given a point in mesh coords, the method returns the closest facet from mesh.
+    int get_closest_facet(const Vec3f &point) const;
+
     Vec3f get_triangle_normal(size_t facet_idx) const;
 
 private:
