@@ -49,10 +49,11 @@ using t_option = std::unique_ptr<Option>;	//!
 
 /// Represents option lines
 class Line {
+	bool		m_is_separator{ false };
 public:
     wxString	label;
     wxString	label_tooltip;
-	wxString	label_path;
+	std::string	label_path;
 
     size_t		full_width {0}; 
 	wxColour*	full_Label_color {nullptr};
@@ -71,6 +72,9 @@ public:
     }
 	Line(wxString label, wxString tooltip) :
 		label(_(label)), label_tooltip(_(tooltip)) {}
+	Line() : m_is_separator(true) {}
+
+	bool is_separator() const { return m_is_separator; }
 
     const std::vector<widget_t>&	get_extra_widgets() const {return m_extra_widgets;}
     const std::vector<Option>&		get_options() const { return m_options; }
@@ -87,19 +91,20 @@ using t_optionfield_map = std::map<t_config_option_key, t_field>;
 using t_opt_map = std::map< std::string, std::pair<std::string, int> >;
 
 class OptionsGroup {
-	wxStaticBox*	stb;
+protected:
+	wxStaticBox*	stb {nullptr};
 public:
     const bool		staticbox {true};
     const wxString	title;
     size_t			label_width = 20 ;// {200};
     wxSizer*		sizer {nullptr};
 	OG_CustomCtrl*  custom_ctrl{ nullptr };
+	int				ctrl_horiz_alignment{ wxALIGN_LEFT};
     column_t		extra_column {nullptr};
     t_change		m_on_change { nullptr };
 	// To be called when the field loses focus, to assign a new initial value to the field.
 	// Used by the relative position / rotation / scale manipulation fields of the Object Manipulation UI.
     t_kill_focus    m_fill_empty_value { nullptr };
-    t_kill_focus    m_set_focus { nullptr };
 	std::function<DynamicPrintConfig()>	m_get_initial_config{ nullptr };
 	std::function<DynamicPrintConfig()>	m_get_sys_config{ nullptr };
 	std::function<bool()>	have_sys_config{ nullptr };
@@ -124,12 +129,13 @@ public:
 	void		activate_line(Line& line);
 
 	// create all controls for the option group from the m_lines
-	bool		activate(std::function<void()> throw_if_canceled = [](){});
+	bool		activate(std::function<void()> throw_if_canceled = [](){}, int horiz_alignment = wxALIGN_LEFT);
 	// delete all controls from the option group
 	void		clear(bool destroy_custom_ctrl = false);
 
-    Line		create_single_option_line(const Option& option, const wxString& path = wxEmptyString) const;
-    void		append_single_option_line(const Option& option, const wxString& path = wxEmptyString) { append_line(create_single_option_line(option, path)); }
+    Line		create_single_option_line(const Option& option, const std::string& path = std::string()) const;
+    void		append_single_option_line(const Option& option, const std::string& path = std::string()) { append_line(create_single_option_line(option, path)); }
+	void		append_separator();
 
     // return a non-owning pointer reference 
     inline Field*	get_field(const t_config_option_key& id) const{
@@ -171,6 +177,11 @@ public:
     wxGridSizer*        get_grid_sizer() { return m_grid_sizer; }
 	const std::vector<Line>& get_lines() { return m_lines; }
 	bool				is_legend_line();
+	// if we have to set the same control alignment for different option groups, 
+    // we have to set same max contrtol width to all of them
+	void				set_max_win_width(int max_win_width);
+
+	bool				is_activated() { return sizer != nullptr; }
 
 protected:
 	std::map<t_config_option_key, Option>	m_options;
@@ -207,10 +218,13 @@ protected:
 	const t_field&		build_field(const Option& opt);
 
     virtual void		on_kill_focus(const std::string& opt_key) {};
-	virtual void		on_set_focus(const std::string& opt_key);
 	virtual void		on_change_OG(const t_config_option_key& opt_id, const boost::any& value);
 	virtual void		back_to_initial_value(const std::string& opt_key) {}
 	virtual void		back_to_sys_value(const std::string& opt_key) {}
+
+public:
+	static wxString		get_url(const std::string& path_end);
+	static bool			launch_browser(const std::string& path_end);
 };
 
 class ConfigOptionsGroup: public OptionsGroup {
@@ -224,23 +238,24 @@ public:
 	ConfigOptionsGroup(	wxWindow* parent) :
 		OptionsGroup(parent, wxEmptyString, true, nullptr) {}
 
-	const std::string& config_category() const throw() { return m_config_category; }
+	const wxString& config_category() const throw() { return m_config_category; }
+	int config_type() const throw() { return m_config_type; }
 	const t_opt_map&   opt_map() const throw() { return m_opt_map; }
 
-	void 		set_config_category(const std::string &category) { this->m_config_category = category; }
+	void 		set_config_category_and_type(const wxString &category, int type) { m_config_category = category; m_config_type = type; }
     void        set_config(DynamicPrintConfig* config) { m_config = config; m_modelconfig = nullptr; }
 	Option		get_option(const std::string& opt_key, int opt_index = -1);
-	Line		create_single_option_line(const std::string& title, const wxString& path = wxEmptyString, int idx = -1) /*const*/{
+	Line		create_single_option_line(const std::string& title, const std::string& path = std::string(), int idx = -1) /*const*/{
 		Option option = get_option(title, idx);
 		return OptionsGroup::create_single_option_line(option, path);
 	}
-	Line		create_single_option_line(const Option& option, const wxString& path = wxEmptyString) const {
+	Line		create_single_option_line(const Option& option, const std::string& path = std::string()) const {
 		return OptionsGroup::create_single_option_line(option, path);
 	}
-	void		append_single_option_line(const Option& option, const wxString& path = wxEmptyString)	{
+	void		append_single_option_line(const Option& option, const std::string& path = std::string())	{
 		OptionsGroup::append_single_option_line(option, path);
 	}
-	void		append_single_option_line(const std::string title, const wxString& path = wxEmptyString, int idx = -1)
+	void		append_single_option_line(const std::string title, const std::string& path = std::string(), int idx = -1)
 	{
 		Option option = get_option(title, idx);
 		append_single_option_line(option, path);
@@ -273,9 +288,9 @@ private:
     const DynamicPrintConfig*	m_config {nullptr};
     // If the config is modelconfig, then ModelConfig::touch() has to be called after value change.
     ModelConfig*				m_modelconfig { nullptr };
-	bool						m_full_labels{ 0 };
 	t_opt_map					m_opt_map;
-    std::string             	m_config_category;
+    wxString                    m_config_category;
+    int                         m_config_type;
 
     // Change an option on m_config, possibly call ModelConfig::touch().
 	void 	change_opt_value(const t_config_option_key& opt_key, const boost::any& value, int opt_index = 0);
@@ -289,6 +304,9 @@ public:
 	~ogStaticText() {}
 
 	void		SetText(const wxString& value, bool wrap = true);
+	// Set special path end. It will be used to generation of the hyperlink on info page
+	void		SetPathEnd(const std::string& link);
+	void		FocusText(bool focus);
 };
 
 }}
