@@ -9,6 +9,7 @@
 
 #include "Selection.hpp"
 
+#include "libslic3r/enum_bitmask.hpp"
 #include "libslic3r/Preset.hpp"
 #include "libslic3r/BoundingBox.hpp"
 #include "libslic3r/GCode/GCodeProcessor.hpp"
@@ -22,17 +23,22 @@ class wxString;
 
 namespace Slic3r {
 
+class BuildVolume;
 class Model;
 class ModelObject;
+enum class ModelObjectCutAttribute : int;
+using ModelObjectCutAttributes = enum_bitmask<ModelObjectCutAttribute>;
 class ModelInstance;
 class Print;
 class SLAPrint;
 enum SLAPrintObjectStep : unsigned int;
+enum class ConversionType : int;
 
 using ModelInstancePtrs = std::vector<ModelInstance*>;
 
 namespace UndoRedo {
     class Stack;
+    enum class SnapshotType : unsigned char;
     struct Snapshot;
 }
 
@@ -48,7 +54,6 @@ class GLCanvas3D;
 class Mouse3DController;
 class NotificationManager;
 struct Camera;
-class Bed3D;
 class GLToolbar;
 class PlaterPresetComboBox;
 
@@ -79,6 +84,7 @@ public:
     void sys_color_changed();
     void search();
     void jump_to_option(size_t selected);
+    void jump_to_option(const std::string& opt_key, Preset::Type type, const std::wstring& category);
 
     ObjectManipulation*     obj_manipul();
     ObjectList*             obj_list();
@@ -108,6 +114,10 @@ public:
     void                    update_searcher();
     void                    update_ui_from_settings();
 
+#ifdef _MSW_DARK_MODE
+    void                    show_mode_sizer(bool show);
+#endif
+
     std::vector<PlaterPresetComboBox*>&   combos_filament();
     Search::OptionsSearcher&        get_searcher();
     std::string&                    get_search_line();
@@ -127,9 +137,20 @@ public:
     Plater(const Plater &) = delete;
     Plater &operator=(Plater &&) = delete;
     Plater &operator=(const Plater &) = delete;
-    ~Plater();
+    ~Plater() = default;
+
+    bool is_project_dirty() const;
+    bool is_presets_dirty() const;
+    void update_project_dirty_from_presets();
+    int  save_project_if_dirty(const wxString& reason);
+    void reset_project_dirty_after_save();
+    void reset_project_dirty_initial_presets();
+#if ENABLE_PROJECT_DIRTY_STATE_DEBUG_WINDOW
+    void render_project_state_debug_window() const;
+#endif // ENABLE_PROJECT_DIRTY_STATE_DEBUG_WINDOW
 
     Sidebar& sidebar();
+    const Model& model() const;
     Model& model();
     const Print& fff_print() const;
     Print& fff_print();
@@ -150,15 +171,14 @@ public:
     std::vector<size_t> load_files(const std::vector<boost::filesystem::path>& input_files, bool load_model = true, bool load_config = true, bool imperial_units = false);
     // To be called when providing a list of files to the GUI slic3r on command line.
     std::vector<size_t> load_files(const std::vector<std::string>& input_files, bool load_model = true, bool load_config = true, bool imperial_units = false);
-#if ENABLE_DRAG_AND_DROP_FIX
     // to be called on drag and drop
     bool load_files(const wxArrayString& filenames);
-#endif // ENABLE_DRAG_AND_DROP_FIX
 
     const wxString& get_last_loaded_gcode() const { return m_last_loaded_gcode; }
 
     void update();
     void stop_jobs();
+    bool is_any_job_running() const;
     void select_view(const std::string& direction);
     void select_view_3D(const std::string& name);
 
@@ -176,7 +196,7 @@ public:
 
     // Called after the Preferences dialog is closed and the program settings are saved.
     // Update the UI based on the current preferences.
-    void update_ui_from_settings(bool apply_free_camera_correction = true);
+    void update_ui_from_settings();
 
     void select_all();
     void deselect_all();
@@ -191,15 +211,17 @@ public:
     void fill_bed_with_instances();
     bool is_selection_empty() const;
     void scale_selection_to_fit_print_volume();
-    void convert_unit(bool from_imperial_unit);
+    void convert_unit(ConversionType conv_type);
+    void toggle_layers_editing(bool enable);
 
-    void cut(size_t obj_idx, size_t instance_idx, coordf_t z, bool keep_upper = true, bool keep_lower = true, bool rotate_lower = false);
+    void cut(size_t obj_idx, size_t instance_idx, coordf_t z, ModelObjectCutAttributes attributes);
 
     void export_gcode(bool prefer_removable);
     void export_stl(bool extended = false, bool selection_only = false);
     void export_amf();
-    void export_3mf(const boost::filesystem::path& output_path = boost::filesystem::path());
+    bool export_3mf(const boost::filesystem::path& output_path = boost::filesystem::path());
     void reload_from_disk();
+    void replace_with_stl();
     void reload_all_from_disk();
     bool has_toolpaths_to_export() const;
     void export_toolpaths_to_obj() const;
@@ -207,17 +229,23 @@ public:
     void reslice_SLA_supports(const ModelObject &object, bool postpone_error_messages = false);
     void reslice_SLA_hollowing(const ModelObject &object, bool postpone_error_messages = false);
     void reslice_SLA_until_step(SLAPrintObjectStep step, const ModelObject &object, bool postpone_error_messages = false);
+
+    void clear_before_change_mesh(int obj_idx);
+    void changed_mesh(int obj_idx);
+
     void changed_object(int obj_idx);
     void changed_objects(const std::vector<size_t>& object_idxs);
     void schedule_background_process(bool schedule = true);
     bool is_background_process_update_scheduled() const;
     void suppress_background_process(const bool stop_background_process) ;
-    void fix_through_netfabb(const int obj_idx, const int vol_idx = -1);
     void send_gcode();
 	void eject_drive();
 
     void take_snapshot(const std::string &snapshot_name);
     void take_snapshot(const wxString &snapshot_name);
+    void take_snapshot(const std::string &snapshot_name, UndoRedo::SnapshotType snapshot_type);
+    void take_snapshot(const wxString &snapshot_name, UndoRedo::SnapshotType snapshot_type);
+
     void undo();
     void redo();
     void undo_to(int selection);
@@ -233,15 +261,16 @@ public:
     void leave_gizmos_stack();
 
     void on_extruders_change(size_t extruders_count);
+    bool update_filament_colors_in_full_config();
     void on_config_change(const DynamicPrintConfig &config);
     void force_filament_colors_update();
     void force_print_bed_update();
     // On activating the parent window.
     void on_activate();
-    std::vector<std::string> get_extruder_colors_from_plater_config(const GCodeProcessor::Result* const result = nullptr) const;
-    std::vector<std::string> get_colors_for_color_print(const GCodeProcessor::Result* const result = nullptr) const;
+    std::vector<std::string> get_extruder_colors_from_plater_config(const GCodeProcessorResult* const result = nullptr) const;
+    std::vector<std::string> get_colors_for_color_print(const GCodeProcessorResult* const result = nullptr) const;
 
-    void update_object_menu();
+    void update_menus();
     void show_action_buttons(const bool is_ready_to_slice) const;
 
     wxString get_project_filename(const wxString& extension = wxEmptyString) const;
@@ -255,10 +284,9 @@ public:
     GLCanvas3D* canvas3D();
     const GLCanvas3D * canvas3D() const;
     GLCanvas3D* get_current_canvas3D();
-    BoundingBoxf bed_shape_bb() const;
     
     void arrange();
-    void find_new_position(const ModelInstancePtrs  &instances, coord_t min_d);
+    void find_new_position(const ModelInstancePtrs  &instances);
 
     void set_current_canvas_as_dirty();
     void unbind_canvas_event_handlers();
@@ -271,6 +299,10 @@ public:
     void copy_selection_to_clipboard();
     void paste_from_clipboard();
     void search(bool plater_is_active);
+    void mirror(Axis axis);
+    void split_object();
+    void split_volume();
+    void optimize_rotation();
 
     bool can_delete() const;
     bool can_delete_all() const;
@@ -278,6 +310,7 @@ public:
     bool can_decrease_instances() const;
     bool can_set_instance_to_object() const;
     bool can_fix_through_netfabb() const;
+    bool can_simplify() const;
     bool can_split_to_objects() const;
     bool can_split_to_volumes() const;
     bool can_arrange() const;
@@ -287,6 +320,12 @@ public:
     bool can_undo() const;
     bool can_redo() const;
     bool can_reload_from_disk() const;
+    bool can_replace_with_stl() const;
+    bool can_mirror() const;
+    bool can_split(bool to_objects) const;
+#if ENABLE_ENHANCED_PRINT_VOLUME_FIT
+    bool can_scale_to_print_volume() const;
+#endif // ENABLE_ENHANCED_PRINT_VOLUME_FIT
 
     void msw_rescale();
     void sys_color_changed();
@@ -304,8 +343,7 @@ public:
     unsigned int get_environment_texture_id() const;
 #endif // ENABLE_ENVIRONMENT_MAP
 
-    const Bed3D& get_bed() const;
-    Bed3D& get_bed();
+    const BuildVolume& build_volume() const;
 
     const GLToolbar& get_view_toolbar() const;
     GLToolbar& get_view_toolbar();
@@ -324,13 +362,15 @@ public:
     Mouse3DController& get_mouse3d_controller();
 
 	void set_bed_shape() const;
-    void set_bed_shape(const Pointfs& shape, const std::string& custom_texture, const std::string& custom_model, bool force_as_custom = false) const;
+    void set_bed_shape(const Pointfs& shape, const double max_print_height, const std::string& custom_texture, const std::string& custom_model, bool force_as_custom = false) const;
 
-	const NotificationManager* get_notification_manager() const;
-	NotificationManager* get_notification_manager();
+    NotificationManager * get_notification_manager();
+    const NotificationManager * get_notification_manager() const;
+
+    void init_notification_manager();
 
     void bring_instance_forward();
-
+    
     // ROII wrapper for suppressing the Undo / Redo snapshot to be taken.
 	class SuppressSnapshots
 	{
@@ -347,15 +387,23 @@ public:
 		Plater *m_plater;
 	};
 
-	// ROII wrapper for taking an Undo / Redo snapshot while disabling the snapshot taking by the methods called from inside this snapshot.
+    // RAII wrapper for taking an Undo / Redo snapshot while disabling the snapshot taking by the methods called from inside this snapshot.
 	class TakeSnapshot
 	{
 	public:
+        TakeSnapshot(Plater *plater, const std::string &snapshot_name);
 		TakeSnapshot(Plater *plater, const wxString &snapshot_name) : m_plater(plater)
 		{
 			m_plater->take_snapshot(snapshot_name);
 			m_plater->suppress_snapshots();
 		}
+        TakeSnapshot(Plater* plater, const std::string& snapshot_name, UndoRedo::SnapshotType snapshot_type);
+        TakeSnapshot(Plater *plater, const wxString &snapshot_name, UndoRedo::SnapshotType snapshot_type) : m_plater(plater)
+        {
+            m_plater->take_snapshot(snapshot_name, snapshot_type);
+            m_plater->suppress_snapshots();
+        }
+
 		~TakeSnapshot()
 		{
 			m_plater->allow_snapshots();
@@ -366,15 +414,26 @@ public:
 
     bool inside_snapshot_capture();
 
-#if ENABLE_RENDER_STATISTICS
     void toggle_render_statistic_dialog();
     bool is_render_statistic_dialog_visible() const;
-#endif // ENABLE_RENDER_STATISTICS
 
 	// Wrapper around wxWindow::PopupMenu to suppress error messages popping out while tracking the popup menu.
 	bool PopupMenu(wxMenu *menu, const wxPoint& pos = wxDefaultPosition);
     bool PopupMenu(wxMenu *menu, int x, int y) { return this->PopupMenu(menu, wxPoint(x, y)); }
 
+    // get same Plater/ObjectList menus
+    wxMenu* object_menu();
+    wxMenu* part_menu();
+    wxMenu* sla_object_menu();
+    wxMenu* default_menu();
+    wxMenu* instance_menu();
+    wxMenu* layer_menu();
+    wxMenu* multi_selection_menu();
+
+    static bool has_illegal_filename_characters(const wxString& name);
+    static bool has_illegal_filename_characters(const std::string& name);
+    static void show_illegal_characters_warning(wxWindow* parent);
+
 private:
     struct priv;
     std::unique_ptr<priv> p;
